@@ -74,7 +74,20 @@ upgrade_project ironic $RUN_DIR $BASE_DEVSTACK_BRANCH $TARGET_DEVSTACK_BRANCH
 # NOTE(rloo): make sure it is OK to do an upgrade. Except that we aren't
 # parsing/checking the output of this command because the output could change
 # based on the checks it makes.
-$IRONIC_BIN_DIR/ironic-status upgrade check
+$IRONIC_BIN_DIR/ironic-status upgrade check && ret_val=$? || ret_val=$?
+if [ $ret_val -gt 1 ] ; then
+    # NOTE(TheJulia): We need to evaluate the return code from the
+    # upgrade status check as the framework defines
+    # Warnings are permissible and returned as status code 1, errors are
+    # returned as greater than 1 which means there is a major upgrade
+    # stopping issue which needs to be addressed.
+    echo "WARNING: Status check failed, we're going to attempt to apply the schema update and then re-evaluate."
+    $IRONIC_BIN_DIR/ironic-dbsync --config-file=$IRONIC_CONF_FILE upgrade
+    $IRONIC_BIN_DIR/ironic-status upgrade check && ret_val=$? || ret_val=$?
+    if [ $ret_val -gt 1 ] ; then
+        die $LINENO "Ironic DB Status check failed, returned: $ret_val"
+    fi
+fi
 
 $IRONIC_BIN_DIR/ironic-dbsync --config-file=$IRONIC_CONF_FILE
 
