@@ -14,9 +14,7 @@
 #    under the License.
 
 from ironic import api
-from ironic.api.controllers import base
 from ironic.api.controllers import link
-from ironic.api import types as atypes
 
 
 def has_next(collection, limit):
@@ -24,7 +22,57 @@ def has_next(collection, limit):
     return len(collection) and len(collection) == limit
 
 
-def get_next(collection, limit, url=None, key_field='uuid', **kwargs):
+def list_convert_with_links(items, item_name, limit, url, fields=None,
+                            sanitize_func=None, key_field='uuid',
+                            sanitizer_args=None, **kwargs):
+    """Build a collection dict including the next link for paging support.
+
+    :param items:
+        List of unsanitized items to include in the collection
+    :param item_name:
+        Name of dict key for items value
+    :param limit:
+        Paging limit
+    :param url:
+        Base URL for building next link
+    :param fields:
+        Optional fields to use for sanitize function
+    :param sanitize_func:
+        Optional sanitize function run on each item, item changes will be
+        done in-place
+    :param key_field:
+        Key name for building next URL
+    :param sanitizer_args:
+        Dictionary with additional arguments to be passed to the sanitizer.
+    :param kwargs:
+        other arguments passed to ``get_next``
+    :returns:
+        A dict containing ``item_name`` and ``next`` values
+    """
+    assert url, "BUG: collections require a base URL"
+    assert limit is None or isinstance(limit, int), \
+        f"BUG: limit must be None or int, got {type(limit)}"
+
+    items_dict = {
+        item_name: items
+    }
+    next_uuid = get_next(
+        items, limit, url=url, fields=fields, key_field=key_field, **kwargs)
+    if next_uuid:
+        items_dict['next'] = next_uuid
+
+    if sanitize_func:
+        if sanitizer_args:
+            for item in items:
+                sanitize_func(item, fields, **sanitizer_args)
+        else:
+            for item in items:
+                sanitize_func(item, fields=fields)
+
+    return items_dict
+
+
+def get_next(collection, limit, url, key_field='uuid', **kwargs):
     """Return a link to the next subset of the collection."""
     if not has_next(collection, limit):
         return None
@@ -49,30 +97,3 @@ def get_next(collection, limit, url=None, key_field='uuid', **kwargs):
 
     return link.make_link('next', api.request.public_url,
                           url, next_args)['href']
-
-
-class Collection(base.Base):
-
-    next = str
-    """A link to retrieve the next subset of the collection"""
-
-    @property
-    def collection(self):
-        return getattr(self, self._type)
-
-    @classmethod
-    def get_key_field(cls):
-        return 'uuid'
-
-    def has_next(self, limit):
-        """Return whether collection has more items."""
-        return has_next(self.collection, limit)
-
-    def get_next(self, limit, url=None, **kwargs):
-        """Return a link to the next subset of the collection."""
-        resource_url = url or self._type
-        the_next = get_next(self.collection, limit, url=resource_url,
-                            key_field=self.get_key_field(), **kwargs)
-        if the_next is None:
-            return atypes.Unset
-        return the_next
