@@ -21,16 +21,9 @@ import sys
 
 from oslo_config import cfg
 from oslo_log import log
-try:
-    from oslo_reports import guru_meditation_report as gmr
-    from oslo_reports import opts as gmr_opts
-except ImportError:
-    gmr = None
 
-from ironic.common import profiler
 from ironic.common import service as ironic_service
 from ironic.common import wsgi_service
-from ironic import version
 
 CONF = cfg.CONF
 
@@ -39,22 +32,14 @@ LOG = log.getLogger(__name__)
 
 def main():
     # Parse config file and command line options, then start logging
-    ironic_service.prepare_service(sys.argv)
-
-    if gmr is not None:
-        gmr_opts.set_defaults(CONF)
-        gmr.TextGuruMeditation.setup_autorun(version)
-    else:
-        LOG.debug('Guru meditation reporting is disabled '
-                  'because oslo.reports is not installed')
-
-    profiler.setup('ironic_api', CONF.host)
+    ironic_service.prepare_service('ironic_api', sys.argv)
+    ironic_service.ensure_rpc_transport()
 
     # Build and start the WSGI app
     launcher = ironic_service.process_launcher()
     server = wsgi_service.WSGIService('ironic_api', CONF.api.enable_ssl_api)
     launcher.launch_service(server, workers=server.workers)
-    launcher.wait()
+    sys.exit(launcher.wait())
 
 
 if __name__ == '__main__':
