@@ -819,8 +819,8 @@ class AgentInProgress(IronicException):
                  'presently executing a command. Error %(error)s')
 
 
-class InsufficentMemory(IronicException):
-    _msg_fmt = _("Available memory at %(free)s, Insufficent as %(required)s "
+class InsufficientMemory(IronicException):
+    _msg_fmt = _("Available memory at %(free)s, Insufficient as %(required)s "
                  "is required to proceed at this time.")
 
 
@@ -835,3 +835,29 @@ class IncorrectConfiguration(IronicException):
 
 class NodeVerifyFailure(IronicException):
     _msg_fmt = _("Failed to verify node %(node)s: %(reason)s")
+
+
+class ImageRefIsARedirect(IronicException):
+    _msg_fmt = _("Received a URL redirect when attempting to evaluate "
+                 "image reference %(image_ref)s pointing to "
+                 "%(redirect_url)s. This may, or may not be recoverable.")
+    redirect_url = None
+
+    def __init__(self, image_ref=None, redirect_url=None, msg=None):
+        self.redirect_url = redirect_url
+        # Kwargs are expected by ironic_lib's IronicException to convert
+        # the message.
+        super(ImageRefIsARedirect, self).__init__(
+            message=msg,
+            image_ref=image_ref,
+            redirect_url=redirect_url)
+
+
+class ConcurrentActionLimit(IronicException):
+    # NOTE(TheJulia): We explicitly don't report the concurrent
+    # action limit configuration value as a security guard since
+    # if informed of the limit, an attacker can tailor their attack.
+    _msg_fmt = _("Unable to process request at this time. "
+                 "The concurrent action limit for %(task_type)s "
+                 "has been reached. Please contact your administrator "
+                 "and try again later.")
