@@ -14,9 +14,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
 import os
+import shutil
+import tempfile
+from urllib import parse as urlparse
 
 from ironic_lib import utils as ironic_utils
+import jinja2
+from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import fileutils
@@ -27,12 +33,15 @@ from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
 from ironic.common import image_service as service
 from ironic.common import images
+from ironic.common import kickstart_utils as ks_utils
 from ironic.common import states
 from ironic.common import utils
+from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import image_cache
+from ironic.drivers import utils as driver_utils
 from ironic import objects
 
 LOG = logging.getLogger(__name__)
@@ -50,6 +59,7 @@ DHCPV6_BOOTFILE_NAME = '59'  # rfc5970
 DHCP_TFTP_SERVER_ADDRESS = '150'  # rfc5859
 DHCP_IPXE_ENCAP_OPTS = '175'  # Tentatively Assigned
 DHCP_TFTP_PATH_PREFIX = '210'  # rfc5071
+DHCP_SERVER_IP_ADDRESS = '255'  # dnsmasq server-ip-address
 
 DEPLOY_KERNEL_RAMDISK_LABELS = ['deploy_kernel', 'deploy_ramdisk']
 RESCUE_KERNEL_RAMDISK_LABELS = ['rescue_kernel', 'rescue_ramdisk']
@@ -57,13 +67,16 @@ KERNEL_RAMDISK_LABELS = {'deploy': DEPLOY_KERNEL_RAMDISK_LABELS,
                          'rescue': RESCUE_KERNEL_RAMDISK_LABELS}
 
 
-def get_root_dir():
-    """Returns the directory where the config files and images will live."""
-    return CONF.pxe.tftp_root
+def _get_root_dir(ipxe_enabled):
+    if ipxe_enabled:
+        return CONF.deploy.http_root
+    else:
+        return CONF.pxe.tftp_root
 
 
-def get_ipxe_root_dir():
-    return CONF.deploy.http_root
+def ensure_tree(path):
+    mode = CONF.pxe.dir_permission or 0o755
+    os.makedirs(path, mode=mode, exist_ok=True)
 
 
 def _ensure_config_dirs_exist(task, ipxe_enabled=False):
@@ -73,10 +86,7 @@ def _ensure_config_dirs_exist(task, ipxe_enabled=False):
     :param ipxe_enabled: Default false boolean to indicate if ipxe
                          is in use by the caller.
     """
-    if ipxe_enabled:
-        root_dir = get_ipxe_root_dir()
-    else:
-        root_dir = get_root_dir()
+    root_dir = _get_root_dir(ipxe_enabled)
     node_dir = os.path.join(root_dir, task.node.uuid)
     pxe_dir = os.path.join(root_dir, PXE_CFG_DIR_NAME)
     # NOTE: We should only change the permissions if the folder
@@ -85,9 +95,7 @@ def _ensure_config_dirs_exist(task, ipxe_enabled=False):
     # which may not recurse downward.
     for directory in (node_dir, pxe_dir):
         if not os.path.isdir(directory):
-            fileutils.ensure_tree(directory)
-            if CONF.pxe.dir_permission:
-                os.chmod(directory, CONF.pxe.dir_permission)
+            ensure_tree(directory)
 
 
 def _link_mac_pxe_configs(task, ipxe_enabled=False):
@@ -112,8 +120,9 @@ def _link_mac_pxe_configs(task, ipxe_enabled=False):
         create_link(_get_pxe_mac_path(port.address, client_id=client_id,
                                       ipxe_enabled=ipxe_enabled))
         # Grub2 MAC address only
-        create_link(_get_pxe_grub_mac_path(port.address,
-                                           ipxe_enabled=ipxe_enabled))
+        for path in _get_pxe_grub_mac_path(port.address,
+                                           ipxe_enabled=ipxe_enabled):
+            create_link(path)
 
 
 def _link_ip_address_pxe_configs(task, ipxe_enabled=False):
@@ -155,8 +164,10 @@ def _link_ip_address_pxe_configs(task, ipxe_enabled=False):
 
 
 def _get_pxe_grub_mac_path(mac, ipxe_enabled=False):
-    root_dir = get_ipxe_root_dir() if ipxe_enabled else get_root_dir()
-    return os.path.join(root_dir, mac + '.conf')
+    root_dir = _get_root_dir(ipxe_enabled)
+    yield os.path.join(root_dir, "%s-%s-%s" %
+                       ("grub.cfg", "01", mac.replace(':', "-").lower()))
+    yield os.path.join(root_dir, mac + '.conf')
 
 
 def _get_pxe_mac_path(mac, delimiter='-', client_id=None,
@@ -178,9 +189,9 @@ def _get_pxe_mac_path(mac, delimiter='-', client_id=None,
         if client_id:
             hw_type = '20-'
         mac_file_name = hw_type + mac_file_name
-        return os.path.join(get_root_dir(), PXE_CFG_DIR_NAME,
+        return os.path.join(CONF.pxe.tftp_root, PXE_CFG_DIR_NAME,
                             mac_file_name)
-    return os.path.join(get_ipxe_root_dir(), PXE_CFG_DIR_NAME,
+    return os.path.join(CONF.deploy.http_root, PXE_CFG_DIR_NAME,
                         mac_file_name)
 
 
@@ -215,17 +226,17 @@ def get_kernel_ramdisk_info(node_uuid, driver_info, mode='deploy',
 
     Note: driver_info should be validated outside of this method.
     """
-    if ipxe_enabled:
-        root_dir = get_ipxe_root_dir()
-    else:
-        root_dir = get_root_dir()
+    root_dir = _get_root_dir(ipxe_enabled)
     image_info = {}
     labels = KERNEL_RAMDISK_LABELS[mode]
     for label in labels:
-        image_info[label] = (
-            str(driver_info[label]),
-            os.path.join(root_dir, node_uuid, label)
-        )
+        try:
+            image_info[label] = (
+                str(driver_info[label]),
+                os.path.join(root_dir, node_uuid, label)
+            )
+        except KeyError:
+            pass  # may be absent in rare cases, verified in parse_driver_info
     return image_info
 
 
@@ -238,10 +249,56 @@ def get_pxe_config_file_path(node_uuid, ipxe_enabled=False):
     :returns: The path to the node's PXE configuration file.
 
     """
-    if ipxe_enabled:
-        return os.path.join(get_ipxe_root_dir(), node_uuid, 'config')
+    return os.path.join(_get_root_dir(ipxe_enabled), node_uuid, 'config')
+
+
+def get_file_path_from_label(node_uuid, root_dir, label):
+    """Generate absolute paths to various images from their name(label)
+
+    This method generates absolute file system path on the conductor where
+    various images need to be placed. For example the kickstart template, file
+    and stage2 squashfs.img needs to be placed in the ipxe_root_dir since they
+    will be transferred by anaconda ramdisk over http(s). The generated paths
+    will be added to the image_info dictionary as values.
+
+    :param node_uuid: the UUID of the node
+    :param root_dir: Directory in which the image must be placed
+    :param label: Name of the image
+    """
+    if label == 'ks_template':
+        return os.path.join(CONF.deploy.http_root, node_uuid,
+                            'ks.cfg.template')
+    elif label == 'ks_cfg':
+        return os.path.join(CONF.deploy.http_root, node_uuid, 'ks.cfg')
+    elif label == 'stage2':
+        return os.path.join(CONF.deploy.http_root, node_uuid, 'LiveOS',
+                            'squashfs.img')
     else:
-        return os.path.join(get_root_dir(), node_uuid, 'config')
+        return os.path.join(root_dir, node_uuid, label)
+
+
+def get_http_url_path_from_label(http_url, node_uuid, label):
+    """Generate http url path to various image artifacts
+
+    This method generates http(s) urls for various image artifacts int the
+    webserver root. The generated urls will be added to the pxe_options dict
+    and used to render pxe/ipxe configuration templates.
+
+    :param http_url: URL to access the root of the webserver
+    :param node_uuid: the UUID of the node
+    :param label: Name of the image
+    """
+    if label == 'ks_template':
+        return '/'.join([http_url, node_uuid, 'ks.cfg.template'])
+    elif label == 'ks_cfg':
+        return '/'.join([http_url, node_uuid, 'ks.cfg'])
+    elif label == 'stage2':
+        # we store stage2 in http_root/node_uuid/LiveOS/squashfs.img
+        # Specifying http://host/node_uuid as stage2 url will make anaconda
+        # automatically load the squashfs.img from LiveOS directory.
+        return '/'.join([http_url, node_uuid])
+    else:
+        return '/'.join([http_url, node_uuid, label])
 
 
 def create_pxe_config(task, pxe_options, template=None, ipxe_enabled=False):
@@ -297,7 +354,8 @@ def create_pxe_config(task, pxe_options, template=None, ipxe_enabled=False):
               'DISK_IDENTIFIER': pxe_config_disk_ident}
 
     pxe_config = utils.render_template(template, params)
-    utils.write_to_file(pxe_config_file_path, pxe_config)
+    utils.write_to_file(pxe_config_file_path, pxe_config,
+                        CONF.pxe.file_permission)
 
     # Always write the mac addresses
     _link_mac_pxe_configs(task, ipxe_enabled=ipxe_enabled)
@@ -320,7 +378,8 @@ def create_ipxe_boot_script():
     """Render the iPXE boot script into the HTTP root directory"""
     boot_script = utils.render_template(
         CONF.pxe.ipxe_boot_script,
-        {'ipxe_for_mac_uri': PXE_CFG_DIR_NAME + '/'})
+        {'ipxe_for_mac_uri': PXE_CFG_DIR_NAME + '/',
+         'ipxe_fallback_script': CONF.pxe.ipxe_fallback_script})
     bootfile_path = os.path.join(
         CONF.deploy.http_root,
         os.path.basename(CONF.pxe.ipxe_boot_script))
@@ -329,7 +388,8 @@ def create_ipxe_boot_script():
     # which should be rather rare
     if (not os.path.isfile(bootfile_path)
             or not utils.file_has_content(bootfile_path, boot_script)):
-        utils.write_to_file(bootfile_path, boot_script)
+        utils.write_to_file(bootfile_path, boot_script,
+                            CONF.pxe.file_permission)
 
 
 def clean_up_pxe_config(task, ipxe_enabled=False):
@@ -364,14 +424,11 @@ def clean_up_pxe_config(task, ipxe_enabled=False):
             _get_pxe_mac_path(port.address, client_id=client_id,
                               ipxe_enabled=ipxe_enabled))
         # Grub2 MAC address based confiuration
-        ironic_utils.unlink_without_raise(
-            _get_pxe_grub_mac_path(port.address, ipxe_enabled=ipxe_enabled))
-    if ipxe_enabled:
-        utils.rmtree_without_raise(os.path.join(get_ipxe_root_dir(),
-                                                task.node.uuid))
-    else:
-        utils.rmtree_without_raise(os.path.join(get_root_dir(),
-                                                task.node.uuid))
+        for path in _get_pxe_grub_mac_path(port.address,
+                                           ipxe_enabled=ipxe_enabled):
+            ironic_utils.unlink_without_raise(path)
+    utils.rmtree_without_raise(os.path.join(_get_root_dir(ipxe_enabled),
+                                            task.node.uuid))
 
 
 def _dhcp_option_file_or_url(task, urlboot=False, ip_version=None):
@@ -432,7 +489,7 @@ def dhcp_options_for_instance(task, ipxe_enabled=False, url_boot=False,
     else:
         use_ip_version = int(CONF.pxe.ip_version)
     dhcp_opts = []
-    dhcp_provider_name = CONF.dhcp.dhcp_provider
+    api = dhcp_factory.DHCPFactory().provider
     if use_ip_version == 4:
         boot_file_param = DHCP_BOOTFILE_NAME
     else:
@@ -461,7 +518,7 @@ def dhcp_options_for_instance(task, ipxe_enabled=False, url_boot=False,
         ipxe_script_url = '/'.join([CONF.deploy.http_url, script_name])
         # if the request comes from dumb firmware send them the iPXE
         # boot image.
-        if dhcp_provider_name == 'neutron':
+        if api.supports_ipxe_tag():
             # Neutron use dnsmasq as default DHCP agent. Neutron carries the
             # configuration to relate to the tags below. The ipxe6 tag was
             # added in the Stein cycle which identifies the iPXE User-Class
@@ -507,9 +564,11 @@ def dhcp_options_for_instance(task, ipxe_enabled=False, url_boot=False,
         # from it's own path, but Petitboot needs it to be specified by this
         # option since it doesn't use pxelinux.0 loader.
         if not url_boot:
+            # Enforce trailing slash
+            prefix = os.path.join(CONF.pxe.tftp_root, '')
             dhcp_opts.append(
                 {'opt_name': DHCP_TFTP_PATH_PREFIX,
-                 'opt_value': get_tftp_path_prefix()})
+                 'opt_value': prefix})
 
     if not url_boot:
         dhcp_opts.append({'opt_name': DHCP_TFTP_SERVER_NAME,
@@ -530,7 +589,7 @@ def dhcp_options_for_instance(task, ipxe_enabled=False, url_boot=False,
     # Related bug was opened on Neutron side:
     # https://bugs.launchpad.net/neutron/+bug/1723354
     if not url_boot:
-        dhcp_opts.append({'opt_name': 'server-ip-address',
+        dhcp_opts.append({'opt_name': DHCP_SERVER_IP_ADDRESS,
                           'opt_value': CONF.pxe.tftp_server})
 
     # Append the IP version for all the configuration options
@@ -540,29 +599,11 @@ def dhcp_options_for_instance(task, ipxe_enabled=False, url_boot=False,
     return dhcp_opts
 
 
-def get_tftp_path_prefix():
-    """Adds trailing slash (if needed) necessary for path-prefix
-
-    :return: CONF.pxe.tftp_root ensured to have a trailing slash
-    """
-    return os.path.join(CONF.pxe.tftp_root, '')
-
-
-def get_path_relative_to_tftp_root(file_path):
-    """Return file relative path to CONF.pxe.tftp_root
-
-    :param file_path: full file path to be made relative path.
-    :returns: The path relative to CONF.pxe.tftp_root
-    """
-    return os.path.relpath(file_path, get_tftp_path_prefix())
-
-
 def is_ipxe_enabled(task):
     """Return true if ipxe is set.
 
     :param task: A TaskManager object
-    :returns: boolean true if ``[pxe]ipxe_enabled`` is configured
-              or if the task driver instance is the iPXE driver.
+    :returns: boolean true if the task driver instance is the iPXE driver.
     """
     return 'ipxe_boot' in task.driver.boot.capabilities
 
@@ -582,6 +623,11 @@ def parse_driver_info(node, mode='deploy'):
     :returns: A dict with the driver_info values.
     :raises: MissingParameterValue
     """
+    if not deploy_utils.needs_agent_ramdisk(node, mode=mode):
+        # Ramdisk deploy does not need an agent, nor does it support any other,
+        # options. Skipping.
+        return {}
+
     info = node.driver_info
 
     params_to_check = KERNEL_RAMDISK_LABELS[mode]
@@ -621,10 +667,7 @@ def get_instance_image_info(task, ipxe_enabled=False):
     if (node.driver_internal_info.get('is_whole_disk_image')
             or deploy_utils.get_boot_option(node) == 'local'):
         return image_info
-    if ipxe_enabled:
-        root_dir = get_ipxe_root_dir()
-    else:
-        root_dir = get_root_dir()
+    root_dir = _get_root_dir(ipxe_enabled)
     i_info = node.instance_info
     if i_info.get('boot_iso'):
         image_info['boot_iso'] = (
@@ -632,20 +675,102 @@ def get_instance_image_info(task, ipxe_enabled=False):
             os.path.join(root_dir, node.uuid, 'boot_iso'))
 
         return image_info
+    image_properties = None
+    d_info = deploy_utils.get_image_instance_info(node)
+    isap = node.driver_internal_info.get('is_source_a_path')
+
+    def _get_image_properties():
+        nonlocal image_properties, isap
+        if not image_properties and not isap:
+            i_service = service.get_image_service(
+                d_info['image_source'],
+                context=ctx)
+            image_properties = i_service.show(
+                d_info['image_source'])['properties']
+        # TODO(TheJulia): At some point, we should teach this code
+        # to understand that with a path, it *can* retrieve the
+        # manifest from the HTTP(S) endpoint, which can populate
+        # image_properties, and drive path to variable population
+        # like is done with basically Glance.
 
     labels = ('kernel', 'ramdisk')
-    d_info = deploy_utils.get_image_instance_info(node)
+    if not isap:
+        anaconda_labels = ('stage2', 'ks_template', 'ks_cfg')
+    else:
+        # When a path is used, a stage2 ramdisk can be determiend
+        # automatically by anaconda, so it is not an explicit
+        # requirement.
+        anaconda_labels = ('ks_template', 'ks_cfg')
+
     if not (i_info.get('kernel') and i_info.get('ramdisk')):
-        glance_service = service.GlanceImageService(context=ctx)
-        iproperties = glance_service.show(d_info['image_source'])['properties']
-        for label in labels:
-            i_info[label] = str(iproperties[label + '_id'])
-        node.instance_info = i_info
-        node.save()
-    for label in labels:
+        # NOTE(rloo): If both are not specified in instance_info
+        # we won't use any of them. We'll use the values specified
+        # with the image, which we assume have been set.
+        _get_image_properties()
+        if image_properties:
+            # This is intended for Glance usage, but all image properties
+            # should be routed through the image service request routing.
+            for label in labels:
+                i_info[label] = str(image_properties[label + '_id'])
+            node.instance_info = i_info
+            node.save()
+        # TODO(TheJulia): Add functionality to look/grab the hints file
+        # for anaconda and just run with the entire path.
+
+        # stage2: installer stage2 squashfs image
+        # ks_template: anaconda kickstart template
+        # ks_cfg - rendered ks_template
+
+        # NOTE(rloo): We save stage2 & ks_template values in case they
+        # are changed by the user after we start using them and to
+        # prevent re-computing them again.
+        if not node.driver_internal_info.get('stage2'):
+            if i_info.get('stage2'):
+                node.set_driver_internal_info('stage2', i_info['stage2'])
+            elif not isap:
+                # If the source is not a path, then we need a stage2 ramdisk.
+                _get_image_properties()
+                if 'stage2_id' not in image_properties:
+                    msg = (_("'stage2_id' is missing from the properties of "
+                             "the OS image %s. The anaconda deploy interface "
+                             "requires this to be set with the OS image or "
+                             "in instance_info['stage2']. ") %
+                           d_info['image_source'])
+                    raise exception.ImageUnacceptable(msg)
+                else:
+                    node.set_driver_internal_info(
+                        'stage2', str(image_properties['stage2_id']))
+    # NOTE(TheJulia): A kickstart template is entirely independent
+    # of the stage2 ramdisk. In the end, it was the configuration which
+    # told anaconda how to execute.
+    if i_info.get('ks_template'):
+        # If the value is set, we always overwrite it, in the event
+        # a rebuild is occuring or something along those lines.
+        node.set_driver_internal_info('ks_template',
+                                      i_info['ks_template'])
+    else:
+        _get_image_properties()
+        # ks_template is an optional property on the image
+        if image_properties and 'ks_template' in image_properties:
+            node.set_driver_internal_info(
+                'ks_template', str(image_properties['ks_template']))
+        else:
+            # If not defined, default to the overall system default
+            # kickstart template, as opposed to a user supplied
+            # template.
+            node.set_driver_internal_info(
+                'ks_template',
+                'file://' + os.path.abspath(
+                    CONF.anaconda.default_ks_template
+                )
+            )
+
+    node.save()
+
+    for label in labels + anaconda_labels:
         image_info[label] = (
-            i_info[label],
-            os.path.join(root_dir, node.uuid, label)
+            i_info.get(label) or node.driver_internal_info.get(label, ''),
+            get_file_path_from_label(node.uuid, root_dir, label)
         )
 
     return image_info
@@ -681,6 +806,7 @@ def build_deploy_pxe_options(task, pxe_info, mode='deploy',
     node = task.node
     kernel_label = '%s_kernel' % mode
     ramdisk_label = '%s_ramdisk' % mode
+    initrd_filename = ramdisk_label
     for label, option in ((kernel_label, 'deployment_aki_path'),
                           (ramdisk_label, 'deployment_ari_path')):
         if ipxe_enabled:
@@ -689,38 +815,56 @@ def build_deploy_pxe_options(task, pxe_info, mode='deploy',
                     and service_utils.is_glance_image(image_href)):
                 pxe_opts[option] = images.get_temp_url_for_glance_image(
                     task.context, image_href)
+                if label == ramdisk_label:
+                    path = urlparse.urlparse(pxe_opts[option]).path.strip('/')
+                    if path:
+                        initrd_filename = path.split('/')[-1]
             else:
                 pxe_opts[option] = '/'.join([CONF.deploy.http_url, node.uuid,
                                             label])
         else:
-            pxe_opts[option] = get_path_relative_to_tftp_root(
-                pxe_info[label][1])
+            pxe_opts[option] = os.path.relpath(pxe_info[label][1],
+                                               CONF.pxe.tftp_root)
     if ipxe_enabled:
-        pxe_opts['initrd_filename'] = ramdisk_label
+        pxe_opts['initrd_filename'] = initrd_filename
     return pxe_opts
 
 
 def build_instance_pxe_options(task, pxe_info, ipxe_enabled=False):
     pxe_opts = {}
     node = task.node
+    isap = node.driver_internal_info.get('is_source_a_path')
 
     for label, option in (('kernel', 'aki_path'),
-                          ('ramdisk', 'ari_path')):
+                          ('ramdisk', 'ari_path'),
+                          ('stage2', 'stage2_url'),
+                          ('ks_template', 'ks_template_path'),
+                          ('ks_cfg', 'ks_cfg_url')):
         if label in pxe_info:
-            if ipxe_enabled:
+            if ipxe_enabled or label in ('stage2', 'ks_template', 'ks_cfg'):
                 # NOTE(pas-ha) do not use Swift TempURLs for kernel and
                 # ramdisk of user image when boot_option is not local,
                 # as this breaks instance reboot later when temp urls
                 # have timed out.
-                pxe_opts[option] = '/'.join(
-                    [CONF.deploy.http_url, node.uuid, label])
+                pxe_opts[option] = get_http_url_path_from_label(
+                    CONF.deploy.http_url, node.uuid, label)
             else:
                 # It is possible that we don't have kernel/ramdisk or even
                 # image_source to determine if it's a whole disk image or not.
                 # For example, when transitioning to 'available' state
                 # for first time from 'manage' state.
-                pxe_opts[option] = get_path_relative_to_tftp_root(
-                    pxe_info[label][1])
+                pxe_opts[option] = os.path.relpath(pxe_info[label][1],
+                                                   CONF.pxe.tftp_root)
+
+    # NOTE(TheJulia): This is basically anaconda specific, but who knows
+    # one day! Copy image_source to repo_url if it is a URL to a directory
+    # path, and an explicit stage2 URL is not defined as .treeinfo is totally
+    # a thing and anaconda's dracut element knows the secrets of how to
+    # get and use the treeinfo file. And yes, this is a hidden file. :\
+    # example:
+    # http://mirror.stream.centos.org/9-stream/BaseOS/x86_64/os/.treeinfo
+    if isap and 'stage2_url' not in pxe_opts:
+        pxe_opts['repo_url'] = node.instance_info.get('image_source')
 
     pxe_opts.setdefault('aki_path', 'no_kernel')
     pxe_opts.setdefault('ari_path', 'no_ramdisk')
@@ -742,15 +886,17 @@ def build_instance_pxe_options(task, pxe_info, ipxe_enabled=False):
     return pxe_opts
 
 
-def build_extra_pxe_options(task=None, ramdisk_params=None):
+def build_extra_pxe_options(task, ramdisk_params=None):
+    pxe_append_params = driver_utils.get_kernel_append_params(
+        task.node, default=CONF.pxe.kernel_append_params)
     # Enable debug in IPA according to CONF.debug if it was not
     # specified yet
-    pxe_append_params = CONF.pxe.pxe_append_params
     if CONF.debug and 'ipa-debug' not in pxe_append_params:
         pxe_append_params += ' ipa-debug=1'
     if ramdisk_params:
-        pxe_append_params += ' ' + ' '.join('%s=%s' % tpl
-                                            for tpl in ramdisk_params.items())
+        pxe_append_params += ' ' + ' '.join(
+            ('%s=%s' % tpl) if tpl[1] is not None else tpl[0]
+            for tpl in ramdisk_params.items())
     if task and task.context.global_id:
         pxe_append_params += (
             ' ipa-global-request-id=%s' % task.context.global_id)
@@ -810,7 +956,8 @@ def build_service_pxe_config(task, instance_image_info,
                              root_uuid_or_disk_id,
                              ramdisk_boot=False,
                              ipxe_enabled=False,
-                             is_whole_disk_image=None):
+                             is_whole_disk_image=None,
+                             anaconda_boot=False):
     node = task.node
     pxe_config_path = get_pxe_config_file_path(node.uuid,
                                                ipxe_enabled=ipxe_enabled)
@@ -842,9 +989,43 @@ def build_service_pxe_config(task, instance_image_info,
         pxe_config_path, root_uuid_or_disk_id,
         boot_mode_utils.get_boot_mode(node),
         is_whole_disk_image,
-        deploy_utils.is_trusted_boot_requested(node),
-        deploy_utils.is_iscsi_boot(task), ramdisk_boot,
-        ipxe_enabled=ipxe_enabled)
+        iscsi_boot=deploy_utils.is_iscsi_boot(task),
+        ramdisk_boot=ramdisk_boot,
+        ipxe_enabled=ipxe_enabled,
+        anaconda_boot=anaconda_boot)
+
+
+def build_kickstart_config_options(task):
+    """Build the kickstart template options for a node
+
+    This method builds the kickstart template options for a node,
+    given all the required parameters.
+
+    The options should then be passed to pxe_utils.create_kickstart_config to
+    create the actual config files.
+
+    :param task: A TaskManager object
+    :returns: A dictionary of kickstart options to be used in the kickstart
+              template.
+    """
+    params = {}
+    node = task.node
+    manager_utils.add_secret_token(node, pregenerated=True)
+    node.save()
+    params['liveimg_url'] = node.instance_info['image_url']
+    if node.driver_internal_info.get('is_source_a_path', False):
+        # Record a value so it matches as the template opts in.
+        params['is_source_a_path'] = 'true'
+    if CONF.anaconda.insecure_heartbeat:
+        params['insecure_heartbeat'] = 'true'
+    params['agent_token'] = node.driver_internal_info['agent_secret_token']
+    heartbeat_url = '%s/v1/heartbeat/%s' % (
+        deploy_utils.get_ironic_api_url().rstrip('/'),
+        node.uuid
+    )
+    params['heartbeat_url'] = heartbeat_url
+    params['config_drive'] = ks_utils.prepare_config_drive(task)
+    return {'ks_options': params}
 
 
 def get_volume_pxe_options(task):
@@ -861,12 +1042,7 @@ def get_volume_pxe_options(task):
             return prop
         return __return_item_or_first_if_list(properties.get(key + 's', ''))
 
-    def __generate_iscsi_url(properties):
-        """Returns iscsi url."""
-        portal = __get_property(properties, 'target_portal')
-        iqn = __get_property(properties, 'target_iqn')
-        lun = __get_property(properties, 'target_lun')
-
+    def __format_portal(portal, iqn, lun):
         if ':' in portal:
             host, port = portal.split(':')
         else:
@@ -875,6 +1051,20 @@ def get_volume_pxe_options(task):
         return ("iscsi:%(host)s::%(port)s:%(lun)s:%(iqn)s" %
                 {'host': host, 'port': port, 'lun': lun, 'iqn': iqn})
 
+    def __generate_iscsi_url(properties):
+        """Returns iscsi url."""
+        iqn = __get_property(properties, 'target_iqn')
+        lun = __get_property(properties, 'target_lun')
+        if 'target_portals' in properties:
+            portals = properties.get('target_portals')
+            formatted_portals = []
+            for portal in portals:
+                formatted_portals.append(__format_portal(portal, iqn, lun))
+            return ' '.join(formatted_portals)
+        else:
+            portal = __get_property(properties, 'target_portal')
+            return __format_portal(portal, iqn, lun)
+
     pxe_options = {}
     node = task.node
     boot_volume = node.driver_internal_info.get('boot_from_volume')
@@ -914,33 +1104,73 @@ def get_volume_pxe_options(task):
     return pxe_options
 
 
-def validate_boot_parameters_for_trusted_boot(node):
-    """Check if boot parameters are valid for trusted boot."""
-    boot_mode = boot_mode_utils.get_boot_mode(node)
-    boot_option = deploy_utils.get_boot_option(node)
-    is_whole_disk_image = node.driver_internal_info.get('is_whole_disk_image')
-    # 'is_whole_disk_image' is not supported by trusted boot, because there is
-    # no Kernel/Ramdisk to measure at all.
-    if (boot_mode != 'bios'
-        or is_whole_disk_image
-        or boot_option != 'netboot'):
-        msg = (_("Trusted boot is only supported in BIOS boot mode with "
-                 "netboot and without whole_disk_image, but Node "
-                 "%(node_uuid)s was configured with boot_mode: %(boot_mode)s, "
-                 "boot_option: %(boot_option)s, is_whole_disk_image: "
-                 "%(is_whole_disk_image)s: at least one of them is wrong, and "
-                 "this can be caused by enable secure boot.") %
-               {'node_uuid': node.uuid, 'boot_mode': boot_mode,
-                'boot_option': boot_option,
-                'is_whole_disk_image': is_whole_disk_image})
-        LOG.error(msg)
-        raise exception.InvalidParameterValue(msg)
+def validate_kickstart_template(ks_template):
+    """Validate the kickstart template
+
+    :param ks_template: Path to the kickstart template
+    :raises: InvalidKickstartTemplate
+    """
+    ks_options = {'liveimg_url': 'fake_image_url',
+                  'agent_token': 'fake_token',
+                  'config_drive': '',
+                  'heartbeat_url': 'fake_heartbeat_url'}
+    params = {'ks_options': ks_options}
+    try:
+        rendered_tmpl = utils.render_template(ks_template, params, strict=True)
+    except jinja2.exceptions.UndefinedError as exc:
+        msg = (_("The kickstart template includes a variable that is not "
+                 "a valid kickstart option. Rendering the template returned "
+                 " %(msg)s. The valid options are %(valid_options)s.") %
+               {'msg': exc.message,
+                'valid_options': ','.join(ks_options.keys())})
+        raise exception.InvalidKickstartTemplate(msg)
+
+    missing_required_options = []
+    for var, value in ks_options.items():
+        if rendered_tmpl.find(value) == -1:
+            missing_required_options.append(var)
+    if missing_required_options:
+        msg = (_("Following required kickstart option variables are missing "
+                 "from the kickstart template: %(missing_opts)s.") %
+               {'missing_opts': ','.join(missing_required_options)})
+        raise exception.InvalidKickstartTemplate(msg)
+    return rendered_tmpl
+
+
+def validate_kickstart_file(ks_cfg):
+    """Check if the kickstart file is valid
+
+    :param ks_cfg: Contents of kickstart file to validate
+    :raises: InvalidKickstartFile
+    """
+    if not os.path.isfile('/usr/bin/ksvalidator'):
+        LOG.warning(
+            "Unable to validate the kickstart file as ksvalidator binary is "
+            "missing. Please install pykickstart package to enable "
+            "validation of kickstart file."
+        )
+        return
+
+    with tempfile.NamedTemporaryFile(
+            dir=CONF.tempdir, suffix='.cfg', mode='wt') as ks_file:
+        ks_file.write(ks_cfg)
+        ks_file.flush()
+        try:
+            utils.execute(
+                'ksvalidator', ks_file.name, check_on_exit=[0], attempts=1
+            )
+        except processutils.ProcessExecutionError as e:
+            msg = (_("The kickstart file generated does not pass validation. "
+                     "The ksvalidator tool returned the following error: %s") %
+                   e)
+            raise exception.InvalidKickstartFile(msg)
 
 
 def prepare_instance_pxe_config(task, image_info,
                                 iscsi_boot=False,
                                 ramdisk_boot=False,
-                                ipxe_enabled=False):
+                                ipxe_enabled=False,
+                                anaconda_boot=False):
     """Prepares the config file for PXE boot
 
     :param task: a task from TaskManager.
@@ -950,6 +1180,7 @@ def prepare_instance_pxe_config(task, image_info,
     :param ramdisk_boot: if the boot is to a ramdisk configuration.
     :param ipxe_enabled: Default false boolean to indicate if ipxe
                          is in use by the caller.
+    :param anaconda_boot: if the boot is to a anaconda ramdisk configuration.
     :returns: None
     """
     node = task.node
@@ -969,7 +1200,7 @@ def prepare_instance_pxe_config(task, image_info,
         node.uuid, ipxe_enabled=ipxe_enabled)
     if not os.path.isfile(pxe_config_path):
         pxe_options = build_pxe_config_options(
-            task, image_info, service=ramdisk_boot,
+            task, image_info, service=ramdisk_boot or anaconda_boot,
             ipxe_enabled=ipxe_enabled)
         if ipxe_enabled:
             pxe_config_template = (
@@ -984,7 +1215,24 @@ def prepare_instance_pxe_config(task, image_info,
         pxe_config_path, None,
         boot_mode_utils.get_boot_mode(node), False,
         iscsi_boot=iscsi_boot, ramdisk_boot=ramdisk_boot,
-        ipxe_enabled=ipxe_enabled)
+        ipxe_enabled=ipxe_enabled, anaconda_boot=anaconda_boot)
+
+
+def prepare_instance_kickstart_config(task, image_info, anaconda_boot=False):
+    """Prepare to boot anaconda ramdisk by generating kickstart file
+
+    :param task: a task from TaskManager.
+    :param image_info: a dict of values of instance image
+                       metadata to set on the configuration file.
+    :param anaconda_boot: if the boot is to a anaconda ramdisk configuration.
+    """
+    if not anaconda_boot:
+        return
+    ks_options = build_kickstart_config_options(task)
+    kickstart_template = image_info['ks_template'][1]
+    ks_cfg = utils.render_template(kickstart_template, ks_options)
+    utils.write_to_file(image_info['ks_cfg'][1], ks_cfg,
+                        CONF.pxe.file_permission)
 
 
 @image_cache.cleanup(priority=25)
@@ -1003,15 +1251,34 @@ def cache_ramdisk_kernel(task, pxe_info, ipxe_enabled=False):
     """Fetch the necessary kernels and ramdisks for the instance."""
     ctx = task.context
     node = task.node
+    t_pxe_info = copy.copy(pxe_info)
     if ipxe_enabled:
-        path = os.path.join(get_ipxe_root_dir(), node.uuid)
+        path = os.path.join(CONF.deploy.http_root, node.uuid)
     else:
-        path = os.path.join(get_root_dir(), node.uuid)
-    fileutils.ensure_tree(path)
+        path = os.path.join(CONF.pxe.tftp_root, node.uuid)
+    ensure_tree(path)
+    # anaconda deploy will have 'stage2' as one of the labels in pxe_info dict
+    if 'stage2' in pxe_info:
+        # stage2 will be stored in ipxe http directory so make sure the
+        # directory exists.
+        file_path = get_file_path_from_label(node.uuid,
+                                             CONF.deploy.http_root,
+                                             'stage2')
+        ensure_tree(os.path.dirname(file_path))
+
+    if 'ks_cfg' in pxe_info:
+        # ks_cfg is rendered later by the driver using ks_template. It cannot
+        # be fetched and cached.
+        t_pxe_info.pop('ks_cfg')
+
     LOG.debug("Fetching necessary kernel and ramdisk for node %s",
               node.uuid)
-    deploy_utils.fetch_images(ctx, TFTPImageCache(), list(pxe_info.values()),
+    images_info = list(t_pxe_info.values())
+    deploy_utils.fetch_images(ctx, TFTPImageCache(), images_info,
                               CONF.force_raw_images)
+    if CONF.pxe.file_permission:
+        for info in images_info:
+            os.chmod(info[1], CONF.pxe.file_permission)
 
 
 def clean_up_pxe_env(task, images_info, ipxe_enabled=False):
@@ -1031,3 +1298,73 @@ def clean_up_pxe_env(task, images_info, ipxe_enabled=False):
 
     clean_up_pxe_config(task, ipxe_enabled=ipxe_enabled)
     TFTPImageCache().clean_up()
+
+
+def place_loaders_for_boot(base_path):
+    """Place configured bootloaders from the host OS.
+
+    Example: grubaa64.efi:/path/to/grub-aarch64.efi,...
+
+    :param base_path: Destination path where files should be copied to.
+    """
+    loaders = CONF.pxe.loader_file_paths
+    if not loaders or not base_path:
+        # Do nothing, return.
+        return
+
+    for dest, src in loaders.items():
+        (head, _tail) = os.path.split(dest)
+        if head:
+            if head.startswith('/'):
+                # NOTE(TheJulia): The intent here is to error if the operator
+                # has put absolute paths in place, as we can and likely should
+                # copy to multiple folders based upon the protocol operation
+                # being used. Absolute paths are problematic there, where
+                # as a relative path is more a "well, that is silly, but okay
+                # misconfiguration.
+                msg = ('File paths configured for [pxe]loader_file_paths '
+                       'must be relative paths. Entry: %s') % dest
+                raise exception.IncorrectConfiguration(msg)
+            else:
+                try:
+                    ensure_tree(os.path.join(base_path, head))
+                except OSError as e:
+                    msg = ('Failed to create supplied directories in '
+                           'asset copy paths. Error: %s') % e
+                    raise exception.IncorrectConfiguration(msg)
+
+        full_dest = os.path.join(base_path, dest)
+        LOG.debug('Copying bootloader %(dest)s from %(src)s.',
+                  {'src': src, 'dest': full_dest})
+        try:
+            shutil.copy2(src, full_dest)
+            if CONF.pxe.file_permission:
+                os.chmod(full_dest, CONF.pxe.file_permission)
+        except OSError as e:
+            msg = ('Error encountered while attempting to '
+                   'copy a configured bootloader into '
+                   'the requested destination. %s' % e)
+            LOG.error(msg)
+            raise exception.IncorrectConfiguration(error=msg)
+
+
+def place_common_config():
+    """Place template generated config which is not node specific.
+
+    Currently places the initial grub config for grub network boot.
+    """
+    if not CONF.pxe.initial_grub_template:
+        return
+
+    grub_dir_path = os.path.join(_get_root_dir(False), 'grub')
+    if not os.path.isdir(grub_dir_path):
+        fileutils.ensure_tree(grub_dir_path)
+        if CONF.pxe.dir_permission:
+            os.chmod(grub_dir_path, CONF.pxe.dir_permission)
+
+    initial_grub = utils.render_template(
+        CONF.pxe.initial_grub_template,
+        {'tftp_root': _get_root_dir(False)})
+    initial_grub_path = os.path.join(grub_dir_path, 'grub.cfg')
+
+    utils.write_to_file(initial_grub_path, initial_grub)
