@@ -18,6 +18,7 @@
 
 """Utilities and helper functions."""
 
+from collections import abc
 import contextlib
 import datetime
 import errno
@@ -27,6 +28,7 @@ import os
 import re
 import shutil
 import tempfile
+import time
 
 import jinja2
 from oslo_concurrency import processutils
@@ -34,7 +36,9 @@ from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import fileutils
 from oslo_utils import netutils
+from oslo_utils import strutils
 from oslo_utils import timeutils
+import psutil
 import pytz
 
 from ironic.common import exception
@@ -52,8 +56,6 @@ TZ_RE = r'((?P<tz_sign>[+-])(?P<tz_hour>\d{2}):(?P<tz_min>\d{2}))' + \
 DATETIME_RE = re.compile(
     '%sT%s(%s)?' % (DATE_RE, TIME_RE, TZ_RE))
 
-warn_deprecated_extra_vif_port_id = False
-
 
 def _get_root_helper():
     # NOTE(jlvillal): This function has been moved to ironic-lib. And is
@@ -277,9 +279,11 @@ def rmtree_without_raise(path):
                     {'path': path, 'e': e})
 
 
-def write_to_file(path, contents):
+def write_to_file(path, contents, permission=None):
     with open(path, 'w') as f:
         f.write(contents)
+    if permission:
+        os.chmod(path, permission)
 
 
 def create_link_without_raise(source, link):
@@ -322,7 +326,7 @@ def mount(src, dest, *args):
         to run the process.
     """
     args = ('mount', ) + args + (src, dest)
-    execute(*args, run_as_root=True, check_exit_code=[0])
+    execute(*args, run_as_root=True)
 
 
 def umount(loc, *args):
@@ -335,7 +339,7 @@ def umount(loc, *args):
         to run the process.
     """
     args = ('umount', ) + args + (loc, )
-    execute(*args, run_as_root=True, check_exit_code=[0])
+    execute(*args, run_as_root=True)
 
 
 def check_dir(directory_to_check=None, required_space=1):
@@ -468,13 +472,15 @@ def validate_network_port(port, port_name="Port"):
         {'port_name': port_name, 'port': port})
 
 
-def render_template(template, params, is_file=True):
+def render_template(template, params, is_file=True, strict=False):
     """Renders Jinja2 template file with given parameters.
 
     :param template: full path to the Jinja2 template file
     :param params: dictionary with parameters to use when rendering
     :param is_file: whether template is file or string with template itself
-    :returns: the rendered template as a string
+    :param strict: Enable strict template rendering. Default is False
+    :returns: Rendered template
+    :raises: jinja2.exceptions.UndefinedError
     """
     if is_file:
         tmpl_path, tmpl_name = os.path.split(template)
@@ -486,23 +492,15 @@ def render_template(template, params, is_file=True):
     # and still complains with B701 for that line
     # NOTE(pas-ha) not using default_for_string=False as we set the name
     # of the template above for strings too.
-    env = jinja2.Environment(loader=loader,  # nosec B701
-                             autoescape=jinja2.select_autoescape())
+    env = jinja2.Environment(  # nosec B701
+        loader=loader,
+        autoescape=jinja2.select_autoescape(),
+        undefined=jinja2.StrictUndefined if strict else jinja2.Undefined
+    )
     tmpl = env.get_template(tmpl_name)
     return tmpl.render(params, enumerate=enumerate)
 
 
-def warn_about_deprecated_extra_vif_port_id():
-    global warn_deprecated_extra_vif_port_id
-    if not warn_deprecated_extra_vif_port_id:
-        warn_deprecated_extra_vif_port_id = True
-        LOG.warning("Starting with API version 1.28, attaching/detaching VIF "
-                    "to/from a port or port group via extra['vif_port_id'] is "
-                    "deprecated and will not be supported starting in the "
-                    "Stein release. API endpoint v1/nodes/<node>/vifs should "
-                    "be used instead.")
-
-
 def parse_instance_info_capabilities(node):
     """Parse the instance_info capabilities.
 
@@ -580,3 +578,106 @@ def wrap_ipv6(ip):
     if ipaddress.ip_address(ip).version == 6:
         return "[%s]" % ip
     return ip
+
+
+def file_mime_type(path):
+    """Gets a mime type of the given file."""
+    return execute('file', '--brief', '--mime-type', path,
+                   use_standard_locale=True)[0].strip()
+
+
+def _get_mb_ram_available():
+    # NOTE(TheJulia): The .available value is the memory that can be given
+    # to a process without this process beginning to swap itself.
+    return psutil.virtual_memory().available / 1024 / 1024
+
+
+def is_memory_insufficient(raise_if_fail=False):
+    """Checks available system memory and holds the deployment process.
+
+    Evaluates the current system memory available, meaning can be
+    allocated to a process by the kernel upon allocation request,
+    and delays the execution until memory has been freed,
+    or until it has timed out.
+
+    This method will issue a sleep, if the amount of available memory is
+    insufficient. This is configured using the
+    ``[DEFAULT]minimum_memory_wait_time`` and the
+    ``[DEFAULT]minimum_memory_wait_retries``.
+
+    :param raise_if_fail: Default False, but if set to true an
+                          InsufficientMemory exception is raised
+                          upon insufficient memory.
+    :returns: True if the check has timed out. Otherwise None is returned.
+    :raises: InsufficientMemory if the raise_if_fail parameter is set to
+             True.
+    """
+    required_memory = CONF.minimum_required_memory
+    loop_count = 0
+
+    while _get_mb_ram_available() < required_memory:
+        log_values = {
+            'available': _get_mb_ram_available(),
+            'required': required_memory,
+        }
+        if CONF.minimum_memory_warning_only:
+            LOG.warning('Memory is at %(available)s MiB, required is '
+                        '%(required)s. Ironic is in warning-only mode '
+                        'which can be changed by altering the '
+                        '[DEFAULT]minimum_memory_warning_only',
+                        log_values)
+            return False
+        if loop_count >= CONF.minimum_memory_wait_retries:
+            LOG.error('Memory is at %(available)s MiB, required is '
+                      '%(required)s. Notifying caller that we have '
+                      'exceeded retries.',
+                      log_values)
+            if raise_if_fail:
+                raise exception.InsufficientMemory(
+                    free=_get_mb_ram_available(),
+                    required=required_memory)
+            return True
+        LOG.warning('Memory is at %(available)s MiB, required is '
+                    '%(required)s, waiting.', log_values)
+        # Sleep so interpreter can switch threads.
+        time.sleep(CONF.minimum_memory_wait_time)
+        loop_count = loop_count + 1
+
+
+_LARGE_KEYS = frozenset(['system_logs'])
+
+
+def remove_large_keys(var):
+    """Remove specific keys from the var, recursing into dicts and lists."""
+    if isinstance(var, abc.Mapping):
+        return {key: (remove_large_keys(value)
+                      if key not in _LARGE_KEYS else '<...>')
+                for key, value in var.items()}
+    elif isinstance(var, abc.Sequence) and not isinstance(var, str):
+        return var.__class__(map(remove_large_keys, var))
+    else:
+        return var
+
+
+def fast_track_enabled(node):
+    is_enabled = node.driver_info.get('fast_track')
+    if is_enabled is None:
+        return CONF.deploy.fast_track
+    else:
+        try:
+            return strutils.bool_from_string(is_enabled, strict=True)
+        except ValueError as exc:
+            raise exception.InvalidParameterValue(
+                _("Invalid value of fast_track: %s") % exc)
+
+
+def is_fips_enabled():
+    """Check if FIPS mode is enabled in the system."""
+    try:
+        with open('/proc/sys/crypto/fips_enabled', 'r') as f:
+            content = f.read()
+            if content == "1\n":
+                return True
+    except Exception:
+        pass
+    return False
