@@ -12,6 +12,7 @@
 
 """Base conductor manager functionality."""
 
+import copy
 import inspect
 import threading
 
@@ -19,11 +20,11 @@ import eventlet
 import futurist
 from futurist import periodics
 from futurist import rejection
+from ironic_lib import mdns
 from oslo_db import exception as db_exception
 from oslo_log import log
 from oslo_utils import excutils
 from oslo_utils import versionutils
-import six
 
 from ironic.common import context as ironic_context
 from ironic.common import driver_factory
@@ -33,11 +34,13 @@ from ironic.common.i18n import _
 from ironic.common import release_mappings as versions
 from ironic.common import rpc
 from ironic.common import states
+from ironic.conductor import allocations
 from ironic.conductor import notification_utils as notify_utils
 from ironic.conductor import task_manager
+from ironic.conductor import utils
 from ironic.conf import CONF
 from ironic.db import api as dbapi
-from ironic.drivers import base as driver_base
+from ironic.drivers.modules import deploy_utils
 from ironic import objects
 from ironic.objects import fields as obj_fields
 
@@ -45,27 +48,6 @@ from ironic.objects import fields as obj_fields
 LOG = log.getLogger(__name__)
 
 
-def _check_enabled_interfaces():
-    """Sanity-check enabled_*_interfaces configs.
-
-    We do this before we even bother to try to load up drivers. If we have any
-    dynamic drivers enabled, then we need interfaces enabled as well.
-
-    :raises: ConfigInvalid if an enabled interfaces config option is empty.
-    """
-    empty_confs = []
-    iface_types = ['enabled_%s_interfaces' % i
-                   for i in driver_base.ALL_INTERFACES]
-    for iface_type in iface_types:
-        conf_value = getattr(CONF, iface_type)
-        if not conf_value:
-            empty_confs.append(iface_type)
-    if empty_confs:
-        msg = (_('Configuration options %s cannot be an empty list.') %
-               ', '.join(empty_confs))
-        raise exception.ConfigInvalid(error_msg=msg)
-
-
 class BaseConductorManager(object):
 
     def __init__(self, host, topic):
@@ -77,6 +59,34 @@ class BaseConductorManager(object):
         self.sensors_notifier = rpc.get_sensors_notifier()
         self._started = False
         self._shutdown = None
+        self._zeroconf = None
+        self.dbapi = None
+
+    def prepare_host(self):
+        """Prepares host for initialization
+
+        Prepares the conductor for basic operation by removing any existing
+        transitory node power states and reservations which were previously
+        held by this host.
+
+        Under normal operation, this is also when the initial database
+        connectivity is established for the conductor's normal operation.
+        """
+        # NOTE(TheJulia) We need to clear locks early on in the process
+        # of starting where the database shows we still hold them.
+        # This must be done before we re-register our existence in the
+        # conductors table and begin accepting new requests via RPC as
+        # if we do not then we may squash our *new* locks from new work.
+
+        if not self.dbapi:
+            LOG.debug('Initializing database client for %s.', self.host)
+            self.dbapi = dbapi.get_instance()
+        LOG.debug('Removing stale locks from the database matching '
+                  'this conductor\'s hostname: %s', self.host)
+        # clear all target_power_state with locks by this conductor
+        self.dbapi.clear_node_target_power_state(self.host)
+        # clear all locks held by this conductor before registering
+        self.dbapi.clear_node_reservations_for_conductor(self.host)
 
     def init_host(self, admin_context=None):
         """Initialize the conductor host.
@@ -94,7 +104,8 @@ class BaseConductorManager(object):
                                  'conductor manager'))
         self._shutdown = False
 
-        self.dbapi = dbapi.get_instance()
+        if not self.dbapi:
+            self.dbapi = dbapi.get_instance()
 
         self._keepalive_evt = threading.Event()
         """Event for the keepalive thread."""
@@ -113,14 +124,8 @@ class BaseConductorManager(object):
             use_groups=self._use_groups())
         """Consistent hash ring which maps drivers to conductors."""
 
-        # TODO(dtantsur): remove in Stein
-        if CONF.enabled_drivers:
-            raise RuntimeError("The enabled_drivers configuration option "
-                               "no longer has any effect and must be empty")
-
-        _check_enabled_interfaces()
-
-        # NOTE(deva): these calls may raise DriverLoadError or DriverNotFound
+        # NOTE(tenbrae): these calls may raise DriverLoadError or
+        # DriverNotFound
         # NOTE(vdrok): Instantiate network and storage interface factory on
         # startup so that all the interfaces are loaded at the very
         # beginning, and failures prevent the conductor from starting.
@@ -142,10 +147,6 @@ class BaseConductorManager(object):
 
         self._collect_periodic_tasks(admin_context)
 
-        # clear all target_power_state with locks by this conductor
-        self.dbapi.clear_node_target_power_state(self.host)
-        # clear all locks held by this conductor before registering
-        self.dbapi.clear_node_reservations_for_conductor(self.host)
         try:
             # Register this conductor with the cluster
             self.conductor = objects.Conductor.register(
@@ -204,6 +205,16 @@ class BaseConductorManager(object):
                 LOG.critical('Failed to start keepalive')
                 self.del_host()
 
+        # Resume allocations that started before the restart.
+        try:
+            self._spawn_worker(self._resume_allocations,
+                               ironic_context.get_admin_context())
+        except exception.NoFreeConductorWorker:
+            LOG.warning('Failed to start worker for resuming allocations.')
+
+        if CONF.conductor.enable_mdns:
+            self._publish_endpoint()
+
         self._started = True
 
     def _use_groups(self):
@@ -288,6 +299,8 @@ class BaseConductorManager(object):
             return
         self._shutdown = True
         self._keepalive_evt.set()
+        # clear all locks held by this conductor before deregistering
+        self.dbapi.clear_node_reservations_for_conductor(self.host)
         if deregister:
             try:
                 # Inform the cluster that this conductor is shutting down.
@@ -308,6 +321,11 @@ class BaseConductorManager(object):
         self._periodic_tasks.stop()
         self._periodic_tasks.wait()
         self._executor.shutdown(wait=True)
+
+        if self._zeroconf is not None:
+            self._zeroconf.close()
+            self._zeroconf = None
+
         self._started = False
 
     def _register_and_validate_hardware_interfaces(self, hardware_types):
@@ -332,15 +350,21 @@ class BaseConductorManager(object):
         # first unregister, in case we have cruft laying around
         self.conductor.unregister_all_hardware_interfaces()
 
+        interfaces = []
         for ht_name, ht in hardware_types.items():
             interface_map = driver_factory.enabled_supported_interfaces(ht)
             for interface_type, interface_names in interface_map.items():
                 default_interface = driver_factory.default_interface(
                     ht, interface_type, driver_name=ht_name)
-                self.conductor.register_hardware_interfaces(ht_name,
-                                                            interface_type,
-                                                            interface_names,
-                                                            default_interface)
+                interface = {}
+                interface["hardware_type"] = ht_name
+                interface["interface_type"] = interface_type
+                for interface_name in interface_names:
+                    interface["interface_name"] = interface_name
+                    interface["default"] = \
+                        (interface_name == default_interface)
+                    interfaces.append(copy.copy(interface))
+        self.conductor.register_hardware_interfaces(interfaces)
 
         # TODO(jroll) validate against other conductor, warn if different
         # how do we do this performantly? :|
@@ -419,9 +443,7 @@ class BaseConductorManager(object):
         except exception.DriverNotFound:
             return False
 
-        return self.host in ring.get_nodes(
-            node_uuid.encode('utf-8'),
-            replicas=CONF.hash_distribution_replicas)
+        return self.host in ring.get_nodes(node_uuid.encode('utf-8'))
 
     def _fail_if_in_state(self, context, filters, provision_state,
                           sort_key, callback_method=None,
@@ -434,46 +456,50 @@ class BaseConductorManager(object):
         in whatever provisioning activity it was currently doing.
         That failure is processed here.
 
-        :param: context: request context
-        :param: filters: criteria (as a dictionary) to get the desired
-                         list of nodes that satisfy the filter constraints.
-                         For example, if filters['provisioned_before'] = 60,
-                         this would process nodes whose provision_updated_at
-                         field value was 60 or more seconds before 'now'.
-        :param: provision_state: provision_state that the node is in,
-                                 for the provisioning activity to have failed,
-                                 either one string or a set.
-        :param: sort_key: the nodes are sorted based on this key.
-        :param: callback_method: the callback method to be invoked in a
-                                 spawned thread, for a failed node. This
-                                 method must take a :class:`TaskManager` as
-                                 the first (and only required) parameter.
-        :param: err_handler: for a failed node, the error handler to invoke
-                             if an error occurs trying to spawn an thread
-                             to do the callback_method.
-        :param: last_error: the error message to be updated in node.last_error
-        :param: keep_target_state: if True, a failed node will keep the same
-                                   target provision state it had before the
-                                   failure. Otherwise, the node's target
-                                   provision state will be determined by the
-                                   fsm.
+        :param context: request context
+        :param filters: criteria (as a dictionary) to get the desired
+                        list of nodes that satisfy the filter constraints.
+                        For example, if filters['provisioned_before'] = 60,
+                        this would process nodes whose provision_updated_at
+                        field value was 60 or more seconds before 'now'.
+        :param provision_state: provision_state that the node is in,
+                                for the provisioning activity to have failed,
+                                either one string or a set.
+        :param sort_key: the nodes are sorted based on this key.
+        :param callback_method: the callback method to be invoked in a
+                                spawned thread, for a failed node. This
+                                method must take a :class:`TaskManager` as
+                                the first (and only required) parameter.
+        :param err_handler: for a failed node, the error handler to invoke
+                            if an error occurs trying to spawn an thread
+                            to do the callback_method.
+        :param last_error: the error message to be updated in node.last_error
+        :param keep_target_state: if True, a failed node will keep the same
+                                  target provision state it had before the
+                                  failure. Otherwise, the node's target
+                                  provision state will be determined by the
+                                  fsm.
 
         """
-        if isinstance(provision_state, six.string_types):
+        if isinstance(provision_state, str):
             provision_state = {provision_state}
 
         node_iter = self.iter_nodes(filters=filters,
                                     sort_key=sort_key,
                                     sort_dir='asc')
-
+        desired_maintenance = filters.get('maintenance')
         workers_count = 0
         for node_uuid, driver, conductor_group in node_iter:
             try:
                 with task_manager.acquire(context, node_uuid,
                                           purpose='node state check') as task:
-                    if (task.node.maintenance
-                            or task.node.provision_state
-                            not in provision_state):
+                    # Check maintenance value since it could have changed
+                    # after the filtering was done.
+                    if (desired_maintenance is not None
+                            and desired_maintenance != task.node.maintenance):
+                        continue
+
+                    if task.node.provision_state not in provision_state:
                         continue
 
                     target_state = (None if not keep_target_state else
@@ -487,7 +513,10 @@ class BaseConductorManager(object):
                                            err_handler=err_handler,
                                            target_state=target_state)
                     else:
-                        task.node.last_error = last_error
+                        utils.node_history_record(
+                            task.node, event=last_error,
+                            error=True,
+                            event_type=states.TRANSITION)
                         task.process_event('fail', target_state=target_state)
             except exception.NoFreeConductorWorker:
                 break
@@ -500,7 +529,7 @@ class BaseConductorManager(object):
     def _start_consoles(self, context):
         """Start consoles if set enabled.
 
-        :param: context: request context
+        :param context: request context
         """
         filters = {'console_enabled': True}
 
@@ -531,7 +560,9 @@ class BaseConductorManager(object):
                         LOG.error(msg)
                         # If starting console failed, set node console_enabled
                         # back to False and set node's last error.
-                        task.node.last_error = msg
+                        utils.node_history_record(task.node, event=msg,
+                                                  error=True,
+                                                  event_type=states.STARTFAIL)
                         task.node.console_enabled = False
                         task.node.save()
                         notify_utils.emit_console_notification(
@@ -550,3 +581,20 @@ class BaseConductorManager(object):
             finally:
                 # Yield on every iteration
                 eventlet.sleep(0)
+
+    def _resume_allocations(self, context):
+        """Resume unfinished allocations on restart."""
+        filters = {'state': states.ALLOCATING,
+                   'conductor_affinity': self.conductor.id}
+        for allocation in objects.Allocation.list(context, filters=filters):
+            LOG.debug('Resuming unfinished allocation %s', allocation.uuid)
+            allocations.do_allocate(context, allocation)
+
+    def _publish_endpoint(self):
+        params = {}
+        if CONF.debug:
+            params['ipa_debug'] = True
+        self._zeroconf = mdns.Zeroconf()
+        self._zeroconf.register_service('baremetal',
+                                        deploy_utils.get_ironic_api_url(),
+                                        params=params)
