@@ -49,8 +49,10 @@ def do_node_clean(task, clean_steps=None, disable_ramdisk=False):
         node.save()
 
         task.process_event('done')
-        LOG.info('Automated cleaning is disabled, node %s has been '
-                 'successfully moved to AVAILABLE state.', node.uuid)
+        how = ('API' if node.automated_clean is False else 'configuration')
+        LOG.info('Automated cleaning is disabled via %(how)s, node %(node)s '
+                 'has been successfully moved to AVAILABLE state',
+                 {'how': how, 'node': node})
         return
 
     # NOTE(dtantsur): this is only reachable during automated cleaning,
@@ -67,7 +69,7 @@ def do_node_clean(task, clean_steps=None, disable_ramdisk=False):
         task.driver.power.validate(task)
         if not disable_ramdisk:
             task.driver.network.validate(task)
-    except exception.InvalidParameterValue as e:
+    except (exception.InvalidParameterValue, exception.NetworkError) as e:
         msg = (_('Validation of node %(node)s for cleaning failed: %(msg)s') %
                {'node': node.uuid, 'msg': e})
         return utils.cleaning_error_handler(task, msg)
@@ -245,21 +247,12 @@ def do_next_clean_step(task, step_index, disable_ramdisk=None):
     task.process_event(event)
 
 
-def get_last_error(node):
-    last_error = _('By request, the clean operation was aborted')
-    if node.clean_step:
-        last_error += (
-            _(' during or after the completion of step "%s"')
-            % conductor_steps.step_id(node.clean_step)
-        )
-    return last_error
-
-
 @task_manager.require_exclusive_lock
-def do_node_clean_abort(task):
+def do_node_clean_abort(task, step_name=None):
     """Internal method to abort an ongoing operation.
 
     :param task: a TaskManager instance with an exclusive lock
+    :param step_name: The name of the clean step.
     """
     node = task.node
     try:
@@ -277,13 +270,12 @@ def do_node_clean_abort(task):
                                      set_fail_state=False)
         return
 
-    last_error = get_last_error(node)
     info_message = _('Clean operation aborted for node %s') % node.uuid
-    if node.clean_step:
-        info_message += (
-            _(' during or after the completion of step "%s"')
-            % node.clean_step
-        )
+    last_error = _('By request, the clean operation was aborted')
+    if step_name:
+        msg = _(' after the completion of step "%s"') % step_name
+        last_error += msg
+        info_message += msg
 
     node.last_error = last_error
     node.clean_step = None
@@ -325,7 +317,7 @@ def continue_node_clean(task):
                 target_state = None
 
             task.process_event('fail', target_state=target_state)
-            do_node_clean_abort(task)
+            do_node_clean_abort(task, step_name)
             return
 
         LOG.debug('The cleaning operation for node %(node)s was '
