@@ -20,20 +20,63 @@ Client side of the conductor RPC API.
 
 import random
 
+from ironic_lib.json_rpc import client as json_rpc
+from oslo_log import log
 import oslo_messaging as messaging
 
 from ironic.common import exception
 from ironic.common import hash_ring
 from ironic.common.i18n import _
-from ironic.common.json_rpc import client as json_rpc
 from ironic.common import release_mappings as versions
 from ironic.common import rpc
-from ironic.conductor import manager
 from ironic.conf import CONF
 from ironic.db import api as dbapi
 from ironic.objects import base as objects_base
 
 
+LOG = log.getLogger(__name__)
+
+
+class LocalContext:
+    """Context to make calls to a local conductor."""
+
+    __slots__ = ()
+
+    def call(self, context, rpc_call_name, **kwargs):
+        """Make a local conductor call."""
+        if rpc.GLOBAL_MANAGER is None:
+            raise exception.ServiceUnavailable(
+                _("The built-in conductor is not available, it might have "
+                  "crashed. Please check the logs and correct the "
+                  "configuration, if required."))
+        try:
+            return getattr(rpc.GLOBAL_MANAGER, rpc_call_name)(context,
+                                                              **kwargs)
+        # FIXME(dtantsur): can we somehow avoid wrapping the exception?
+        except messaging.ExpectedException as exc:
+            exc_value, exc_tb = exc.exc_info[1:]
+            raise exc_value.with_traceback(exc_tb) from None
+
+    def cast(self, context, rpc_call_name, **kwargs):
+        """Make a local conductor call.
+
+        It is expected that the underlying call uses a thread to avoid
+        blocking the caller.
+
+        Any exceptions are logged and ignored.
+        """
+        try:
+            return self.call(context, rpc_call_name, **kwargs)
+        except Exception:
+            # In real RPC, casts are completely asynchronous and never return
+            # actual errors.
+            LOG.exception('Ignoring unhandled exception from RPC cast %s',
+                          rpc_call_name)
+
+
+_LOCAL_CONTEXT = LocalContext()
+
+
 class ConductorAPI(object):
     """Client side of the conductor RPC API.
 
@@ -104,19 +147,23 @@ class ConductorAPI(object):
     |    1.50 - Added set_indicator_state, get_indicator_state and
     |           get_supported_indicators.
     |    1.51 - Added agent_verify_ca to heartbeat.
-
+    |    1.52 - Added deploy steps argument to provisioning
+    |    1.53 - Added disable_ramdisk to do_node_clean.
+    |    1.54 - Added optional agent_status and agent_status_message to
+                heartbeat
+    |    1.55 - Added change_node_boot_mode
     """
 
     # NOTE(rloo): This must be in sync with manager.ConductorManager's.
     # NOTE(pas-ha): This also must be in sync with
     #               ironic.common.release_mappings.RELEASE_MAPPING['master']
-    RPC_API_VERSION = '1.51'
+    RPC_API_VERSION = '1.55'
 
     def __init__(self, topic=None):
         super(ConductorAPI, self).__init__()
         self.topic = topic
         if self.topic is None:
-            self.topic = manager.MANAGER_TOPIC
+            self.topic = rpc.MANAGER_TOPIC
 
         serializer = objects_base.IronicObjectSerializer()
         release_ver = versions.RELEASE_MAPPING.get(CONF.pin_release_version)
@@ -127,14 +174,46 @@ class ConductorAPI(object):
             self.client = json_rpc.Client(serializer=serializer,
                                           version_cap=version_cap)
             self.topic = ''
-        else:
+        elif CONF.rpc_transport != 'none':
             target = messaging.Target(topic=self.topic, version='1.0')
             self.client = rpc.get_client(target, version_cap=version_cap,
                                          serializer=serializer)
+        else:
+            self.client = None
 
-        use_groups = self.client.can_send_version('1.47')
         # NOTE(tenbrae): this is going to be buggy
-        self.ring_manager = hash_ring.HashRingManager(use_groups=use_groups)
+        self.ring_manager = hash_ring.HashRingManager()
+
+    def _prepare_call(self, topic, version=None):
+        """Prepare an RPC call.
+
+        If a conductor exists in the same process, a direct function call
+        is used instead of real RPC.
+
+        :param topic: RPC topic to send to.
+        :param version: RPC API version to require.
+        """
+        # FIXME(dtantsur): this doesn't work with either JSON RPC or local
+        # conductor. Do we even need this fallback?
+        topic = topic or self.topic
+        # Normally a topic is a <topic prefix>.<hostname>, we need to extract
+        # the hostname to match it against the current host.
+        host = topic[len(self.topic) + 1:]
+
+        if rpc.GLOBAL_MANAGER is not None and host == CONF.host:
+            # Short-cut to a local function call if there is a built-in
+            # conductor.
+            return _LOCAL_CONTEXT
+
+        # A safeguard for the case someone uses rpc_transport=None with no
+        # built-in conductor.
+        if self.client is None:
+            raise exception.ServiceUnavailable(
+                _("Cannot use 'none' RPC to connect to remote conductor %s")
+                % host)
+
+        # Normal RPC path
+        return self.client.prepare(topic=topic, version=version)
 
     def get_conductor_for(self, node):
         """Get the conductor which the node is mapped to.
@@ -206,13 +285,17 @@ class ConductorAPI(object):
         """Get RPC topic name for the current conductor."""
         return self.topic + "." + CONF.host
 
+    def _can_send_version(self, version):
+        return (self.client.can_send_version(version)
+                if self.client is not None else True)
+
     def can_send_create_port(self):
         """Return whether the RPCAPI supports the create_port method."""
-        return self.client.can_send_version("1.41")
+        return self._can_send_version("1.41")
 
     def can_send_rescue(self):
         """Return whether the RPCAPI supports node rescue methods."""
-        return self.client.can_send_version("1.43")
+        return self._can_send_version("1.43")
 
     def create_node(self, context, node_obj, topic=None):
         """Synchronously, have a conductor validate and create a node.
@@ -228,7 +311,7 @@ class ConductorAPI(object):
         :raises: NoValidDefaultForInterface if no default can be calculated
                  for some interfaces, and explicit values must be provided.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.36')
+        cctxt = self._prepare_call(topic=topic, version='1.36')
         return cctxt.call(context, 'create_node', node_obj=node_obj)
 
     def update_node(self, context, node_obj, topic=None,
@@ -254,7 +337,7 @@ class ConductorAPI(object):
                  for some interfaces, and explicit values must be provided.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.1')
+        cctxt = self._prepare_call(topic=topic, version='1.1')
         return cctxt.call(context, 'update_node', node_obj=node_obj,
                           reset_interfaces=reset_interfaces)
 
@@ -275,10 +358,50 @@ class ConductorAPI(object):
                  async task.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.39')
+        cctxt = self._prepare_call(topic=topic, version='1.39')
         return cctxt.call(context, 'change_node_power_state', node_id=node_id,
                           new_state=new_state, timeout=timeout)
 
+    def change_node_boot_mode(self, context, node_id, new_state,
+                              topic=None):
+        """Change a node's boot mode.
+
+        Synchronously, acquire lock and start the conductor background task
+        to change boot mode of a node.
+
+        :param context: request context.
+        :param node_id: node id or uuid.
+        :param new_state: one of ironic.common.boot_modes values
+            ('bios' or 'uefi')
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NoFreeConductorWorker when there is no free worker to start
+                 async task.
+
+        """
+        cctxt = self._prepare_call(topic=topic, version='1.55')
+        return cctxt.call(context, 'change_node_boot_mode', node_id=node_id,
+                          new_state=new_state)
+
+    def change_node_secure_boot(self, context, node_id, new_state,
+                                topic=None):
+        """Change a node's secure_boot state.
+
+        Synchronously, acquire lock and start the conductor background task
+        to change secure_boot state of a node.
+
+        :param context: request context.
+        :param node_id: node id or uuid.
+        :param new_state: Target secure boot state
+            (True => 'on' or False => 'off')
+        :param topic: RPC topic. Defaults to self.topic.
+        :raises: NoFreeConductorWorker when there is no free worker to start
+                 async task.
+
+        """
+        cctxt = self._prepare_call(topic=topic, version='1.55')
+        return cctxt.call(context, 'change_node_secure_boot', node_id=node_id,
+                          new_state=new_state)
+
     def vendor_passthru(self, context, node_id, driver_method, http_method,
                         info, topic=None):
         """Receive requests for vendor-specific actions.
@@ -312,7 +435,7 @@ class ConductorAPI(object):
                 or return it in the response body (False).
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.20')
+        cctxt = self._prepare_call(topic=topic, version='1.20')
         return cctxt.call(context, 'vendor_passthru', node_id=node_id,
                           driver_method=driver_method,
                           http_method=http_method,
@@ -357,7 +480,7 @@ class ConductorAPI(object):
                 or return it in the response body (False).
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.20')
+        cctxt = self._prepare_call(topic=topic, version='1.20')
         return cctxt.call(context, 'driver_vendor_passthru',
                           driver_name=driver_name,
                           driver_method=driver_method,
@@ -373,7 +496,7 @@ class ConductorAPI(object):
         :returns: dictionary of <method name>:<method metadata> entries.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.21')
+        cctxt = self._prepare_call(topic=topic, version='1.21')
         return cctxt.call(context, 'get_node_vendor_passthru_methods',
                           node_id=node_id)
 
@@ -395,12 +518,12 @@ class ConductorAPI(object):
         :returns: dictionary of <method name>:<method metadata> entries.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.21')
+        cctxt = self._prepare_call(topic=topic, version='1.21')
         return cctxt.call(context, 'get_driver_vendor_passthru_methods',
                           driver_name=driver_name)
 
     def do_node_deploy(self, context, node_id, rebuild, configdrive,
-                       topic=None):
+                       topic=None, deploy_steps=None):
         """Signal to conductor service to perform a deployment.
 
         :param context: request context.
@@ -408,6 +531,7 @@ class ConductorAPI(object):
         :param rebuild: True if this is a rebuild request.
         :param configdrive: A gzipped and base64 encoded configdrive.
         :param topic: RPC topic. Defaults to self.topic.
+        :param deploy_steps: Deploy steps
         :raises: InstanceDeployFailure
         :raises: InvalidParameterValue if validation fails
         :raises: MissingParameterValue if a required parameter is missing
@@ -418,9 +542,14 @@ class ConductorAPI(object):
         undeployed state before this method is called.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.22')
+        version = '1.22'
+        new_kws = {}
+        if deploy_steps:
+            version = '1.52'
+            new_kws['deploy_steps'] = deploy_steps
+        cctxt = self._prepare_call(topic=topic, version=version)
         return cctxt.call(context, 'do_node_deploy', node_id=node_id,
-                          rebuild=rebuild, configdrive=configdrive)
+                          rebuild=rebuild, configdrive=configdrive, **new_kws)
 
     def do_node_tear_down(self, context, node_id, topic=None):
         """Signal to conductor service to tear down a deployment.
@@ -438,7 +567,7 @@ class ConductorAPI(object):
         deployed state before this method is called.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.6')
+        cctxt = self._prepare_call(topic=topic, version='1.6')
         return cctxt.call(context, 'do_node_tear_down', node_id=node_id)
 
     def do_provisioning_action(self, context, node_id, action, topic=None):
@@ -456,7 +585,7 @@ class ConductorAPI(object):
 
         This encapsulates some provisioning actions in a single call.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.23')
+        cctxt = self._prepare_call(topic=topic, version='1.23')
         return cctxt.call(context, 'do_provisioning_action',
                           node_id=node_id, action=action)
 
@@ -470,7 +599,7 @@ class ConductorAPI(object):
         :param node_id: node id or uuid.
         :param topic: RPC topic. Defaults to self.topic.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.27')
+        cctxt = self._prepare_call(topic=topic, version='1.27')
         return cctxt.cast(context, 'continue_node_clean',
                           node_id=node_id)
 
@@ -484,7 +613,7 @@ class ConductorAPI(object):
         :param node_id: node id or uuid.
         :param topic: RPC topic. Defaults to self.topic.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.45')
+        cctxt = self._prepare_call(topic=topic, version='1.45')
         return cctxt.cast(context, 'continue_node_deploy',
                           node_id=node_id)
 
@@ -498,7 +627,7 @@ class ConductorAPI(object):
                   interface validation.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.5')
+        cctxt = self._prepare_call(topic=topic, version='1.5')
         return cctxt.call(context, 'validate_driver_interfaces',
                           node_id=node_id)
 
@@ -514,7 +643,7 @@ class ConductorAPI(object):
         :raises: InvalidState if the node is in the wrong provision
             state to perform deletion.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.9')
+        cctxt = self._prepare_call(topic=topic, version='1.9')
         return cctxt.call(context, 'destroy_node', node_id=node_id)
 
     def get_console_information(self, context, node_id, topic=None):
@@ -528,7 +657,7 @@ class ConductorAPI(object):
         :raises: InvalidParameterValue when the wrong driver info is specified.
         :raises: MissingParameterValue if a required parameter is missing
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.11')
+        cctxt = self._prepare_call(topic=topic, version='1.11')
         return cctxt.call(context, 'get_console_information', node_id=node_id)
 
     def set_console_mode(self, context, node_id, enabled, topic=None):
@@ -546,7 +675,7 @@ class ConductorAPI(object):
         :raises: NoFreeConductorWorker when there is no free worker to start
                  async task.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.11')
+        cctxt = self._prepare_call(topic=topic, version='1.11')
         return cctxt.call(context, 'set_console_mode', node_id=node_id,
                           enabled=enabled)
 
@@ -562,7 +691,7 @@ class ConductorAPI(object):
         :param topic: RPC topic. Defaults to self.topic.
         :returns: created port object.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.41')
+        cctxt = self._prepare_call(topic=topic, version='1.41')
         return cctxt.call(context, 'create_port', port_obj=port_obj)
 
     def update_port(self, context, port_obj, topic=None):
@@ -578,7 +707,7 @@ class ConductorAPI(object):
         :returns: updated port object, including all fields.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.13')
+        cctxt = self._prepare_call(topic=topic, version='1.13')
         return cctxt.call(context, 'update_port', port_obj=port_obj)
 
     def update_portgroup(self, context, portgroup_obj, topic=None):
@@ -595,7 +724,7 @@ class ConductorAPI(object):
         :returns: updated portgroup object, including all fields.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.33')
+        cctxt = self._prepare_call(topic=topic, version='1.33')
         return cctxt.call(context, 'update_portgroup',
                           portgroup_obj=portgroup_obj)
 
@@ -610,7 +739,7 @@ class ConductorAPI(object):
                  not exist.
         :raises: PortgroupNotEmpty if portgroup is not empty
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.33')
+        cctxt = self._prepare_call(topic=topic, version='1.33')
         return cctxt.call(context, 'destroy_portgroup', portgroup=portgroup)
 
     def get_driver_properties(self, context, driver_name, topic=None):
@@ -624,7 +753,7 @@ class ConductorAPI(object):
         :raises: DriverNotFound.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.16')
+        cctxt = self._prepare_call(topic=topic, version='1.16')
         return cctxt.call(context, 'get_driver_properties',
                           driver_name=driver_name)
 
@@ -649,7 +778,7 @@ class ConductorAPI(object):
                  specified or an invalid boot device is specified.
         :raises: MissingParameterValue if missing supplied info.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.17')
+        cctxt = self._prepare_call(topic=topic, version='1.17')
         return cctxt.call(context, 'set_boot_device', node_id=node_id,
                           device=device, persistent=persistent)
 
@@ -675,7 +804,7 @@ class ConductorAPI(object):
                 future boots or not, None if it is unknown.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.17')
+        cctxt = self._prepare_call(topic=topic, version='1.17')
         return cctxt.call(context, 'get_boot_device', node_id=node_id)
 
     def inject_nmi(self, context, node_id, topic=None):
@@ -695,7 +824,7 @@ class ConductorAPI(object):
         :raises: MissingParameterValue if missing supplied info.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.40')
+        cctxt = self._prepare_call(topic=topic, version='1.40')
         return cctxt.call(context, 'inject_nmi', node_id=node_id)
 
     def get_supported_boot_devices(self, context, node_id, topic=None):
@@ -716,7 +845,7 @@ class ConductorAPI(object):
                   in :mod:`ironic.common.boot_devices`.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.17')
+        cctxt = self._prepare_call(topic=topic, version='1.17')
         return cctxt.call(context, 'get_supported_boot_devices',
                           node_id=node_id)
 
@@ -741,7 +870,7 @@ class ConductorAPI(object):
         :raises: MissingParameterValue if missing supplied info.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.50')
+        cctxt = self._prepare_call(topic=topic, version='1.50')
         return cctxt.call(context, 'set_indicator_state', node_id=node_id,
                           component=component, indicator=indicator,
                           state=state)
@@ -767,7 +896,7 @@ class ConductorAPI(object):
             mod:`ironic.common.indicator_states`.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.50')
+        cctxt = self._prepare_call(topic=topic, version='1.50')
         return cctxt.call(context, 'get_indicator_state', node_id=node_id,
                           component=component, indicator=indicator)
 
@@ -799,7 +928,7 @@ class ConductorAPI(object):
                     }
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.50')
+        cctxt = self._prepare_call(topic=topic, version='1.50')
         return cctxt.call(context, 'get_supported_indicators', node_id=node_id,
                           component=component)
 
@@ -819,7 +948,7 @@ class ConductorAPI(object):
                  action to do in the current state.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.24')
+        cctxt = self._prepare_call(topic=topic, version='1.24')
         return cctxt.call(context, 'inspect_hardware', node_id=node_id)
 
     def destroy_port(self, context, port, topic=None):
@@ -832,7 +961,7 @@ class ConductorAPI(object):
         :raises: NodeNotFound if the node associated with the port does not
                  exist.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.25')
+        cctxt = self._prepare_call(topic=topic, version='1.25')
         return cctxt.call(context, 'destroy_port', port=port)
 
     def set_target_raid_config(self, context, node_id, target_raid_config,
@@ -854,7 +983,7 @@ class ConductorAPI(object):
             missing.
         :raises: NodeLocked if node is locked by another conductor.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.30')
+        cctxt = self._prepare_call(topic=topic, version='1.30')
         return cctxt.call(context, 'set_target_raid_config',
                           node_id=node_id,
                           target_raid_config=target_raid_config)
@@ -879,16 +1008,18 @@ class ConductorAPI(object):
         :returns: A dictionary containing the properties that can be mentioned
             for logical disks and a textual description for them.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.30')
+        cctxt = self._prepare_call(topic=topic, version='1.30')
         return cctxt.call(context, 'get_raid_logical_disk_properties',
                           driver_name=driver_name)
 
-    def do_node_clean(self, context, node_id, clean_steps, topic=None):
+    def do_node_clean(self, context, node_id, clean_steps,
+                      disable_ramdisk=None, topic=None):
         """Signal to conductor service to perform manual cleaning on a node.
 
         :param context: request context.
         :param node_id: node ID or UUID.
         :param clean_steps: a list of clean step dictionaries.
+        :param disable_ramdisk: Whether to skip booting ramdisk for cleaning.
         :param topic: RPC topic. Defaults to self.topic.
         :raises: InvalidParameterValue if validation of power driver interface
                  failed.
@@ -898,12 +1029,20 @@ class ConductorAPI(object):
         :raises: NoFreeConductorWorker when there is no free worker to start
                  async task.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.32')
+        # Avoid sending unset parameters to simplify upgrades.
+        params = {}
+        version = '1.32'
+        if disable_ramdisk is not None:
+            params['disable_ramdisk'] = disable_ramdisk
+            version = '1.53'
+
+        cctxt = self._prepare_call(topic=topic, version=version)
         return cctxt.call(context, 'do_node_clean',
-                          node_id=node_id, clean_steps=clean_steps)
+                          node_id=node_id, clean_steps=clean_steps, **params)
 
     def heartbeat(self, context, node_id, callback_url, agent_version,
-                  agent_token=None, agent_verify_ca=None, topic=None):
+                  agent_token=None, agent_verify_ca=None, agent_status=None,
+                  agent_status_message=None, topic=None):
         """Process a node heartbeat.
 
         :param context: request context.
@@ -913,20 +1052,27 @@ class ConductorAPI(object):
         :param agent_token: randomly generated validation token.
         :param agent_version: the version of the agent that is heartbeating
         :param agent_verify_ca: TLS certificate for the agent.
+        :param agent_status: The status of the agent that is heartbeating
+        :param agent_status_message: Optional message describing the agent
+            status
         :raises: InvalidParameterValue if an invalid agent token is received.
         """
         new_kws = {}
         version = '1.34'
-        if self.client.can_send_version('1.42'):
+        if self._can_send_version('1.42'):
             version = '1.42'
             new_kws['agent_version'] = agent_version
-        if self.client.can_send_version('1.49'):
+        if self._can_send_version('1.49'):
             version = '1.49'
             new_kws['agent_token'] = agent_token
-        if self.client.can_send_version('1.51'):
+        if self._can_send_version('1.51'):
             version = '1.51'
             new_kws['agent_verify_ca'] = agent_verify_ca
-        cctxt = self.client.prepare(topic=topic or self.topic, version=version)
+        if self._can_send_version('1.54'):
+            version = '1.54'
+            new_kws['agent_status'] = agent_status
+            new_kws['agent_status_message'] = agent_status_message
+        cctxt = self._prepare_call(topic=topic, version=version)
         return cctxt.call(context, 'heartbeat', node_id=node_id,
                           callback_url=callback_url, **new_kws)
 
@@ -948,11 +1094,11 @@ class ConductorAPI(object):
         :returns: The result of the action method, which may (or may not)
             be an instance of the implementing VersionedObject class.
         """
-        if not self.client.can_send_version('1.31'):
+        if not self._can_send_version('1.31'):
             raise NotImplementedError(_('Incompatible conductor version - '
                                         'please upgrade ironic-conductor '
                                         'first'))
-        cctxt = self.client.prepare(topic=self.topic, version='1.31')
+        cctxt = self._prepare_call(topic=self.topic, version='1.31')
         return cctxt.call(context, 'object_class_action_versions',
                           objname=objname, objmethod=objmethod,
                           object_versions=object_versions,
@@ -974,11 +1120,11 @@ class ConductorAPI(object):
         :returns: A tuple with the updates made to the object and
             the result of the action method
         """
-        if not self.client.can_send_version('1.31'):
+        if not self._can_send_version('1.31'):
             raise NotImplementedError(_('Incompatible conductor version - '
                                         'please upgrade ironic-conductor '
                                         'first'))
-        cctxt = self.client.prepare(topic=self.topic, version='1.31')
+        cctxt = self._prepare_call(topic=self.topic, version='1.31')
         return cctxt.call(context, 'object_action', objinst=objinst,
                           objmethod=objmethod, args=args, kwargs=kwargs)
 
@@ -999,11 +1145,11 @@ class ConductorAPI(object):
             upgrade
         :returns: The downgraded instance of objinst
         """
-        if not self.client.can_send_version('1.31'):
+        if not self._can_send_version('1.31'):
             raise NotImplementedError(_('Incompatible conductor version - '
                                         'please upgrade ironic-conductor '
                                         'first'))
-        cctxt = self.client.prepare(topic=self.topic, version='1.31')
+        cctxt = self._prepare_call(topic=self.topic, version='1.31')
         return cctxt.call(context, 'object_backport_versions', objinst=objinst,
                           object_versions=object_versions)
 
@@ -1022,7 +1168,7 @@ class ConductorAPI(object):
         :raises: VolumeConnectorNotFound if the volume connector cannot be
                  found
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.35')
+        cctxt = self._prepare_call(topic=topic, version='1.35')
         return cctxt.call(context, 'destroy_volume_connector',
                           connector=connector)
 
@@ -1049,7 +1195,7 @@ class ConductorAPI(object):
         :returns: updated volume connector object, including all fields.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.35')
+        cctxt = self._prepare_call(topic=topic, version='1.35')
         return cctxt.call(context, 'update_volume_connector',
                           connector=connector)
 
@@ -1064,7 +1210,7 @@ class ConductorAPI(object):
                  not exist
         :raises: VolumeTargetNotFound if the volume target cannot be found
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.37')
+        cctxt = self._prepare_call(topic=topic, version='1.37')
         return cctxt.call(context, 'destroy_volume_target',
                           target=target)
 
@@ -1089,7 +1235,7 @@ class ConductorAPI(object):
         :returns: updated volume target object, including all fields
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.37')
+        cctxt = self._prepare_call(topic=topic, version='1.37')
         return cctxt.call(context, 'update_volume_target',
                           target=target)
 
@@ -1107,7 +1253,7 @@ class ConductorAPI(object):
         :raises: InvalidParameterValue, if a parameter that's required for
             VIF attach is wrong/missing.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.38')
+        cctxt = self._prepare_call(topic=topic, version='1.38')
         return cctxt.call(context, 'vif_attach', node_id=node_id,
                           vif_info=vif_info)
 
@@ -1123,7 +1269,7 @@ class ConductorAPI(object):
         :raises: InvalidParameterValue, if a parameter that's required for
             VIF detach is wrong/missing.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.38')
+        cctxt = self._prepare_call(topic=topic, version='1.38')
         return cctxt.call(context, 'vif_detach', node_id=node_id,
                           vif_id=vif_id)
 
@@ -1139,7 +1285,7 @@ class ConductorAPI(object):
         :raises: InvalidParameterValue, if a parameter that's required for
             VIF list is wrong/missing.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.38')
+        cctxt = self._prepare_call(topic=topic, version='1.38')
         return cctxt.call(context, 'vif_list', node_id=node_id)
 
     def do_node_rescue(self, context, node_id, rescue_password, topic=None):
@@ -1158,7 +1304,7 @@ class ConductorAPI(object):
         state before this method is called.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.43')
+        cctxt = self._prepare_call(topic=topic, version='1.43')
         return cctxt.call(context, 'do_node_rescue', node_id=node_id,
                           rescue_password=rescue_password)
 
@@ -1176,7 +1322,7 @@ class ConductorAPI(object):
         state before this method is called.
 
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.43')
+        cctxt = self._prepare_call(topic=topic, version='1.43')
         return cctxt.call(context, 'do_node_unrescue', node_id=node_id)
 
     def add_node_traits(self, context, node_id, traits, replace=False,
@@ -1193,7 +1339,7 @@ class ConductorAPI(object):
         :raises: NodeLocked if node is locked by another conductor.
         :raises: NodeNotFound if the node does not exist.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.44')
+        cctxt = self._prepare_call(topic=topic, version='1.44')
         return cctxt.call(context, 'add_node_traits', node_id=node_id,
                           traits=traits, replace=replace)
 
@@ -1209,7 +1355,7 @@ class ConductorAPI(object):
         :raises: NodeNotFound if the node does not exist.
         :raises: NodeTraitNotFound if one of the traits is not found.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.44')
+        cctxt = self._prepare_call(topic=topic, version='1.44')
         return cctxt.call(context, 'remove_node_traits', node_id=node_id,
                           traits=traits)
 
@@ -1220,7 +1366,7 @@ class ConductorAPI(object):
         :param allocation: an allocation object.
         :param topic: RPC topic. Defaults to self.topic.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.48')
+        cctxt = self._prepare_call(topic=topic, version='1.48')
         return cctxt.call(context, 'create_allocation', allocation=allocation)
 
     def destroy_allocation(self, context, allocation, topic=None):
@@ -1232,7 +1378,7 @@ class ConductorAPI(object):
         :raises: InvalidState if the associated node is in the wrong provision
             state to perform deallocation.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.48')
+        cctxt = self._prepare_call(topic=topic, version='1.48')
         return cctxt.call(context, 'destroy_allocation', allocation=allocation)
 
     def get_node_with_token(self, context, node_id, topic=None):
@@ -1245,5 +1391,5 @@ class ConductorAPI(object):
 
         :returns: A Node object with agent token.
         """
-        cctxt = self.client.prepare(topic=topic or self.topic, version='1.49')
+        cctxt = self._prepare_call(topic=topic, version='1.49')
         return cctxt.call(context, 'get_node_with_token', node_id=node_id)
