@@ -102,13 +102,14 @@ raised in the background thread.):
 
 import copy
 import functools
+import traceback
 
 import futurist
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import timeutils
-import retrying
+import tenacity
 
 from ironic.common import driver_factory
 from ironic.common import exception
@@ -141,6 +142,10 @@ def require_exclusive_lock(f):
         else:
             task = args[0]
         if task.shared:
+            LOG.error("Callable %(func)s expected an exclusive lock, got "
+                      "a shared task for node %(node)s. Traceback:\n%(tb)s",
+                      {'func': f.__qualname__, 'node': task.node.uuid,
+                       'tb': ''.join(traceback.format_stack())})
             raise exception.ExclusiveLockRequired()
         # NOTE(lintan): This is a workaround to set the context of async tasks,
         # which should contain an exclusive lock.
@@ -170,7 +175,7 @@ class TaskManager(object):
     """
 
     def __init__(self, context, node_id, shared=False,
-                 purpose='unspecified action', retry=True,
+                 purpose='unspecified action', retry=True, patient=False,
                  load_driver=True):
         """Create a new TaskManager.
 
@@ -185,6 +190,12 @@ class TaskManager(object):
                        lock. Default: False.
         :param purpose: human-readable purpose to put to debug logs.
         :param retry: whether to retry locking if it fails. Default: True.
+        :param patient: whether to indefinitely retry locking if it fails.
+                        Set this to True if there is an operation that does not
+                        have any fallback or built-in retry mechanism, such as
+                        finalizing a state transition during deploy/clean.
+                        The default retry behavior is to retry a configured
+                        number of times and then give up. Default: False.
         :param load_driver: whether to load the ``driver`` object. Set this to
                             False if loading the driver is undesired or
                             impossible.
@@ -200,9 +211,14 @@ class TaskManager(object):
 
         self.context = context
         self._node = None
+        self._ports = None
+        self._portgroups = None
+        self._volume_connectors = None
+        self._volume_targets = None
         self.node_id = node_id
         self.shared = shared
         self._retry = retry
+        self._patient = patient
 
         self.fsm = states.machine.copy()
         self._purpose = purpose
@@ -226,13 +242,6 @@ class TaskManager(object):
                 self._debug_timer.restart()
                 self.node = node
 
-            self.ports = objects.Port.list_by_node_id(context, self.node.id)
-            self.portgroups = objects.Portgroup.list_by_node_id(context,
-                                                                self.node.id)
-            self.volume_connectors = objects.VolumeConnector.list_by_node_id(
-                context, self.node.id)
-            self.volume_targets = objects.VolumeTarget.list_by_node_id(
-                context, self.node.id)
             if load_driver:
                 self.driver = driver_factory.build_driver_for_task(self)
             else:
@@ -253,6 +262,67 @@ class TaskManager(object):
             self.fsm.initialize(start_state=self.node.provision_state,
                                 target_state=self.node.target_provision_state)
 
+    @property
+    def ports(self):
+        try:
+            if self._ports is None:
+                self._ports = objects.Port.list_by_node_id(self.context,
+                                                           self.node.id)
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                self.release_resources()
+        return self._ports
+
+    @ports.setter
+    def ports(self, ports):
+        self._ports = ports
+
+    @property
+    def portgroups(self):
+        try:
+            if self._portgroups is None:
+                self._portgroups = objects.Portgroup.list_by_node_id(
+                    self.context, self.node.id)
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                self.release_resources()
+        return self._portgroups
+
+    @portgroups.setter
+    def portgroups(self, portgroups):
+        self._portgroups = portgroups
+
+    @property
+    def volume_connectors(self):
+        try:
+            if self._volume_connectors is None:
+                self._volume_connectors = \
+                    objects.VolumeConnector.list_by_node_id(
+                        self.context, self.node.id)
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                self.release_resources()
+        return self._volume_connectors
+
+    @volume_connectors.setter
+    def volume_connectors(self, volume_connectors):
+        self._volume_connectors = volume_connectors
+
+    @property
+    def volume_targets(self):
+        try:
+            if self._volume_targets is None:
+                self._volume_targets = objects.VolumeTarget.list_by_node_id(
+                    self.context, self.node.id)
+        except Exception:
+            with excutils.save_and_reraise_exception():
+                self.release_resources()
+        return self._volume_targets
+
+    @volume_targets.setter
+    def volume_targets(self, volume_targets):
+        self._volume_targets = volume_targets
+
     def load_driver(self):
         if self.driver is None:
             self.driver = driver_factory.build_driver_for_task(self)
@@ -260,18 +330,22 @@ class TaskManager(object):
     def _lock(self):
         self._debug_timer.restart()
 
-        if self._retry:
-            attempts = CONF.conductor.node_locked_retry_attempts
+        if self._patient:
+            stop_after = tenacity.stop_never
+        elif self._retry:
+            stop_after = tenacity.stop_after_attempt(
+                CONF.conductor.node_locked_retry_attempts)
         else:
-            attempts = 1
+            stop_after = tenacity.stop_after_attempt(1)
 
         # NodeLocked exceptions can be annoying. Let's try to alleviate
-        # some of that pain by retrying our lock attempts. The retrying
-        # module expects a wait_fixed value in milliseconds.
-        @retrying.retry(
-            retry_on_exception=lambda e: isinstance(e, exception.NodeLocked),
-            stop_max_attempt_number=attempts,
-            wait_fixed=CONF.conductor.node_locked_retry_interval * 1000)
+        # some of that pain by retrying our lock attempts.
+        @tenacity.retry(
+            retry=tenacity.retry_if_exception_type(exception.NodeLocked),
+            stop=stop_after,
+            wait=tenacity.wait_fixed(
+                CONF.conductor.node_locked_retry_interval),
+            reraise=True)
         def reserve_node():
             self.node = objects.Node.reserve(self.context, CONF.host,
                                              self.node_id)
@@ -291,8 +365,8 @@ class TaskManager(object):
         when provided with one.
 
         :param purpose: optionally change the purpose of the lock
-        :param retry: whether to retry locking if it fails, the class-level
-            value is used by default
+        :param retry: whether to retry locking if it fails, the
+            class-level value is used by default
         :raises: NodeLocked if an exclusive lock remains on the node after
                             "node_locked_retry_attempts"
         """
@@ -530,6 +604,14 @@ class TaskManager(object):
             # emitted at __exit__().
             self._saved_node = self.node
 
+    def resume_cleaning(self):
+        """A helper to resume cleaning with the right target state."""
+        if self.node.target_provision_state == states.MANAGEABLE:
+            target_state = states.MANAGEABLE
+        else:
+            target_state = None
+        self.process_event('resume', target_state=target_state)
+
     def __enter__(self):
         return self
 
