@@ -16,6 +16,7 @@
 #    under the License.
 
 from oslo_config import cfg
+from oslo_config import types
 
 from ironic.common.i18n import _
 
@@ -58,6 +59,10 @@ opts = [
                min=0,
                help=_('Interval between checks of orphaned allocations, '
                       'in seconds. Set to 0 to disable checks.')),
+    cfg.IntOpt('cache_clean_up_interval',
+               default=3600, min=0,
+               help=_('Interval between cleaning up image caches, in seconds. '
+                      'Set to 0 to disable periodic clean-up.')),
     cfg.IntOpt('deploy_callback_timeout',
                default=1800,
                min=0,
@@ -270,6 +275,115 @@ opts = [
                       'will be used by ironic when building UEFI-bootable ISO '
                       'out of kernel and ramdisk. Required for UEFI boot from '
                       'partition images.')),
+    cfg.MultiOpt('clean_step_priority_override',
+                 item_type=types.Dict(),
+                 default={},
+                 help=_('Priority to run automated clean steps for both '
+                        'in-band and out of band clean steps, provided in '
+                        'interface.step_name:priority format, e.g. '
+                        'deploy.erase_devices_metadata:123. The option can '
+                        'be specified multiple times to define priorities '
+                        'for multiple steps. If set to 0, this specific step '
+                        'will not run during cleaning. If unset for an '
+                        'inband clean step, will use the priority set in the '
+                        'ramdisk.')),
+    cfg.BoolOpt('node_history',
+                default=True,
+                mutable=True,
+                help=_('Boolean value, default True, if node event history '
+                       'is to be recorded. Errors and other noteworthy '
+                       'events in relation to a node are journaled to a '
+                       'database table which incurs some additional load. '
+                       'A periodic task does periodically remove entries '
+                       'from the database. Please note, if this is disabled, '
+                       'the conductor will continue to purge entries as '
+                       'long as [conductor]node_history_cleanup_batch_count '
+                       'is not 0.')),
+    cfg.IntOpt('node_history_max_entries',
+               default=300,
+               min=0,
+               mutable=True,
+               help=_('Maximum number of history entries which will be stored '
+                      'in the database per node. Default is 300. This setting '
+                      'excludes the minimum number of days retained using the '
+                      '[conductor]node_history_minimum_days setting.')),
+    cfg.IntOpt('node_history_cleanup_interval',
+               min=0,
+               default=86400,
+               mutable=False,
+               help=_('Interval in seconds at which node history entries '
+                      'can be cleaned up in the database. Setting to 0 '
+                      'disables the periodic task. Defaults to once a day, '
+                      'or 86400 seconds.')),
+    cfg.IntOpt('node_history_cleanup_batch_count',
+               min=0,
+               default=1000,
+               mutable=False,
+               help=_('The target number of node history records to purge '
+                      'from the database when performing clean-up. '
+                      'Deletes are performed by node, and a node with excess '
+                      'records for a node will still be deleted. '
+                      'Defaults to 1000. Operators who find node history '
+                      'building up may wish to '
+                      'lower this threshold and decrease the time between '
+                      'cleanup operations using the '
+                      '``node_history_cleanup_interval`` setting.')),
+    cfg.IntOpt('node_history_minimum_days',
+               min=0,
+               default=0,
+               mutable=True,
+               help=_('The minimum number of days to explicitly keep on '
+                      'hand in the database history entries for nodes. '
+                      'This is exclusive from the [conductor]'
+                      'node_history_max_entries setting as users of '
+                      'this setting are anticipated to need to retain '
+                      'history by policy.')),
+    cfg.MultiOpt('verify_step_priority_override',
+                 item_type=types.Dict(),
+                 default={},
+                 mutable=True,
+                 help=_('Priority to run automated verify steps '
+                        'provided in interface.step_name:priority format,'
+                        'e.g. management.clear_job_queue:123. The option can '
+                        'be specified multiple times to define priorities '
+                        'for multiple steps. If set to 0, this specific step '
+                        'will not run during verification. ')),
+    cfg.BoolOpt('automatic_lessee',
+                default=False,
+                mutable=True,
+                help=_('If the conductor should record the Project ID '
+                       'indicated by Keystone for a requested deployment. '
+                       'Allows rights to be granted to directly access the '
+                       'deployed node as a lessee within the RBAC security '
+                       'model. The conductor does *not* record this value '
+                       'otherwise, and this information is not backfilled '
+                       'for prior instances which have been deployed.')),
+    cfg.IntOpt('max_concurrent_deploy',
+               default=250,
+               min=1,
+               mutable=True,
+               help=_('The maximum number of concurrent nodes in deployment '
+                      'which are permitted in this Ironic system. '
+                      'If this limit is reached, new requests will be '
+                      'rejected until the number of deployments in progress '
+                      'is lower than this maximum. As this is a security '
+                      'mechanism requests are not queued, and this setting '
+                      'is a global setting applying to all requests this '
+                      'conductor receives, regardless of access rights. '
+                      'The concurrent deployment limit cannot be disabled.')),
+    cfg.IntOpt('max_concurrent_clean',
+               default=50,
+               min=1,
+               mutable=True,
+               help=_('The maximum number of concurrent nodes in cleaning '
+                      'which are permitted in this Ironic system. '
+                      'If this limit is reached, new requests will be '
+                      'rejected until the number of nodes in cleaning '
+                      'is lower than this maximum. As this is a security '
+                      'mechanism requests are not queued, and this setting '
+                      'is a global setting applying to all requests this '
+                      'conductor receives, regardless of access rights. '
+                      'The concurrent clean limit cannot be disabled.')),
 ]
 
 
