@@ -85,7 +85,7 @@ api_opts = [
 
 driver_opts = [
     cfg.ListOpt('enabled_hardware_types',
-                default=['ipmi'],
+                default=['ipmi', 'redfish'],
                 help=_('Specify the list of hardware types to load during '
                        'service initialization. Missing hardware types, or '
                        'hardware types which fail to initialize, will prevent '
@@ -96,12 +96,12 @@ driver_opts = [
                        'system may be found by enumerating the '
                        '"ironic.hardware.types" entrypoint.')),
     cfg.ListOpt('enabled_bios_interfaces',
-                default=['no-bios'],
+                default=['no-bios', 'redfish'],
                 help=_ENABLED_IFACE_HELP.format('bios')),
     cfg.StrOpt('default_bios_interface',
                help=_DEFAULT_IFACE_HELP.format('bios')),
     cfg.ListOpt('enabled_boot_interfaces',
-                default=['pxe'],
+                default=['ipxe', 'pxe', 'redfish-virtual-media'],
                 help=_ENABLED_IFACE_HELP.format('boot')),
     cfg.StrOpt('default_boot_interface',
                help=_DEFAULT_IFACE_HELP.format('boot')),
@@ -111,17 +111,17 @@ driver_opts = [
     cfg.StrOpt('default_console_interface',
                help=_DEFAULT_IFACE_HELP.format('console')),
     cfg.ListOpt('enabled_deploy_interfaces',
-                default=['direct'],
+                default=['direct', 'ramdisk'],
                 help=_ENABLED_IFACE_HELP.format('deploy')),
     cfg.StrOpt('default_deploy_interface',
                help=_DEFAULT_IFACE_HELP.format('deploy')),
     cfg.ListOpt('enabled_inspect_interfaces',
-                default=['no-inspect'],
+                default=['no-inspect', 'redfish'],
                 help=_ENABLED_IFACE_HELP.format('inspect')),
     cfg.StrOpt('default_inspect_interface',
                help=_DEFAULT_IFACE_HELP.format('inspect')),
     cfg.ListOpt('enabled_management_interfaces',
-                default=['ipmitool'],
+                default=None,  # automatically calculate
                 help=_ENABLED_IFACE_HELP.format('management')),
     cfg.StrOpt('default_management_interface',
                help=_DEFAULT_IFACE_HELP.format('management')),
@@ -131,12 +131,12 @@ driver_opts = [
     cfg.StrOpt('default_network_interface',
                help=_DEFAULT_IFACE_HELP.format('network')),
     cfg.ListOpt('enabled_power_interfaces',
-                default=['ipmitool'],
+                default=None,  # automatically calculate
                 help=_ENABLED_IFACE_HELP.format('power')),
     cfg.StrOpt('default_power_interface',
                help=_DEFAULT_IFACE_HELP.format('power')),
     cfg.ListOpt('enabled_raid_interfaces',
-                default=['agent', 'no-raid'],
+                default=['agent', 'no-raid', 'redfish'],
                 help=_ENABLED_IFACE_HELP.format('raid')),
     cfg.StrOpt('default_raid_interface',
                help=_DEFAULT_IFACE_HELP.format('raid')),
@@ -152,7 +152,7 @@ driver_opts = [
                default='noop',
                help=_DEFAULT_IFACE_HELP.format('storage')),
     cfg.ListOpt('enabled_vendor_interfaces',
-                default=['ipmitool', 'no-vendor'],
+                default=['ipmitool', 'redfish', 'no-vendor'],
                 help=_ENABLED_IFACE_HELP.format('vendor')),
     cfg.StrOpt('default_vendor_interface',
                help=_DEFAULT_IFACE_HELP.format('vendor')),
@@ -201,6 +201,13 @@ image_opts = [
                 mutable=True,
                 help=_('If True, convert backing images to "raw" disk image '
                        'format.')),
+    cfg.FloatOpt('raw_image_growth_factor',
+                 default=2.0,
+                 min=1.0,
+                 help=_('The scale factor used for estimating the size of a '
+                        'raw image converted from compact image '
+                        'formats such as QCOW2. '
+                        'Default is 2.0, must be greater than 1.0.')),
     cfg.StrOpt('isolinux_bin',
                default='/usr/lib/syslinux/isolinux.bin',
                help=_('Path to isolinux binary file.')),
@@ -211,7 +218,11 @@ image_opts = [
     cfg.StrOpt('grub_config_path',
                default='/boot/grub/grub.cfg',
                help=_('GRUB2 configuration file location on the UEFI ISO '
-                      'images produced by ironic.')),
+                      'images produced by ironic. The default value is '
+                      'usually incorrect and should not be relied on. '
+                      'If you use a GRUB2 image from a certain distribution, '
+                      'use a distribution-specific path here, e.g. '
+                      'EFI/ubuntu/grub.cfg')),
     cfg.StrOpt('grub_config_template',
                default=os.path.join('$pybasedir',
                                     'common/grub_conf.template'),
@@ -238,10 +249,16 @@ image_opts = [
 
 img_cache_opts = [
     cfg.BoolOpt('parallel_image_downloads',
-                default=False,
+                default=True,
                 mutable=True,
                 help=_('Run image downloads and raw format conversions in '
-                       'parallel.')),
+                       'parallel.'),
+                deprecated_for_removal=True,
+                deprecated_reason=_('Use image_download_concurrency')),
+    cfg.IntOpt('image_download_concurrency',
+               default=20, min=1,
+               help=_('How many image downloads and raw format conversions '
+                      'to run in parallel. Only affects image caches.')),
 ]
 
 netconf_opts = [
@@ -345,9 +362,36 @@ service_opts = [
     cfg.StrOpt('rpc_transport',
                default='oslo',
                choices=[('oslo', _('use oslo.messaging transport')),
-                        ('json-rpc', _('use JSON RPC transport'))],
+                        ('json-rpc', _('use JSON RPC transport')),
+                        ('none', _('No RPC, only use local conductor'))],
                help=_('Which RPC transport implementation to use between '
                       'conductor and API services')),
+    cfg.BoolOpt('minimum_memory_warning_only',
+                mutable=True,
+                default=False,
+                help=_('Setting to govern if Ironic should only warn instead '
+                       'of attempting to hold back the request in order to '
+                       'prevent the exhaustion of system memory.')),
+    cfg.IntOpt('minimum_required_memory',
+               mutable=True,
+               default=1024,
+               help=_('Minimum memory in MiB for the system to have '
+                      'available prior to starting a memory intensive '
+                      'process on the conductor.')),
+    cfg.IntOpt('minimum_memory_wait_time',
+               mutable=True,
+               default=15,
+               help=_('Seconds to wait between retries for free memory '
+                      'before launching the process. This, combined with '
+                      '``memory_wait_retries`` allows the conductor to '
+                      'determine how long we should attempt to directly '
+                      'retry.')),
+    cfg.IntOpt('minimum_memory_wait_retries',
+               mutable=True,
+               default=6,
+               help=_('Number of retries to hold onto the worker before '
+                      'failing or returning the thread to the pool if '
+                      'the conductor can automatically retry.')),
 ]
 
 utils_opts = [
@@ -362,7 +406,7 @@ utils_opts = [
                       'dir.')),
 ]
 
-cert_verify_opts = [
+webserver_opts = [
     cfg.StrOpt('webserver_verify_ca',
                default='True',
                mutable=True,
@@ -380,9 +424,35 @@ cert_verify_opts = [
                       'is set to True i.e the certificates present in the '
                       'standard path are used for SSL verification.'
                       'Defaults to True.')),
+    cfg.IntOpt('webserver_connection_timeout',
+               default=60,
+               help=_('Connection timeout when accessing remote web servers '
+                      'with images.')),
 ]
 
 
+def list_opts():
+    _default_opt_lists = [
+        api_opts,
+        driver_opts,
+        exc_log_opts,
+        hash_opts,
+        image_opts,
+        img_cache_opts,
+        netconf_opts,
+        notification_opts,
+        path_opts,
+        portgroup_opts,
+        service_opts,
+        utils_opts,
+        webserver_opts,
+    ]
+    full_opt_list = []
+    for options in _default_opt_lists:
+        full_opt_list.extend(options)
+    return full_opt_list
+
+
 def register_opts(conf):
     conf.register_opts(api_opts)
     conf.register_opts(driver_opts)
@@ -396,4 +466,4 @@ def register_opts(conf):
     conf.register_opts(portgroup_opts)
     conf.register_opts(service_opts)
     conf.register_opts(utils_opts)
-    conf.register_opts(cert_verify_opts)
+    conf.register_opts(webserver_opts)
