@@ -14,11 +14,12 @@
 #    under the License.
 
 from oslo_log import log as logging
+from oslo_utils import excutils
 
-from ironic.common import boot_modes
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import utils as common_utils
+from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers import utils as driver_utils
@@ -66,6 +67,7 @@ def _set_boot_mode_on_bm(task, ironic_boot_mode, fail_if_unsupported=False):
         LOG.info("Baremetal node boot mode is set to boot "
                  "mode %(boot_mode)s",
                  {'uuid': task.node.uuid, 'boot_mode': ironic_boot_mode})
+        manager_utils.node_cache_boot_mode(task)
 
 
 def sync_boot_mode(task):
@@ -108,10 +110,8 @@ def sync_boot_mode(task):
     # the new boot mode may be set in 'driver_internal_info/deploy_boot_mode'
 
     if not ironic_boot_mode and not bm_boot_mode:
-        driver_internal_info = node.driver_internal_info
         default_boot_mode = CONF.deploy.default_boot_mode
-        driver_internal_info['deploy_boot_mode'] = default_boot_mode
-        node.driver_internal_info = driver_internal_info
+        node.set_driver_internal_info('deploy_boot_mode', default_boot_mode)
         node.save()
 
         LOG.debug("Ironic node %(uuid)s boot mode will be set to default "
@@ -121,9 +121,7 @@ def sync_boot_mode(task):
         _set_boot_mode_on_bm(task, default_boot_mode)
 
     elif not ironic_boot_mode and bm_boot_mode:
-        driver_internal_info = node.driver_internal_info
-        driver_internal_info['deploy_boot_mode'] = bm_boot_mode
-        node.driver_internal_info = driver_internal_info
+        node.set_driver_internal_info('deploy_boot_mode', bm_boot_mode)
         node.save()
 
         LOG.debug("Ironic node %(uuid)s boot mode is set to boot mode "
@@ -170,30 +168,12 @@ def is_secure_boot_requested(node):
     return sec_boot == 'true'
 
 
-def is_trusted_boot_requested(node):
-    """Returns True if trusted_boot is requested for deploy.
-
-    This method checks instance property for trusted_boot and returns True
-    if it is requested.
-
-    :param node: a single Node.
-    :raises: InvalidParameterValue if the capabilities string is not a
-             dictionary or is malformed.
-    :returns: True if trusted_boot is requested.
-    """
-
-    capabilities = common_utils.parse_instance_info_capabilities(node)
-    trusted_boot = capabilities.get('trusted_boot', 'false').lower()
-
-    return trusted_boot == 'true'
-
-
 def get_boot_mode_for_deploy(node):
     """Returns the boot mode that would be used for deploy.
 
     This method returns boot mode to be used for deploy.
-    It returns 'uefi' if 'secure_boot' is set to 'true' or returns 'bios' if
-    'trusted_boot' is set to 'true' in 'instance_info/capabilities' of node.
+    It returns 'uefi' if 'secure_boot' is set to 'true' in
+    'instance_info/capabilities' of node.
     Otherwise it returns value of 'boot_mode' in 'properties/capabilities'
     of node if set. If that is not set, it returns boot mode in
     'internal_driver_info/deploy_boot_mode' for the node.
@@ -213,36 +193,38 @@ def get_boot_mode_for_deploy(node):
         LOG.debug('Deploy boot mode is uefi for %s.', node.uuid)
         return 'uefi'
 
-    if is_trusted_boot_requested(node):
-        # TODO(lintan) Trusted boot also supports uefi, but at the moment,
-        # it should only boot with bios.
-        LOG.debug('Deploy boot mode is bios for %s.', node.uuid)
-        return 'bios'
-
     # NOTE(etingof):
     # The search for a boot mode should be in the priority order:
     #
-    # 1) instance_info
-    # 2) properties.capabilities
-    # 3) driver_internal_info
+    # 1) instance_info.capabilities
+    # 2) instance_info.deploy_boot_mode (deprecated in Wallaby)
+    # 3) properties.capabilities
+    # 4) driver_internal_info.deploy_boot_mode (internal)
     #
     # Because:
     #
-    # (1) can be deleted before teardown
-    # (3) will never be touched if node properties/capabilities
+    # (1) and (2) are deleted during teardown
+    # (4) will never be touched if node properties/capabilities
     #     are still present.
-    # (2) becomes operational default as the last resort
-
-    instance_info = node.instance_info
+    # (3) becomes operational default as the last resort
 
+    inst_boot_mode = (
+        common_utils.parse_instance_info_capabilities(node).get('boot_mode')
+    )
     cap_boot_mode = driver_utils.get_node_capability(node, 'boot_mode')
 
-    boot_mode = instance_info.get('deploy_boot_mode')
-    if boot_mode is None:
-        boot_mode = cap_boot_mode
-        if cap_boot_mode is None:
-            driver_internal_info = node.driver_internal_info
-            boot_mode = driver_internal_info.get('deploy_boot_mode')
+    old_boot_mode = node.instance_info.get('deploy_boot_mode')
+    if old_boot_mode:
+        LOG.warning('Using instance_info/deploy_boot_mode is deprecated, '
+                    'please use instance_info/capabilities with boot mode '
+                    'for node %s', node.uuid)
+
+    boot_mode = (
+        inst_boot_mode
+        or old_boot_mode
+        or cap_boot_mode
+        or node.driver_internal_info.get('deploy_boot_mode')
+    )
 
     if not boot_mode:
         return
@@ -282,17 +264,56 @@ def get_boot_mode(node):
     boot_mode = get_boot_mode_for_deploy(node)
     if boot_mode:
         return boot_mode
-    # TODO(hshiina): The default boot mode will be changed to UEFI.
-    global warn_about_default_boot_mode
-    if (not warn_about_default_boot_mode
-            and CONF.deploy.default_boot_mode == boot_modes.LEGACY_BIOS):
-        warn_about_default_boot_mode = True
-        LOG.warning('Boot mode is not configured for node %(node_uuid)s '
-                    'explicitly. The default boot mode is "%(bios)s", but, '
-                    'the default will be changed to "%(uefi)s" in the future. '
-                    'It is recommended to set the boot option into '
-                    'properties/capabilities/boot_mode for all nodes.',
-                    {'node_uuid': node.uuid,
-                     'bios': boot_modes.LEGACY_BIOS,
-                     'uefi': boot_modes.UEFI})
     return CONF.deploy.default_boot_mode
+
+
+@task_manager.require_exclusive_lock
+def configure_secure_boot_if_needed(task):
+    """Configures secure boot if it has been requested for the node."""
+    if not is_secure_boot_requested(task.node):
+        return
+
+    try:
+        task.driver.management.set_secure_boot_state(task, True)
+    except exception.UnsupportedDriverExtension:
+        # TODO(dtantsur): make a failure in Xena
+        LOG.warning('Secure boot was requested for node %(node)s but its '
+                    'management interface %(driver)s does not support it. '
+                    'This warning will become an error in a future release.',
+                    {'node': task.node.uuid,
+                     'driver': task.node.get_interface('management')})
+    except Exception as exc:
+        with excutils.save_and_reraise_exception():
+            LOG.error('Failed to configure secure boot for node %(node)s: '
+                      '%(error)s',
+                      {'node': task.node.uuid, 'error': exc},
+                      exc_info=not isinstance(exc, exception.IronicException))
+    else:
+        LOG.info('Secure boot has been enabled for node %s', task.node.uuid)
+        manager_utils.node_cache_boot_mode(task)
+
+
+@task_manager.require_exclusive_lock
+def deconfigure_secure_boot_if_needed(task):
+    """Deconfigures secure boot if it has been requested for the node."""
+    if not is_secure_boot_requested(task.node):
+        return
+
+    try:
+        task.driver.management.set_secure_boot_state(task, False)
+    except exception.UnsupportedDriverExtension:
+        # NOTE(dtantsur): don't make it a hard failure to allow tearing down
+        # misconfigured nodes.
+        LOG.debug('Secure boot was requested for node %(node)s but its '
+                  'management interface %(driver)s does not support it.',
+                  {'node': task.node.uuid,
+                   'driver': task.node.get_interface('management')})
+    except Exception as exc:
+        with excutils.save_and_reraise_exception():
+            LOG.error('Failed to deconfigure secure boot for node %(node)s: '
+                      '%(error)s',
+                      {'node': task.node.uuid, 'error': exc},
+                      exc_info=not isinstance(exc, exception.IronicException))
+    else:
+        LOG.info('Secure boot has been disabled for node %s', task.node.uuid)
+        manager_utils.node_cache_boot_mode(task)
