@@ -55,10 +55,8 @@ LOG = logging.getLogger(__name__)
 METRICS = metrics_utils.get_metrics_logger(__name__)
 
 SUPPORTED_CAPABILITIES = {
-    'boot_option': ('local', 'netboot', 'ramdisk'),
     'boot_mode': ('bios', 'uefi'),
     'secure_boot': ('true', 'false'),
-    'trusted_boot': ('true', 'false'),
     'disk_label': ('msdos', 'gpt'),
 }
 
@@ -130,16 +128,16 @@ def _replace_root_uuid(path, root_uuid):
 
 
 def _replace_boot_line(path, boot_mode, is_whole_disk_image,
-                       trusted_boot=False, iscsi_boot=False,
-                       ramdisk_boot=False, ipxe_enabled=False):
+                       iscsi_boot=False, ramdisk_boot=False,
+                       ipxe_enabled=False, anaconda_boot=False):
     if is_whole_disk_image:
         boot_disk_type = 'boot_whole_disk'
-    elif trusted_boot:
-        boot_disk_type = 'trusted_boot'
     elif iscsi_boot:
         boot_disk_type = 'boot_iscsi'
     elif ramdisk_boot:
         boot_disk_type = 'boot_ramdisk'
+    elif anaconda_boot:
+        boot_disk_type = 'boot_anaconda'
     else:
         boot_disk_type = 'boot_partition'
 
@@ -160,10 +158,13 @@ def _replace_disk_identifier(path, disk_identifier):
 
 
 # NOTE(TheJulia): This should likely be migrated to pxe_utils.
+# TODO(dtantsur): with the removal of netboot, root_uuid_or_disk_id and
+# the logic of replacing ROOT can be dropped, while is_whole_disk_image can
+# be renamed to something like netboot_fallback.
 def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
-                      is_whole_disk_image, trusted_boot=False,
-                      iscsi_boot=False, ramdisk_boot=False,
-                      ipxe_enabled=False):
+                      is_whole_disk_image, iscsi_boot=False,
+                      ramdisk_boot=False, ipxe_enabled=False,
+                      anaconda_boot=False):
     """Switch a pxe config from deployment mode to service mode.
 
     :param path: path to the pxe config file in tftpboot.
@@ -171,22 +172,21 @@ def switch_pxe_config(path, root_uuid_or_disk_id, boot_mode,
                                  disk_id in case of whole disk image.
     :param boot_mode: if boot mode is uefi or bios.
     :param is_whole_disk_image: if the image is a whole disk image or not.
-    :param trusted_boot: if boot with trusted_boot or not. The usage of
-        is_whole_disk_image and trusted_boot are mutually exclusive. You can
-        have one or neither, but not both.
     :param iscsi_boot: if boot is from an iSCSI volume or not.
     :param ramdisk_boot: if the boot is to be to a ramdisk configuration.
     :param ipxe_enabled: A default False boolean value to tell the method
                          if the caller is using iPXE.
+    :param anaconda_boot: if the boot is to be to an anaconda configuration.
     """
-    if not ramdisk_boot and root_uuid_or_disk_id is not None:
+    if (not (ramdisk_boot or anaconda_boot)
+            and root_uuid_or_disk_id is not None):
         if not is_whole_disk_image:
             _replace_root_uuid(path, root_uuid_or_disk_id)
         else:
             _replace_disk_identifier(path, root_uuid_or_disk_id)
 
-    _replace_boot_line(path, boot_mode, is_whole_disk_image, trusted_boot,
-                       iscsi_boot, ramdisk_boot, ipxe_enabled)
+    _replace_boot_line(path, boot_mode, is_whole_disk_image, iscsi_boot,
+                       ramdisk_boot, ipxe_enabled, anaconda_boot)
 
 
 def check_for_missing_params(info_dict, error_msg, param_prefix=''):
@@ -301,21 +301,36 @@ def agent_add_clean_params(task):
 
     :param task: a TaskManager instance.
     """
-    info = task.node.driver_internal_info
 
     random_iterations = CONF.deploy.shred_random_overwrite_iterations
-    info['agent_erase_devices_iterations'] = random_iterations
+    node = task.node
+    node.set_driver_internal_info('agent_erase_devices_iterations',
+                                  random_iterations)
     zeroize = CONF.deploy.shred_final_overwrite_with_zeros
-    info['agent_erase_devices_zeroize'] = zeroize
+    node.set_driver_internal_info('agent_erase_devices_zeroize', zeroize)
     erase_fallback = CONF.deploy.continue_if_disk_secure_erase_fails
-    info['agent_continue_if_ata_erase_failed'] = erase_fallback
+    node.set_driver_internal_info('agent_continue_if_secure_erase_failed',
+                                  erase_fallback)
+    # NOTE(janders) ``agent_continue_if_ata_erase_failed`` is deprecated and
+    # will be removed in the "Y" cycle. The replacement option
+    # ``agent_continue_if_secure_erase_failed`` is used to control shred
+    #  fallback for both ATA Secure Erase and NVMe Secure Erase.
+    # The ``agent_continue_if_ata_erase_failed`` line can
+    # be deleted along with this comment when support for it is fully removed.
+    node.set_driver_internal_info('agent_continue_if_ata_erase_failed',
+                                  erase_fallback)
+    nvme_secure_erase = CONF.deploy.enable_nvme_secure_erase
+    node.set_driver_internal_info('agent_enable_nvme_secure_erase',
+                                  nvme_secure_erase)
     secure_erase = CONF.deploy.enable_ata_secure_erase
-    info['agent_enable_ata_secure_erase'] = secure_erase
-    info['disk_erasure_concurrency'] = CONF.deploy.disk_erasure_concurrency
-    info['agent_erase_skip_read_only'] = CONF.deploy.erase_skip_read_only
+    node.set_driver_internal_info('agent_enable_ata_secure_erase',
+                                  secure_erase)
+    node.set_driver_internal_info('disk_erasure_concurrency',
+                                  CONF.deploy.disk_erasure_concurrency)
+    node.set_driver_internal_info('agent_erase_skip_read_only',
+                                  CONF.deploy.erase_skip_read_only)
 
-    task.node.driver_internal_info = info
-    task.node.save()
+    node.save()
 
 
 def try_set_boot_device(task, device, persistent=True):
@@ -356,7 +371,13 @@ def get_disk_label(node):
     :returns: the disk label or None if no disk label was specified.
     """
     capabilities = utils.parse_instance_info_capabilities(node)
-    return capabilities.get('disk_label')
+    label = capabilities.get('disk_label')
+    # NOTE(TheJulia): If the node is UEFI based, we should likely just default
+    # the table type to gpt as otherwise we rely upon the user to supply the
+    # right information, and for UEFI mode bios may work, but is wrong.
+    if label is None and boot_mode_utils.get_boot_mode(node) == 'uefi':
+        label = 'gpt'
+    return label
 
 
 def get_pxe_boot_file(node):
@@ -493,36 +514,17 @@ def validate_capabilities(node):
                  'value': value, 'valid_values': ', '.join(valid_values)})
 
 
-def validate_image_properties(ctx, deploy_info, properties):
-    """Validate the image.
-
-    For Glance images it checks that the image exists in Glance and its
-    properties or deployment info contain the properties passed. If it's not a
-    Glance image, it checks that deployment info contains needed properties.
+def get_image_properties(ctx, image_href):
+    """Get properties of the image.
 
     :param ctx: security context
-    :param deploy_info: the deploy_info to be validated
-    :param properties: the list of image meta-properties to be validated.
-    :raises: InvalidParameterValue if:
-        * connection to glance failed;
-        * authorization for accessing image failed;
-        * HEAD request to image URL failed or returned response code != 200;
-        * HEAD request response does not contain Content-Length header;
-        * the protocol specified in image URL is not supported.
-    :raises: MissingParameterValue if the image doesn't contain
-        the mentioned properties.
+    :param image_href: reference to the image
+    :return: properties as a dictionary
+    :raises: InvalidParameterValue if the image cannot be accessed
     """
-    image_href = deploy_info.get('image_source')
-    boot_iso = deploy_info.get('boot_iso')
-    if image_href and boot_iso:
-        raise exception.InvalidParameterValue(_(
-            "An 'image_source' and 'boot_iso' parameter may not be "
-            "specified at the same time."))
-    if not image_href:
-        image_href = boot_iso
     try:
         img_service = image_service.get_image_service(image_href, context=ctx)
-        image_props = img_service.show(image_href)['properties']
+        return img_service.show(image_href)['properties']
     except (exception.GlanceConnectionFailed,
             exception.ImageNotAuthorized,
             exception.Invalid):
@@ -535,6 +537,66 @@ def validate_image_properties(ctx, deploy_info, properties):
     except exception.ImageRefValidationFailed as e:
         raise exception.InvalidParameterValue(err=e)
 
+
+def validate_image_properties(task, deploy_info):
+    """Validate the image.
+
+    For Glance images it checks that the image exists in Glance and its
+    properties or deployment info contain the properties passed. If it's not a
+    Glance image, it checks that deployment info contains needed properties.
+
+    :param task: TaskManager instance with a valid node
+    :param deploy_info: the deploy_info to be validated
+    :raises: InvalidParameterValue if:
+        * connection to glance failed;
+        * authorization for accessing image failed;
+        * HEAD request to image URL failed or returned response code != 200;
+        * HEAD request response does not contain Content-Length header;
+        * the protocol specified in image URL is not supported.
+    :raises: MissingParameterValue if the image doesn't contain
+        the mentioned properties.
+    """
+    node = task.node
+    image_href = deploy_info.get('image_source')
+    boot_iso = deploy_info.get('boot_iso')
+    isap = task.node.driver_internal_info.get('is_source_a_path')
+    if image_href and boot_iso:
+        raise exception.InvalidParameterValue(_(
+            "An 'image_source' and 'boot_iso' parameter may not be "
+            "specified at the same time."))
+    if not image_href:
+        image_href = boot_iso
+
+    boot_option = get_boot_option(task.node)
+
+    if (boot_iso
+            or node.driver_internal_info.get('is_whole_disk_image')
+            or boot_option == 'local'
+            or isap):
+        # No image properties are required in this case, but validate that the
+        # image at least looks reasonable.
+        try:
+            # This doesn't actually test *getting* the defined url or file
+            # but actually validates we can parse the data *to* connect.
+            image_service.get_image_service(image_href, context=task.context)
+        except exception.ImageRefValidationFailed as e:
+            raise exception.InvalidParameterValue(err=e)
+        if not isap:
+            # If the source is not a path, but otherwise matches, we need to
+            # exit validation here. Deployments, such as ramdisk or anaconda
+            # need further parameter validation and this supplies it.
+            return
+
+    if service_utils.is_glance_image(image_href):
+        properties = ['kernel_id', 'ramdisk_id']
+        if boot_option == 'kickstart':
+            properties.append('stage2_id')
+        image_props = get_image_properties(task.context, image_href)
+    else:
+        # We are likely netbooting in this case...
+        properties = ['kernel', 'ramdisk']
+        image_props = {}
+
     missing_props = []
     for prop in properties:
         if not (deploy_info.get(prop) or image_props.get(prop)):
@@ -547,12 +609,6 @@ def validate_image_properties(ctx, deploy_info, properties):
             "%(properties)s") % {'image': image_href, 'properties': props})
 
 
-def get_default_boot_option():
-    """Gets the default boot option."""
-    # TODO(TheJulia): Deprecated: Remove after Ussuri.
-    return CONF.deploy.default_boot_option
-
-
 def get_boot_option(node):
     """Gets the boot option.
 
@@ -560,15 +616,33 @@ def get_boot_option(node):
     :raises: InvalidParameterValue if the capabilities string is not a
          dict or is malformed.
     :returns: A string representing the boot option type. Defaults to
-        'netboot'.
+        configuration setting [deploy]default_boot_mode.
     """
+    if is_anaconda_deploy(node):
+        return 'kickstart'
+    if is_ramdisk_deploy(node):
+        return 'ramdisk'
+    return 'local'
 
-    # NOTE(TheJulia): Software raid always implies local deployment
-    if is_software_raid(node):
-        return 'local'
-    capabilities = utils.parse_instance_info_capabilities(node)
-    return capabilities.get('boot_option',
-                            CONF.deploy.default_boot_option).lower()
+
+# FIXME(dtantsur): relying on deploy interface name is an anti-pattern.
+# Refactor the code so that the deploy interface itself provides the only boot
+# option it supports.
+
+def is_ramdisk_deploy(node):
+    return node.get_interface('deploy') == 'ramdisk'
+
+
+def is_anaconda_deploy(node):
+    """Determine if Anaconda deploy interface is in use for the deployment.
+
+    :param node: A single Node.
+    :returns: A boolean value of True when Anaconda deploy interface is in use
+              otherwise False
+    """
+    if node.get_interface('deploy') == 'anaconda':
+        return True
+    return False
 
 
 def is_software_raid(node):
@@ -588,6 +662,9 @@ def is_software_raid(node):
     return software_raid
 
 
+IPA_URL_PARAM_NAME = 'ipa-api-url'
+
+
 def build_agent_options(node):
     """Build the options to be passed to the agent ramdisk.
 
@@ -596,7 +673,7 @@ def build_agent_options(node):
         agent ramdisk.
     """
     agent_config_opts = {
-        'ipa-api-url': get_ironic_api_url(),
+        IPA_URL_PARAM_NAME: get_ironic_api_url(),
     }
     return agent_config_opts
 
@@ -646,17 +723,15 @@ def prepare_inband_cleaning(task, manage_boot=True):
     agent_add_clean_params(task)
 
     if manage_boot:
-        ramdisk_opts = build_agent_options(task.node)
-        task.driver.boot.prepare_ramdisk(task, ramdisk_opts)
+        prepare_agent_boot(task)
 
     # NOTE(dtantsur): calling prepare_ramdisk may power off the node, so we
     # need to check fast-track again and reboot if needed.
     fast_track = manager_utils.is_fast_track(task)
     if not fast_track:
         manager_utils.node_power_action(task, states.REBOOT)
-
-    # Tell the conductor we are waiting for the agent to boot.
-    return states.CLEANWAIT
+        # Tell the conductor we are waiting for the agent to boot.
+        return states.CLEANWAIT
 
 
 def tear_down_inband_cleaning(task, manage_boot=True):
@@ -715,21 +790,34 @@ def get_image_instance_info(node):
     # more explicit unit testing to exist.
     info = {}
 
-    is_whole_disk_image = node.driver_internal_info.get('is_whole_disk_image')
     boot_iso = node.instance_info.get('boot_iso')
-    if not boot_iso:
-        info['image_source'] = node.instance_info.get('image_source')
-    else:
+    image_source = node.instance_info.get('image_source')
+    if boot_iso:
+        if image_source:
+            raise exception.InvalidParameterValue(_(
+                "An 'image_source' and 'boot_iso' parameter may not be "
+                "specified at the same time."))
         info['boot_iso'] = boot_iso
-
-    if not is_whole_disk_image and not boot_iso:
-        if not service_utils.is_glance_image(info['image_source']):
+    else:
+        boot_option = get_boot_option(node)
+        if boot_option == 'ramdisk':
+            # Ramdisk deploy does not require an image
             info['kernel'] = node.instance_info.get('kernel')
             info['ramdisk'] = node.instance_info.get('ramdisk')
+        else:
+            info['image_source'] = image_source
+
+            is_whole_disk_image = node.driver_internal_info.get(
+                'is_whole_disk_image')
+            if (not is_whole_disk_image
+                    and boot_option != 'local'
+                    and not service_utils.is_glance_image(image_source)):
+                info['kernel'] = node.instance_info.get('kernel')
+                info['ramdisk'] = node.instance_info.get('ramdisk')
 
     error_msg = (_("Cannot validate image information for node %s because one "
                    "or more parameters are missing from its instance_info and "
-                   "insufficent information is present to boot from a remote "
+                   "insufficient information is present to boot from a remote "
                    "volume")
                  % node.uuid)
     check_for_missing_params(info, error_msg)
@@ -737,11 +825,10 @@ def get_image_instance_info(node):
     return info
 
 
-_ERR_MSG_INVALID_DEPLOY = _("Cannot validate parameter for driver deploy. "
-                            "Invalid parameter %(param)s. Reason: %(reason)s")
+_ERR_MSG_INVALID_DEPLOY = _("Invalid parameter %(param)s: %(reason)s")
 
 
-def parse_instance_info(node):
+def parse_instance_info(node, image_deploy=True):
     """Gets the instance specific Node deployment info.
 
     This method validates whether the 'instance_info' property of the
@@ -749,6 +836,9 @@ def parse_instance_info(node):
     deploy images to the node.
 
     :param node: a single Node.
+    :param image_deploy: If the deployment interface is aware this
+                         is an image based deployment, default
+                         True.
     :returns: A dict with the instance_info values.
     :raises: MissingParameterValue, if any of the required parameters are
         missing.
@@ -760,16 +850,22 @@ def parse_instance_info(node):
     i_info = {}
     i_info['image_source'] = info.get('image_source')
     iwdi = node.driver_internal_info.get('is_whole_disk_image')
+    boot_option = get_boot_option(node)
     if not iwdi:
         if (i_info['image_source']
+                and boot_option != 'local'
                 and not service_utils.is_glance_image(
                     i_info['image_source'])):
             i_info['kernel'] = info.get('kernel')
             i_info['ramdisk'] = info.get('ramdisk')
-        i_info['root_gb'] = info.get('root_gb')
+        if image_deploy:
+            # root_gb is expected with partition images.
+            i_info['root_gb'] = info.get('root_gb')
 
-    error_msg = _("Cannot validate driver deploy. Some parameters were missing"
-                  " in node's instance_info")
+    # NOTE(TheJulia): Kernel/ramdisk may be optional and originated
+    # with pure workload network booting.
+
+    error_msg = _("Some parameters were missing in node's instance_info")
     check_for_missing_params(i_info, error_msg)
 
     # This is used in many places, so keep it even for whole-disk images.
@@ -790,7 +886,8 @@ def parse_instance_info(node):
             err_msg_invalid = _("Cannot deploy whole disk image with "
                                 "swap or ephemeral size set")
             raise exception.InvalidParameterValue(err_msg_invalid)
-    else:
+    elif image_deploy:
+        # NOTE(TheJulia): This only applies to partition image deploys.
         _validate_layout_properties(node, info, i_info)
 
     i_info['configdrive'] = info.get('configdrive')
@@ -838,13 +935,12 @@ def _check_disk_layout_unchanged(node, i_info):
     """
     # If a node has been deployed to, this is the instance information
     # used for that deployment.
-    driver_internal_info = node.driver_internal_info
-    if 'instance' not in driver_internal_info:
+    if 'instance' not in node.driver_internal_info:
         return
 
     error_msg = ''
     for param in DISK_LAYOUT_PARAMS:
-        param_value = int(driver_internal_info['instance'][param])
+        param_value = int(node.driver_internal_info['instance'][param])
         if param_value != int(i_info[param]):
             error_msg += (_(' Deployed value of %(param)s was %(param_value)s '
                             'but requested value is %(request_value)s.') %
@@ -978,11 +1074,15 @@ def _validate_image_url(node, url, secret=False):
         it will not be shown in logs.
     """
     try:
+        # NOTE(TheJulia): This method only validates that an exception
+        # is NOT raised. In other words, that the endpoint does not
+        # return a 200. If we're fed a folder list, this will still
+        # work, which is a good and bad thing at the same time. :/
         image_service.HttpImageService().validate_href(url, secret)
     except exception.ImageRefValidationFailed as e:
         with excutils.save_and_reraise_exception():
             LOG.error("The specified URL is not a valid HTTP(S) URL or is "
-                      "not reachable for node %(node)s. Error: %(msg)s",
+                      "not reachable for node %(node)s: %(msg)s",
                       {'node': node.uuid, 'msg': e})
 
 
@@ -1037,6 +1137,24 @@ def _cache_and_convert_image(task, instance_info, image_info=None):
     symlink_dir = _get_http_image_symlink_dir_path()
     fileutils.ensure_tree(symlink_dir)
     symlink_path = _get_http_image_symlink_file_path(task.node.uuid)
+    file_extension = None
+    if get_boot_option(task.node) == 'kickstart':
+        # 'liveimg --url' kickstart command uses the file extension to
+        # identify the OS image type. Without a valid file extension it will
+        # assume the disk image is a partition image and try to 'mount' it on
+        # the ramdisk. See 'liveimg' command for more details
+        # https://pykickstart.readthedocs.io/en/latest/kickstart-docs.html
+        valid_file_extensions = ['.img', '.tar', '.tbz', '.tgz', '.txz',
+                                 '.tar.gz', '.tar.bz2', '.tar.xz']
+        if image_info and 'disk_file_extension' in image_info['properties']:
+            ext = image_info['properties']['disk_file_extension']
+            file_extension = ext if ext in valid_file_extensions else None
+        if file_extension:
+            symlink_path = symlink_path + file_extension
+        else:
+            LOG.warning("The 'disk_file_extension' property was not set on "
+                        "the glance image or not a valid extension so the "
+                        "anaconda installer will try to 'mount' the OS image.")
     utils.create_link_without_raise(image_path, symlink_path)
 
     base_url = CONF.deploy.http_url
@@ -1045,6 +1163,8 @@ def _cache_and_convert_image(task, instance_info, image_info=None):
     http_image_url = '/'.join(
         [base_url, CONF.deploy.http_image_subdir,
          task.node.uuid])
+    if file_extension:
+        http_image_url = http_image_url + file_extension
     _validate_image_url(task.node, http_image_url, secret=False)
     instance_info['image_url'] = http_image_url
 
@@ -1070,7 +1190,12 @@ def build_instance_info_for_deploy(task):
     instance_info = node.instance_info
     iwdi = node.driver_internal_info.get('is_whole_disk_image')
     image_source = instance_info['image_source']
+    isap = node.driver_internal_info.get('is_source_a_path')
+    # If our url ends with a /, i.e. we have been supplied with a path,
+    # we can only deploy this in limited cases for drivers and tools
+    # which are aware of such. i.e. anaconda.
     image_download_source = get_image_download_source(node)
+    boot_option = get_boot_option(task.node)
 
     if service_utils.is_glance_image(image_source):
         glance = image_service.GlanceImageService(context=task.context)
@@ -1093,22 +1218,40 @@ def build_instance_info_for_deploy(task):
         instance_info['image_tags'] = image_info.get('tags', [])
         instance_info['image_properties'] = image_info['properties']
 
-        if not iwdi:
+        if not iwdi and boot_option != 'local':
             instance_info['kernel'] = image_info['properties']['kernel_id']
             instance_info['ramdisk'] = image_info['properties']['ramdisk_id']
     elif (image_source.startswith('file://')
           or image_download_source == 'local'):
+        # NOTE(TheJulia): Intentionally only supporting file:/// as image
+        # based deploy source since we don't want to, nor should we be in
+        # in the business of copying large numbers of files as it is a
+        # huge performance impact.
         _cache_and_convert_image(task, instance_info)
     else:
-        _validate_image_url(node, image_source)
-        instance_info['image_url'] = image_source
+        try:
+            _validate_image_url(node, image_source)
+            # image_url is internal, and used by IPA and some boot templates.
+            # in most cases, it needs to come from image_source explicitly.
+            instance_info['image_url'] = image_source
+        except exception.ImageRefIsARedirect as e:
+            if e.redirect_url:
+                instance_info['image_url'] = e.redirect_url
+            else:
+                raise
 
-    if not iwdi:
-        instance_info['image_type'] = 'partition'
-        i_info = parse_instance_info(node)
-        instance_info.update(i_info)
+    if not isap:
+        if not iwdi:
+            instance_info['image_type'] = images.IMAGE_TYPE_PARTITION
+            i_info = parse_instance_info(node)
+            instance_info.update(i_info)
+        else:
+            instance_info['image_type'] = images.IMAGE_TYPE_WHOLE_DISK
     else:
-        instance_info['image_type'] = 'whole-disk-image'
+        # Call central parsing so we retain things like config drives.
+        i_info = parse_instance_info(node, image_deploy=False)
+        instance_info.update(i_info)
+
     return instance_info
 
 
@@ -1167,19 +1310,17 @@ def populate_storage_driver_internal_info(task):
     boot_capability = ("%s_volume_boot" % vol_type)
     deploy_capability = ("%s_volume_deploy" % vol_type)
     vol_uuid = boot_volume['uuid']
-    driver_internal_info = node.driver_internal_info
     if check_interface_capability(task.driver.boot, boot_capability):
-        driver_internal_info['boot_from_volume'] = vol_uuid
+        node.set_driver_internal_info('boot_from_volume', vol_uuid)
     # NOTE(TheJulia): This would be a convenient place to check
     # if we need to know about deploying the volume.
     if (check_interface_capability(task.driver.deploy, deploy_capability)
             and task.driver.storage.should_write_image(task)):
-        driver_internal_info['boot_from_volume_deploy'] = vol_uuid
+        node.set_driver_internal_info('boot_from_volume_deploy', vol_uuid)
         # NOTE(TheJulia): This is also a useful place to include a
         # root device hint since we should/might/be able to obtain
         # and supply that information to IPA if it needs to write
         # the image to the volume.
-    node.driver_internal_info = driver_internal_info
     node.save()
 
 
@@ -1201,10 +1342,8 @@ def tear_down_storage_configuration(task):
                  {'target': volume.uuid, 'node': task.node.uuid})
 
     node = task.node
-    driver_internal_info = node.driver_internal_info
-    driver_internal_info.pop('boot_from_volume', None)
-    driver_internal_info.pop('boot_from_volume_deploy', None)
-    node.driver_internal_info = driver_internal_info
+    node.del_driver_internal_info('boot_from_volume')
+    node.del_driver_internal_info('boot_from_volume_deploy')
     node.save()
 
 
@@ -1225,7 +1364,6 @@ def is_iscsi_boot(task):
 
 # NOTE(etingof): retain original location of these funcs for compatibility
 is_secure_boot_requested = boot_mode_utils.is_secure_boot_requested
-is_trusted_boot_requested = boot_mode_utils.is_trusted_boot_requested
 get_boot_mode_for_deploy = boot_mode_utils.get_boot_mode_for_deploy
 parse_instance_info_capabilities = (
     utils.parse_instance_info_capabilities
@@ -1242,7 +1380,7 @@ def get_async_step_return_state(node):
     return states.CLEANWAIT if node.clean_step else states.DEPLOYWAIT
 
 
-def _check_agent_token_prior_to_agent_reboot(driver_internal_info):
+def _check_agent_token_prior_to_agent_reboot(node):
     """Removes the agent token if it was not pregenerated.
 
     Removal of the agent token in cases where it is not pregenerated
@@ -1253,11 +1391,11 @@ def _check_agent_token_prior_to_agent_reboot(driver_internal_info):
     already included in the payload and must be generated again
     upon lookup.
 
-    :param driver_internal_info: The driver_interal_info dict object
-                                 from a Node object.
+    :param node: The Node object.
     """
-    if not driver_internal_info.get('agent_secret_token_pregenerated', False):
-        driver_internal_info.pop('agent_secret_token', None)
+    if not node.driver_internal_info.get('agent_secret_token_pregenerated',
+                                         False):
+        node.del_driver_internal_info('agent_secret_token')
 
 
 def set_async_step_flags(node, reboot=None, skip_current_step=None,
@@ -1279,28 +1417,57 @@ def set_async_step_flags(node, reboot=None, skip_current_step=None,
         corresponding polling flag is not set in the node's
         driver_internal_info.
     """
-    info = node.driver_internal_info
-    cleaning = {'reboot': 'cleaning_reboot',
-                'skip': 'skip_current_clean_step',
-                'polling': 'cleaning_polling'}
-    deployment = {'reboot': 'deployment_reboot',
-                  'skip': 'skip_current_deploy_step',
-                  'polling': 'deployment_polling'}
-    fields = cleaning if node.clean_step else deployment
+    if node.clean_step:
+        reboot_field = 'cleaning_reboot'
+        skip_field = 'skip_current_clean_step'
+        polling_field = 'cleaning_polling'
+    else:
+        reboot_field = 'deployment_reboot'
+        skip_field = 'skip_current_deploy_step'
+        polling_field = 'deployment_polling'
+
     if reboot is not None:
-        info[fields['reboot']] = reboot
+        node.set_driver_internal_info(reboot_field, reboot)
         if reboot:
             # If rebooting, we must ensure that we check and remove
             # an agent token if necessary.
-            _check_agent_token_prior_to_agent_reboot(info)
+            _check_agent_token_prior_to_agent_reboot(node)
     if skip_current_step is not None:
-        info[fields['skip']] = skip_current_step
+        node.set_driver_internal_info(skip_field, skip_current_step)
     if polling is not None:
-        info[fields['polling']] = polling
-    node.driver_internal_info = info
+        node.set_driver_internal_info(polling_field, polling)
     node.save()
 
 
+def prepare_agent_boot(task):
+    """Prepare booting the agent on the node.
+
+    :param task: a TaskManager instance.
+    """
+    deploy_opts = build_agent_options(task.node)
+    task.driver.boot.prepare_ramdisk(task, deploy_opts)
+
+
+def reboot_to_finish_step(task):
+    """Reboot the node into IPA to finish a deploy/clean step.
+
+    :param task: a TaskManager instance.
+    :returns: states.CLEANWAIT if cleaning operation in progress
+              or states.DEPLOYWAIT if deploy operation in progress.
+    """
+    disable_ramdisk = task.node.driver_internal_info.get(
+        'cleaning_disable_ramdisk')
+    if not disable_ramdisk:
+        if manager_utils.is_fast_track(task):
+            LOG.debug('Forcing power off on node %s for a clean reboot into '
+                      'the agent image', task.node)
+            manager_utils.node_power_action(task, states.POWER_OFF)
+        prepare_agent_boot(task)
+
+    manager_utils.node_power_action(task, states.REBOOT)
+    return get_async_step_return_state(task.node)
+
+
 def get_root_device_for_deploy(node):
     """Get a root device requested for deployment or None.
 
@@ -1321,5 +1488,19 @@ def get_root_device_for_deploy(node):
     except ValueError as e:
         raise exception.InvalidParameterValue(
             _('Failed to validate the root device hints %(hints)s (from the '
-              'node\'s %(source)s) for node %(node)s. Error: %(error)s') %
+              'node\'s %(source)s) for node %(node)s: %(error)s') %
             {'node': node.uuid, 'hints': hints, 'source': source, 'error': e})
+
+
+def needs_agent_ramdisk(node, mode='deploy'):
+    """Checks whether the node requires an agent ramdisk now."""
+    if mode != 'deploy':
+        return True  # Rescue always needs a ramdisk
+
+    if get_boot_option(node) != 'ramdisk':
+        return True  # Normal deploys need an agent
+
+    # Ramdisk deploys don't need an agent, but cleaning will. Since we don't
+    # want nodes to be stuck on deletion, require an agent when cleaning is
+    # enabled.
+    return not manager_utils.skip_automated_cleaning(node)
