@@ -1,3 +1,4 @@
+# Copyright 2022 Hewlett Packard Enterprise Development LP
 # Copyright 2014 Hewlett-Packard Development Company, L.P.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -19,25 +20,25 @@ Common functionalities shared between different iLO modules.
 import os
 import shutil
 import tempfile
+from urllib import parse as urlparse
 
 from ironic_lib import utils as ironic_utils
 from oslo_log import log as logging
+from oslo_utils import fileutils
 from oslo_utils import importutils
-import six
-import six.moves.urllib.parse as urlparse
-from six.moves.urllib.parse import urljoin
+from oslo_utils import strutils
 
 from ironic.common import boot_devices
 from ironic.common import exception
 from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
+from ironic.common import image_service
 from ironic.common import images
 from ironic.common import swift
 from ironic.common import utils
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers.modules import boot_mode_utils
-from ironic.drivers.modules import deploy_utils
 
 ilo_client = importutils.try_import('proliantutils.ilo.client')
 ilo_error = importutils.try_import('proliantutils.exception')
@@ -57,7 +58,17 @@ REQUIRED_PROPERTIES = {
 OPTIONAL_PROPERTIES = {
     'client_port': _("port to be used for iLO operations. Optional."),
     'client_timeout': _("timeout (in seconds) for iLO operations. Optional."),
-    'ca_file': _("CA certificate file to validate iLO. Optional")
+    'ca_file': _("CA certificate file to validate iLO. This "
+                 "attibute is deprecated and will be removed in "
+                 "future release. Optional"),
+    'ilo_verify_ca': _("Either a Boolean value, a path to a CA_BUNDLE "
+                       "file or directory with certificates of trusted "
+                       "CAs. If set to True the driver will verify the "
+                       "host certificates; if False the driver will ignore "
+                       "verifying the SSL certificate. If it\'s a path the "
+                       "driver will use the specified certificate or one of "
+                       "the certificates in the directory. Defaults to True. "
+                       "Optional")
 }
 
 SNMP_PROPERTIES = {
@@ -96,6 +107,76 @@ BOOT_MODE_GENERIC_TO_ILO = {'bios': 'legacy', 'uefi': 'uefi'}
 BOOT_MODE_ILO_TO_GENERIC = dict(
     (v, k) for (k, v) in BOOT_MODE_GENERIC_TO_ILO.items())
 
+POST_NULL_STATE = 'Null'
+""" Node is in Null post state."""
+
+POST_UNKNOWN_STATE = "Unknown"
+""" Node is in Unknown post state."""
+
+POST_RESET_STATE = "Reset"
+""" Node is in Reset post state."""
+
+POST_POWEROFF_STATE = "PowerOff"
+""" Node is in PowerOff post state."""
+
+POST_INPOST_STATE = "InPost"
+""" Node is in InPost post state."""
+
+POST_INPOSTDISCOVERY_STATE = "InPostDiscoveryComplete"
+""" Node is in InPostDiscoveryComplete post state."""
+
+POST_FINISHEDPOST_STATE = "FinishedPost"
+""" Node is in FinishedPost post state."""
+
+SUPPORTED_BOOT_MODE_LEGACY_BIOS_ONLY = 'legacy bios only'
+""" Node supports only legacy BIOS boot mode."""
+
+SUPPORTED_BOOT_MODE_UEFI_ONLY = 'uefi only'
+""" Node supports only UEFI boot mode."""
+
+SUPPORTED_BOOT_MODE_LEGACY_BIOS_AND_UEFI = 'legacy bios and uefi'
+""" Node supports both legacy BIOS and UEFI boot mode."""
+
+SUPPORTED_SECURITY_PARAMETERS = (
+    ['password_complexity', 'require_login_for_ilo_rbsu', 'ipmi_over_lan',
+     'secure_boot', 'require_host_authentication'])
+
+
+def validate_security_parameter_values(sec_param_info):
+    """Validate security parameter with valid values.
+
+    :param sec_param_info: dict object containing the security parameter info
+    :raises: MissingParameterValue, for missing fields (or values) in
+             security parameter info.
+    :raises: InvalidParameterValue, for unsupported security parameter
+    :returns: tuple of security param, ignore and enable parameters.
+    """
+    info = sec_param_info or {}
+
+    LOG.debug("Validating security param info: %s in progress", info)
+    param = info.get('param')
+
+    if not param:
+        msg = (_("Security parameter info: %(info)s is missing the "
+                 "required 'param' field.") % {'info': info})
+        LOG.error(msg)
+        raise exception.MissingParameterValue(msg)
+
+    if param not in SUPPORTED_SECURITY_PARAMETERS:
+        msg = (_("Security parameter '%(param)s' is not a valid parameter. "
+                 "Supported values are: %(supported_params)s") %
+               {'param': param, 'supported_params': (
+                ", ".join(SUPPORTED_SECURITY_PARAMETERS))})
+        LOG.error(msg)
+        raise exception.InvalidParameterValue(msg)
+
+    ignored = info.get('ignore', False)
+    ignored = strutils.bool_from_string(ignored, default=False)
+    enable = info.get('enable', True)
+    enable = strutils.bool_from_string(enable, default=True)
+
+    return param, enable, ignored
+
 
 def copy_image_to_web_server(source_file_path, destination):
     """Copies the given image to the http web server.
@@ -115,7 +196,7 @@ def copy_image_to_web_server(source_file_path, destination):
 
     """
 
-    image_url = urljoin(CONF.deploy.http_url, destination)
+    image_url = urlparse.urljoin(CONF.deploy.http_url, destination)
     image_path = os.path.join(CONF.deploy.http_root, destination)
     try:
         shutil.copyfile(source_file_path, image_path)
@@ -184,7 +265,7 @@ def remove_image_from_swift(object_name, associated_with=None):
         swift_api = swift.SwiftAPI()
         swift_api.delete_object(container, object_name)
     except exception.SwiftObjectNotFoundError as e:
-        LOG.info("Temporary object %(associated_with_msg)s"
+        LOG.info("Temporary object %(associated_with_msg)s "
                  "was already deleted from Swift. Error: %(err)s",
                  {'associated_with_msg':
                      ("associated with %s " % associated_with
@@ -226,22 +307,8 @@ def parse_driver_info(node):
             "The following required iLO parameters are missing from the "
             "node's driver_info: %s") % missing_info)
 
-    not_integers = []
-    for param in OPTIONAL_PROPERTIES:
-        value = info.get(param, CONF.ilo.get(param))
-        if param == "client_port":
-            d_info[param] = utils.validate_network_port(value, param)
-        elif param == "ca_file":
-            if value and not os.path.isfile(value):
-                raise exception.InvalidParameterValue(_(
-                    '%(param)s "%(value)s" is not found.') %
-                    {'param': param, 'value': value})
-            d_info[param] = value
-        else:
-            try:
-                d_info[param] = int(value)
-            except ValueError:
-                not_integers.append(param)
+    optional_info = _parse_optional_driver_info(node)
+    d_info.update(optional_info)
 
     snmp_info = _parse_snmp_driver_info(info)
     if snmp_info:
@@ -255,11 +322,6 @@ def parse_driver_info(node):
             if param == "console_port":
                 d_info[param] = utils.validate_network_port(value, param)
 
-    if not_integers:
-        raise exception.InvalidParameterValue(_(
-            "The following iLO parameters from the node's driver_info "
-            "should be integers: %s") % not_integers)
-
     return d_info
 
 
@@ -291,7 +353,7 @@ def _parse_snmp_driver_info(info):
         for param in SNMP_OPTIONAL_PROPERTIES:
             value = None
             try:
-                value = six.text_type(info[param]).upper()
+                value = str(info[param]).upper()
             except KeyError:
                 pass
             if value:
@@ -306,6 +368,75 @@ def _parse_snmp_driver_info(info):
     return snmp_info
 
 
+def _parse_optional_driver_info(node):
+    """Parses the optional driver_info parameters.
+
+    :param node: an ironic Node object.
+    :returns: a dictionary containing information of optional properties.
+    :raises: InvalidParameterValue if any parameters are incorrect
+    """
+    info = node.driver_info
+    optional_info = {}
+    not_integers = []
+
+    for param in OPTIONAL_PROPERTIES:
+        if param != 'ilo_verify_ca':
+            value = info.get(param, CONF.ilo.get(param))
+
+        if param == "client_port":
+            optional_info[param] = utils.validate_network_port(value, param)
+        elif param == "client_timeout":
+            try:
+                optional_info[param] = int(value)
+            except ValueError:
+                not_integers.append(param)
+
+    if not_integers:
+        raise exception.InvalidParameterValue(_(
+            "The following iLO parameters from the node's driver_info "
+            "should be integers: %s") % not_integers)
+
+    ca_file_value = info.get('ca_file', CONF.ilo.get('ca_file'))
+    verify_ca = info.get('ilo_verify_ca', CONF.ilo.get('verify_ca'))
+
+    if ca_file_value:
+        LOG.warning("The `driver_info/ca_file` parameter is deprecated "
+                    "in favor of `driver_info/ilo_verify_ca` parameter. "
+                    "Please use `driver_info/ilo_verify_ca` instead of "
+                    "`driver_info/ca_file` in node %(node)s "
+                    "configuration.", {'node': node.uuid})
+
+        if not os.path.isfile(ca_file_value):
+            raise exception.InvalidParameterValue(_(
+                'ca_file "%(value)s" is not found.') %
+                {'value': ca_file_value})
+        optional_info['verify_ca'] = ca_file_value
+    else:
+        # Check if ilo_verify_ca is a Boolean or a file/directory
+        # in the file-system
+        if isinstance(verify_ca, str):
+            if not os.path.isdir(verify_ca) and not os.path.isfile(verify_ca):
+                try:
+                    verify_ca = strutils.bool_from_string(verify_ca,
+                                                          strict=True)
+                except ValueError:
+                    raise exception.InvalidParameterValue(
+                        _('Invalid value type set in driver_info/'
+                          'ilo_verify_ca on node %(node)s. '
+                          'The value should be a Boolean or the path '
+                          'to a file/directory, not "%(value)s"'
+                          ) % {'value': verify_ca, 'node': node.uuid})
+        elif not isinstance(verify_ca, bool):
+            raise exception.InvalidParameterValue(
+                _('Invalid value type set in driver_info/ilo_verify_ca '
+                  'on node %(node)s. The value should be a Boolean '
+                  'or the path to a file/directory, not "%(value)s"'
+                  ) % {'value': verify_ca, 'node': node.uuid})
+        optional_info['verify_ca'] = verify_ca
+
+    return optional_info
+
+
 def get_ilo_object(node):
     """Gets an IloClient object from proliantutils library.
 
@@ -340,7 +471,7 @@ def get_ilo_object(node):
                                       driver_info['ilo_password'],
                                       driver_info['client_timeout'],
                                       driver_info['client_port'],
-                                      cacert=driver_info.get('ca_file'),
+                                      cacert=driver_info['verify_ca'],
                                       snmp_credentials=info)
     return ilo_object
 
@@ -365,6 +496,26 @@ def update_ipmi_properties(task):
     task.node.driver_info = info
 
 
+def update_redfish_properties(task):
+    """Update redfish properties to node driver_info
+
+    This method updates the node's driver info with redfish driver driver_info.
+    :param task: a task from TaskManager.
+    """
+    node = task.node
+    info = node.driver_info
+
+    # updating redfish credentials
+    info['redfish_address'] = info.get('ilo_address')
+    info['redfish_username'] = info.get('ilo_username')
+    info['redfish_password'] = info.get('ilo_password')
+    info['redfish_verify_ca'] = info.get('ilo_verify_ca')
+    info['redfish_system_id'] = '/redfish/v1/Systems/1'
+
+    # saving redfish credentials to task object
+    task.node.driver_info = info
+
+
 def _get_floppy_image_name(node):
     """Returns the floppy image name for a given node.
 
@@ -455,10 +606,11 @@ def set_boot_mode(node, boot_mode):
     except ilo_error.IloCommandNotSupportedError:
         p_boot_mode = DEFAULT_BOOT_MODE
 
-    if BOOT_MODE_ILO_TO_GENERIC[p_boot_mode.lower()] == boot_mode:
-        LOG.info("Node %(uuid)s pending boot mode is %(boot_mode)s.",
-                 {'uuid': node.uuid, 'boot_mode': boot_mode})
-        return
+    if p_boot_mode:
+        if BOOT_MODE_ILO_TO_GENERIC[p_boot_mode.lower()] == boot_mode:
+            LOG.info("Node %(uuid)s pending boot mode is %(boot_mode)s.",
+                     {'uuid': node.uuid, 'boot_mode': boot_mode})
+            return
 
     try:
         ilo_object.set_pending_boot_mode(
@@ -472,6 +624,24 @@ def set_boot_mode(node, boot_mode):
              {'uuid': node.uuid, 'boot_mode': boot_mode})
 
 
+def get_current_boot_mode(node):
+    """Get the current boot mode for a node.
+
+    :param node: an ironic node object.
+    :raises: IloOperationError if failed to fetch boot mode.
+    :raises: IloOperationNotSupported if node does not support getting pending
+             boot mode.
+    """
+    ilo_object = get_ilo_object(node)
+    operation = _("Get current boot mode")
+    try:
+        c_boot_mode = ilo_object.get_current_boot_mode()
+        return BOOT_MODE_ILO_TO_GENERIC[c_boot_mode.lower()]
+    except ilo_error.IloError as ilo_exception:
+        raise exception.IloOperationError(operation=operation,
+                                          error=ilo_exception)
+
+
 def update_boot_mode(task):
     """Update instance_info with boot mode to be used for deploy.
 
@@ -489,9 +659,7 @@ def update_boot_mode(task):
     # No boot mode found. Check if default_boot_mode is defined
     if not boot_mode and (CONF.ilo.default_boot_mode in ['bios', 'uefi']):
         boot_mode = CONF.ilo.default_boot_mode
-        instance_info = node.instance_info
-        instance_info['deploy_boot_mode'] = boot_mode
-        node.instance_info = instance_info
+        node.set_driver_internal_info('deploy_boot_mode', boot_mode)
         node.save()
 
     # Boot mode is computed, setting it for the deploy
@@ -531,13 +699,11 @@ def update_boot_mode(task):
                   "as pending boot mode is unknown.",
                   {'uuid': node.uuid, 'boot_mode': boot_mode})
 
-    instance_info = node.instance_info
-    instance_info['deploy_boot_mode'] = boot_mode
-    node.instance_info = instance_info
+    node.set_driver_internal_info('deploy_boot_mode', boot_mode)
     node.save()
 
 
-def setup_vmedia(task, iso, ramdisk_options):
+def setup_vmedia(task, iso, ramdisk_options=None):
     """Attaches virtual media and sets it as boot device.
 
     This method attaches the given bootable ISO as virtual media, prepares the
@@ -545,15 +711,18 @@ def setup_vmedia(task, iso, ramdisk_options):
 
     :param task: a TaskManager instance containing the node to act on.
     :param iso: a bootable ISO image href to attach to. Should be either
-        of below:
-        * A Swift object - It should be of format 'swift:<object-name>'.
-          It is assumed that the image object is present in
-          CONF.ilo.swift_ilo_container;
-        * A Glance image - It should be format 'glance://<glance-image-uuid>'
-          or just <glance-image-uuid>;
-        * An HTTP URL.
+                of below:
+
+                * A Swift object - It should be of format
+                  ``swift:<object-name>``.
+                  It is assumed that the image object is present in
+                  CONF.ilo.swift_ilo_container;
+                * A Glance image - It should be format
+                  ``glance://<glance-image-uuid>``
+                  or just ``<glance-image-uuid>``;
+                * An HTTP URL.
     :param ramdisk_options: the options to be passed to the ramdisk in virtual
-        media floppy.
+                            media floppy.
     :raises: ImageCreationFailed, if it failed while creating the floppy image.
     :raises: IloOperationError, if some operation on iLO failed.
     """
@@ -576,15 +745,18 @@ def setup_vmedia_for_boot(task, boot_iso, parameters=None):
 
     :param task: a TaskManager instance containing the node to act on.
     :param boot_iso: a bootable ISO image to attach to. Should be either
-        of below:
-        * A Swift object - It should be of format 'swift:<object-name>'.
-          It is assumed that the image object is present in
-          CONF.ilo.swift_ilo_container;
-        * A Glance image - It should be format 'glance://<glance-image-uuid>'
-          or just <glance-image-uuid>;
-        * An HTTP(S) URL.
+                     of below:
+
+                     * A Swift object - It should be of format
+                       ``swift:<object-name>``.
+                       It is assumed that the image object is present in
+                       CONF.ilo.swift_ilo_container;
+                     * A Glance image - It should be format
+                       ``glance://<glance-image-uuid>``
+                       or just ``<glance-image-uuid>``;
+                     * An HTTP URL.
     :param parameters: the parameters to pass in the virtual floppy image
-        in a dictionary.  This is optional.
+                       in a dictionary.  This is optional.
     :raises: ImageCreationFailed, if it failed while creating the floppy image.
     :raises: SwiftOperationError, if any operation with Swift fails.
     :raises: IloOperationError, if attaching virtual media failed.
@@ -716,28 +888,6 @@ def set_secure_boot_mode(task, flag):
                                           error=ilo_exception)
 
 
-def update_secure_boot_mode(task, mode):
-    """Changes secure boot mode for next boot on the node.
-
-    This method changes secure boot mode on the node for next boot. It changes
-    the secure boot mode setting on node only if the deploy has requested for
-    the secure boot.
-    During deploy, this method is used to enable secure boot on the node by
-    passing 'mode' as 'True'.
-    During teardown, this method is used to disable secure boot on the node by
-    passing 'mode' as 'False'.
-
-    :param task: a TaskManager instance containing the node to act on.
-    :param mode: Boolean value requesting the next state for secure boot
-    :raises: IloOperationNotSupported, if operation is not supported on iLO
-    :raises: IloOperationError, if some operation on iLO failed.
-    """
-    if deploy_utils.is_secure_boot_requested(task.node):
-        set_secure_boot_mode(task, mode)
-        LOG.info('Changed secure boot to %(mode)s for node %(node)s',
-                 {'mode': mode, 'node': task.node.uuid})
-
-
 def remove_single_or_list_of_files(file_location):
     """Removes (deletes) the file or list of files.
 
@@ -753,7 +903,7 @@ def remove_single_or_list_of_files(file_location):
         for location in file_location:
             ironic_utils.unlink_without_raise(location)
     # file_location is a single file path
-    elif isinstance(file_location, six.string_types):
+    elif isinstance(file_location, str):
         ironic_utils.unlink_without_raise(file_location)
 
 
@@ -769,8 +919,8 @@ def verify_image_checksum(image_location, expected_checksum):
              verification fails.
     """
     try:
-        with open(image_location, 'rb') as fd:
-            actual_checksum = utils.hash_file(fd)
+        actual_checksum = fileutils.compute_file_checksum(image_location,
+                                                          algorithm='md5')
     except IOError as e:
         LOG.error("Error opening file: %(file)s", {'file': image_location})
         raise exception.ImageRefValidationFailed(image_href=image_location,
@@ -785,3 +935,236 @@ def verify_image_checksum(image_location, expected_checksum):
         LOG.error(msg)
         raise exception.ImageRefValidationFailed(image_href=image_location,
                                                  reason=msg)
+
+
+def get_server_post_state(node):
+    """Get the current state of system POST.
+
+    :param node: an ironic node object.
+    :returns: POST state of the server. The valida states are:-
+        null, Unknown, Reset, PowerOff, InPost, InPostDiscoveryComplete
+        and FinishedPost.
+    :raises: IloOperationError on an error from IloClient library.
+    :raises: IloOperationNotSupported if retrieving post state is not
+        supported on the server.
+    """
+    ilo_object = get_ilo_object(node)
+    operation = _("Get server post state for node %s.") % node.uuid
+    try:
+        return ilo_object.get_host_post_state()
+    except ilo_error.IloCommandNotSupportedError as ilo_exception:
+        raise exception.IloOperationNotSupported(operation=operation,
+                                                 error=ilo_exception)
+    except ilo_error.IloError as ilo_exception:
+        raise exception.IloOperationError(operation=operation,
+                                          error=ilo_exception)
+
+
+def _get_certificate_file_list(cert_file_list):
+    """Get the list of certificates to use.
+
+    :param cert_file_list: certificates file list.
+    :returns: cert_file_list if it's not empty. If empty or None,
+        returns the list of path configured for "webserver_verify_ca"
+        configuration option if the path exists. If the path does not
+        exist, returns empty list.
+    :raises: InvalidParameterValue if argument provided is other than
+        a list.
+    """
+
+    cfl = cert_file_list
+
+    if not cfl:
+        try:
+            verify = strutils.bool_from_string(CONF.webserver_verify_ca,
+                                               strict=True)
+        except ValueError:
+            verify = CONF.webserver_verify_ca
+
+        if isinstance(verify, bool):
+            return []
+
+        if not os.path.exists(verify):
+            LOG.error("Path to the certificate file %(path)s "
+                      "does not exist.", {'path': verify})
+            return []
+
+        cfl = [verify]
+
+    if not isinstance(cfl, list):
+        raise exception.InvalidParameterValue(_(
+            'List of files is expected whereas "%(atype)s" type '
+            'is provided.') % {'atype': type(cfl)})
+
+    return cfl
+
+
+def add_certificates(task, cert_file_list=None):
+    """Adds certificates to the node.
+
+    Adds certificates to the node based on the driver info
+    provided.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param cert_file_list: List of certificates to be added to the node.
+        If None, certificates from path configured in 'webserver_verify_ca'
+        will be added to the node.
+    :raises: IloOperationError on an error from IloClient library.
+    :raises: IloOperationNotSupported if retrieving post state is not
+        supported on the server.
+    :raises: InvalidParameterValue, if any of the required parameters are
+            invalid.
+    """
+
+    node = task.node
+    ilo_object = get_ilo_object(node)
+    d_info = node.driver_info
+
+    export_certs = d_info.get('ilo_add_certificates', True)
+
+    if export_certs is None:
+        export_certs = True
+    else:
+        try:
+            export_certs = strutils.bool_from_string(export_certs,
+                                                     strict=True)
+        except ValueError:
+            raise exception.InvalidParameterValue(
+                _('Invalid value type set in driver_info/'
+                  'ilo_add_certificates on node %(node)s. '
+                  'The value should be a Boolean '
+                  ' not "%(value)s"'
+                  ) % {'value': export_certs, 'node': node.uuid})
+
+    if not export_certs:
+        LOG.info("Adding of certificates to the node %(node)s is not "
+                 "requested. Assuming required certificates are available "
+                 "on the node.", {'node': node.uuid})
+        return
+
+    cfl = _get_certificate_file_list(cert_file_list)
+
+    if not cfl:
+        LOG.debug("Not adding any certificate to the node %(node)s "
+                  "as no certificates are provided", {'node': node.uuid})
+        return
+
+    try:
+        # NOTE(vmud213): Add the certificates to the node which are
+        # eventually being used for TLS verification by the node before
+        # downloading the deploy/instance images during HTTPS boot from
+        # URL.
+        operation = (_("Add certificates to %(node)s from paths "
+                       "%(cpath)s.") % {'cpath': cfl, 'node': node.uuid})
+
+        ilo_object.add_tls_certificate(cfl)
+
+        LOG.info("Successfully added certificates to the node %(node)s from "
+                 "paths %(cpath)s.", {'cpath': cfl, 'node': node.uuid})
+    except ilo_error.IloCommandNotSupportedInBiosError as ilo_exception:
+        raise exception.IloOperationNotSupported(operation=operation,
+                                                 error=ilo_exception)
+    except ilo_error.IloError as ilo_exception:
+        raise exception.IloOperationError(operation=operation,
+                                          error=ilo_exception)
+
+
+def clear_certificates(task, cert_file_list=None):
+    """Clears any certificates added to the node.
+
+    Clears the certificates added to the node as part of any Ironic
+    operation
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param cert_file_list: List of certificates to be removed from node.
+        If None, all the certificates present on the node will be removed.
+    :raises: IloOperationError on an error from IloClient library.
+    :raises: IloOperationNotSupported if retrieving post state is not
+        supported on the server.
+    """
+
+    node = task.node
+    operation = (_("Clearing certificates from node %(node)s.") %
+                 {'node': node.uuid})
+    # NOTE(vmud213): Exclude the certificates used to boot deploy images
+    exclude_cfl = _get_certificate_file_list(None)
+
+    try:
+        ilo_object = get_ilo_object(node)
+        ilo_object.remove_tls_certificate(
+            cert_file_list=cert_file_list, excl_cert_file_list=exclude_cfl)
+    except ilo_error.IloCommandNotSupportedInBiosError as ilo_exception:
+        raise exception.IloOperationNotSupported(operation=operation,
+                                                 error=ilo_exception)
+    except ilo_error.IloError as ilo_exception:
+        raise exception.IloOperationError(operation=operation,
+                                          error=ilo_exception)
+    LOG.info("Cleared TLS certificates from the node %(node)s "
+             "successfully from paths %(cpath)s.",
+             {'node': node.uuid, 'cpath': cert_file_list})
+
+
+def setup_uefi_https(task, iso, persistent=False):
+    """Sets up system to boot from UEFIHTTP boot device.
+
+    Sets the one-time/persistent boot device to UEFIHTTP based
+    on the argument supplied.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param iso: ISO URL to be set to boot from.
+    :param persistent: Indicates whether the system should be set to boot
+        from the given device one-time or each time.
+    :raises: IloOperationError on an error from IloClient library.
+    :raises: IloOperationNotSupported if retrieving post state is not
+        supported on the server.
+    """
+    node = task.node
+    ilo_object = get_ilo_object(node)
+    scheme = urlparse.urlparse(iso).scheme.lower()
+
+    operation = (_("Setting up node %(node)s to boot from URL %(iso)s.") %
+                 {'iso': iso, 'node': node.uuid})
+
+    if scheme != 'https':
+        msg = (_('Error setting up node %(node)s to boot from '
+                 'URL %(iso)s. A secure URL is expected that is exposed '
+                 'over HTTPS.') %
+               {'node': node.uuid, 'iso': iso})
+        raise exception.IloOperationNotSupported(operation=operation,
+                                                 error=msg)
+
+    try:
+        ilo_object.set_http_boot_url(iso)
+        LOG.info("Set the node %(node)s to boot from URL %(iso)s "
+                 "successfully.", {'node': node.uuid, 'iso': iso})
+        if not persistent:
+            ilo_object.set_one_time_boot('UEFIHTTP')
+        else:
+            ilo_object.update_persistent_boot(['UEFIHTTP'])
+
+    except ilo_error.IloCommandNotSupportedInBiosError as ilo_exception:
+        raise exception.IloOperationNotSupported(operation=operation,
+                                                 error=ilo_exception)
+    except ilo_error.IloError as ilo_exception:
+        raise exception.IloOperationError(operation=operation,
+                                          error=ilo_exception)
+
+
+def download(target_file, file_url):
+    """Downloads file based on the scheme.
+
+    It downloads the file (url) to given location.
+    The supported url schemes are file, http, and https.
+    :param target_file: target file for copying the downloaded file.
+    :param file_url: source file url from where file needs to be downloaded.
+    :raises: ImageDownloadFailed, on failure to download the file.
+    """
+    parsed_url = urlparse.urlparse(file_url)
+    if parsed_url.scheme == "file":
+        src_file = parsed_url.path
+        with open(target_file, 'wb') as fd:
+            image_service.FileImageService().download(src_file, fd)
+    elif parsed_url.scheme in ('http', 'https'):
+        src_file = parsed_url.geturl()
+        with open(target_file, 'wb') as fd:
+            image_service.HttpImageService().download(src_file, fd)
