@@ -20,6 +20,7 @@ Utility for caching master images.
 
 import os
 import tempfile
+import threading
 import time
 import uuid
 
@@ -44,6 +45,9 @@ LOG = logging.getLogger(__name__)
 _cache_cleanup_list = []
 
 
+_concurrency_semaphore = threading.Semaphore(CONF.image_download_concurrency)
+
+
 class ImageCache(object):
     """Class handling access to cache for master images."""
 
@@ -84,7 +88,8 @@ class ImageCache(object):
                 with lockutils.lock(img_download_lock_name):
                     _fetch(ctx, href, dest_path, force_raw)
             else:
-                _fetch(ctx, href, dest_path, force_raw)
+                with _concurrency_semaphore:
+                    _fetch(ctx, href, dest_path, force_raw)
             return
 
         # TODO(ghe): have hard links and counts the same behaviour in all fs
@@ -107,11 +112,13 @@ class ImageCache(object):
 
         # TODO(dtantsur): lock expiration time
         with lockutils.lock(img_download_lock_name):
+            img_service = image_service.get_image_service(href, context=ctx)
+            img_info = img_service.show(href)
             # NOTE(vdrok): After rebuild requested image can change, so we
             # should ensure that dest_path and master_path (if exists) are
             # pointing to the same file and their content is up to date
             cache_up_to_date = _delete_master_path_if_stale(master_path, href,
-                                                            ctx)
+                                                            img_info)
             dest_up_to_date = _delete_dest_path_if_stale(master_path,
                                                          dest_path)
 
@@ -129,16 +136,17 @@ class ImageCache(object):
                           {'href': href})
                 return
 
-            LOG.info("Master cache miss for image %(href)s, "
-                     "starting download", {'href': href})
+            LOG.info("Master cache miss for image %(href)s, will download",
+                     {'href': href})
             self._download_image(
-                href, master_path, dest_path, ctx=ctx, force_raw=force_raw)
+                href, master_path, dest_path, img_info,
+                ctx=ctx, force_raw=force_raw)
 
         # NOTE(dtantsur): we increased cache size - time to clean up
         self.clean_up()
 
-    def _download_image(self, href, master_path, dest_path, ctx=None,
-                        force_raw=True):
+    def _download_image(self, href, master_path, dest_path, img_info,
+                        ctx=None, force_raw=True):
         """Download image by href and store at a given path.
 
         This method should be called with uuid-specific lock taken.
@@ -146,6 +154,7 @@ class ImageCache(object):
         :param href: image UUID or href to fetch
         :param master_path: destination master path
         :param dest_path: destination file path
+        :param img_info: image information from the image service
         :param ctx: context
         :param force_raw: boolean value, whether to convert the image to raw
                           format
@@ -156,14 +165,21 @@ class ImageCache(object):
         # TODO(ghe): timeout and retry for downloads
         # TODO(ghe): logging when image cannot be created
         tmp_dir = tempfile.mkdtemp(dir=self.master_dir)
-        tmp_path = os.path.join(tmp_dir, href.split('/')[-1])
+        tmp_path = os.path.join(tmp_dir, os.path.basename(master_path))
 
         try:
-            _fetch(ctx, href, tmp_path, force_raw)
-            # NOTE(dtantsur): no need for global lock here - master_path
-            # will have link count >1 at any moment, so won't be cleaned up
-            os.link(tmp_path, master_path)
-            os.link(master_path, dest_path)
+            with _concurrency_semaphore:
+                _fetch(ctx, href, tmp_path, force_raw)
+
+            if img_info.get('no_cache'):
+                LOG.debug("Caching is disabled for image %s", href)
+                # Cache disabled, link directly to destination
+                os.link(tmp_path, dest_path)
+            else:
+                # NOTE(dtantsur): no need for global lock here - master_path
+                # will have link count >1 at any moment, so won't be cleaned up
+                os.link(tmp_path, master_path)
+                os.link(master_path, dest_path)
         except OSError as exc:
             msg = (_("Could not link image %(img_href)s from %(src_path)s "
                      "to %(dst_path)s, error: %(exc)s") %
@@ -221,6 +237,7 @@ class ImageCache(object):
         """
         threshold = time.time() - self._cache_ttl
         survived = []
+        count = 0
         for file_name, last_used, stat in listing:
             if last_used < threshold:
                 try:
@@ -230,6 +247,7 @@ class ImageCache(object):
                                 "master image cache: %(exc)s",
                                 {'name': file_name, 'exc': exc})
                 else:
+                    count += 1
                     if amount is not None:
                         amount -= stat.st_size
                         if amount <= 0:
@@ -237,6 +255,9 @@ class ImageCache(object):
                             break
             else:
                 survived.append((file_name, last_used, stat))
+        if count:
+            LOG.debug('Removed %(count)d expired file(s) from %(dir)s',
+                      {'count': count, 'dir': self.master_dir})
         return survived, amount
 
     def _clean_up_ensure_cache_size(self, listing, amount):
@@ -259,6 +280,7 @@ class ImageCache(object):
                          for f in os.listdir(self.master_dir))
         total_size = sum(os.path.getsize(f)
                          for f in total_listing)
+        count = 0
         while listing and (total_size > self._cache_size
                            or (amount is not None and amount > 0)):
             file_name, last_used, stat = listing.pop()
@@ -270,6 +292,7 @@ class ImageCache(object):
                             {'name': file_name, 'exc': exc})
             else:
                 total_size -= stat.st_size
+                count += 1
                 if amount is not None:
                     amount -= stat.st_size
 
@@ -279,6 +302,10 @@ class ImageCache(object):
                      "threshold %(expected)d",
                      {'dir': self.master_dir, 'actual': total_size,
                       'expected': self._cache_size})
+        elif count:
+            LOG.debug(
+                'Removed %(count)d file(s) from %(dir)s to free up space',
+                {'count': count, 'dir': self.master_dir})
         return max(amount, 0) if amount is not None else 0
 
 
@@ -314,9 +341,23 @@ def _fetch(context, image_href, path, force_raw=False):
     # then we can firstly clean cache and then invoke images.fetch().
     if force_raw:
         if images.force_raw_will_convert(image_href, path_tmp):
-            required_space = images.converted_size(path_tmp)
+            required_space = images.converted_size(path_tmp, estimate=False)
             directory = os.path.dirname(path_tmp)
-            _clean_up_caches(directory, required_space)
+            try:
+                _clean_up_caches(directory, required_space)
+            except exception.InsufficientDiskSpace:
+
+                # try again with an estimated raw size instead of the full size
+                required_space = images.converted_size(path_tmp, estimate=True)
+                try:
+                    _clean_up_caches(directory, required_space)
+                except exception.InsufficientDiskSpace:
+                    LOG.warning('Not enough space for estimated image size. '
+                                'Consider lowering '
+                                '[DEFAULT]raw_image_growth_factor=%s',
+                                CONF.raw_image_growth_factor)
+                    raise
+
         images.image_to_raw(image_href, path, path_tmp)
     else:
         os.rename(path_tmp, path)
@@ -373,6 +414,13 @@ def clean_up_caches(ctx, directory, images_info):
     _clean_up_caches(directory, total_size)
 
 
+def clean_up_all():
+    """Clean up all entries from all caches."""
+    caches_to_clean = [x[1]() for x in _cache_cleanup_list]
+    for cache in caches_to_clean:
+        cache.clean_up()
+
+
 def cleanup(priority):
     """Decorator method for adding cleanup priority to a class."""
     def _add_property_to_class_func(cls):
@@ -383,12 +431,12 @@ def cleanup(priority):
     return _add_property_to_class_func
 
 
-def _delete_master_path_if_stale(master_path, href, ctx):
+def _delete_master_path_if_stale(master_path, href, img_info):
     """Delete image from cache if it is not up to date with href contents.
 
     :param master_path: path to an image in master cache
     :param href: image href
-    :param ctx: context to use
+    :param img_info: image information from the service
     :returns: True if master_path is up to date with href contents,
         False if master_path was stale and was deleted or it didn't exist
     """
@@ -396,24 +444,25 @@ def _delete_master_path_if_stale(master_path, href, ctx):
         # Glance image contents cannot be updated without changing image's UUID
         return os.path.exists(master_path)
     if os.path.exists(master_path):
-        img_service = image_service.get_image_service(href, context=ctx)
-        img_mtime = img_service.show(href).get('updated_at')
+        img_mtime = img_info.get('updated_at')
         if not img_mtime:
             # This means that href is not a glance image and doesn't have an
-            # updated_at attribute
+            # updated_at attribute. To play on the safe side, redownload the
+            # master copy of the image.
             LOG.warning("Image service couldn't determine last "
-                        "modification time of %(href)s, considering "
-                        "cached image up to date.", {'href': href})
-            return True
-        master_mtime = utils.unix_file_modification_datetime(master_path)
-        if img_mtime <= master_mtime:
-            return True
-        # Delete image from cache as it is outdated
-        LOG.info('Image %(href)s was last modified at %(remote_time)s. '
-                 'Deleting the cached copy "%(cached_file)s since it was '
-                 'last modified at %(local_time)s and may be outdated.',
-                 {'href': href, 'remote_time': img_mtime,
-                  'local_time': master_mtime, 'cached_file': master_path})
+                        "modification time of %(href)s, updating "
+                        "the cached copy %(cached_file)s.",
+                        {'href': href, 'cached_file': master_path})
+        else:
+            master_mtime = utils.unix_file_modification_datetime(master_path)
+            if img_mtime <= master_mtime:
+                return True
+            # Delete image from cache as it is outdated
+            LOG.info('Image %(href)s was last modified at %(remote_time)s. '
+                     'Deleting the cached copy "%(cached_file)s since it was '
+                     'last modified at %(local_time)s and may be outdated.',
+                     {'href': href, 'remote_time': img_mtime,
+                      'local_time': master_mtime, 'cached_file': master_path})
 
         os.unlink(master_path)
     return False
