@@ -13,7 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import base64
 import functools
+import gzip
 import json
 import os
 import shutil
@@ -24,67 +26,70 @@ from ironic_lib import utils as ironic_utils
 from oslo_log import log
 
 from ironic.common import exception
+from ironic.common.glance_service import service_utils
 from ironic.common.i18n import _
 from ironic.common import images
+from ironic.common import states
 from ironic.common import swift
+from ironic.common import utils
 from ironic.conf import CONF
 from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
+from ironic.drivers.modules import image_cache
+from ironic.drivers import utils as driver_utils
 
 LOG = log.getLogger(__name__)
 
 
 class ImageHandler(object):
 
-    _SWIFT_MAP = {
-        "redfish": {
-            "swift_enabled": CONF.redfish.use_swift,
-            "container": CONF.redfish.swift_container,
-            "timeout": CONF.redfish.swift_object_expiry_timeout,
-            "image_subdir": "redfish",
-            "file_permission": CONF.redfish.file_permission,
-            "kernel_params": CONF.redfish.kernel_append_params
-        },
-        "idrac": {
-            "swift_enabled": CONF.redfish.use_swift,
-            "container": CONF.redfish.swift_container,
-            "timeout": CONF.redfish.swift_object_expiry_timeout,
-            "image_subdir": "redfish",
-            "file_permission": CONF.redfish.file_permission,
-            "kernel_params": CONF.redfish.kernel_append_params
-        },
-        "ilo5": {
-            "swift_enabled": not CONF.ilo.use_web_server_for_images,
-            "container": CONF.ilo.swift_ilo_container,
-            "timeout": CONF.ilo.swift_object_expiry_timeout,
-            "image_subdir": "ilo",
-            "file_permission": CONF.ilo.file_permission,
-            "kernel_params": CONF.pxe.pxe_append_params
-        },
-        "ilo": {
-            "swift_enabled": not CONF.ilo.use_web_server_for_images,
-            "container": CONF.ilo.swift_ilo_container,
-            "timeout": CONF.ilo.swift_object_expiry_timeout,
-            "image_subdir": "ilo",
-            "file_permission": CONF.ilo.file_permission,
-            "kernel_params": CONF.pxe.pxe_append_params
-        },
-    }
-
     def __init__(self, driver):
-        self._driver = driver
-        self._container = self._SWIFT_MAP[driver].get("container")
-        self._timeout = self._SWIFT_MAP[driver].get("timeout")
-        self._image_subdir = self._SWIFT_MAP[driver].get("image_subdir")
-        self._file_permission = self._SWIFT_MAP[driver].get("file_permission")
-        # To get the kernel parameters
-        self.kernel_params = self._SWIFT_MAP[driver].get("kernel_params")
+        self.update_driver_config(driver)
 
-    def _is_swift_enabled(self):
-        try:
-            return self._SWIFT_MAP[self._driver].get("swift_enabled")
-        except KeyError:
-            return False
+    def update_driver_config(self, driver):
+        _SWIFT_MAP = {
+            "redfish": {
+                "swift_enabled": CONF.redfish.use_swift,
+                "container": CONF.redfish.swift_container,
+                "timeout": CONF.redfish.swift_object_expiry_timeout,
+                "image_subdir": "redfish",
+                "file_permission": CONF.redfish.file_permission,
+                "kernel_params": CONF.redfish.kernel_append_params
+            },
+            "idrac": {
+                "swift_enabled": CONF.redfish.use_swift,
+                "container": CONF.redfish.swift_container,
+                "timeout": CONF.redfish.swift_object_expiry_timeout,
+                "image_subdir": "redfish",
+                "file_permission": CONF.redfish.file_permission,
+                "kernel_params": CONF.redfish.kernel_append_params
+            },
+            "ilo5": {
+                "swift_enabled": not CONF.ilo.use_web_server_for_images,
+                "container": CONF.ilo.swift_ilo_container,
+                "timeout": CONF.ilo.swift_object_expiry_timeout,
+                "image_subdir": "ilo",
+                "file_permission": CONF.ilo.file_permission,
+                "kernel_params": CONF.ilo.kernel_append_params
+            },
+            "ilo": {
+                "swift_enabled": not CONF.ilo.use_web_server_for_images,
+                "container": CONF.ilo.swift_ilo_container,
+                "timeout": CONF.ilo.swift_object_expiry_timeout,
+                "image_subdir": "ilo",
+                "file_permission": CONF.ilo.file_permission,
+                "kernel_params": CONF.ilo.kernel_append_params
+            },
+        }
+
+        self._driver = driver
+        self.swift_enabled = _SWIFT_MAP[driver].get("swift_enabled")
+        self._container = _SWIFT_MAP[driver].get("container")
+        self._timeout = _SWIFT_MAP[driver].get("timeout")
+        self._image_subdir = _SWIFT_MAP[driver].get("image_subdir")
+        self._file_permission = _SWIFT_MAP[driver].get("file_permission")
+        # To get the kernel parameters
+        self.kernel_params = _SWIFT_MAP[driver].get("kernel_params")
 
     def unpublish_image(self, object_name):
         """Withdraw the image previously made downloadable.
@@ -95,7 +100,7 @@ class ImageHandler(object):
 
         :param object_name: name of the published file (optional)
         """
-        if self._is_swift_enabled():
+        if self.swift_enabled:
             container = self._container
 
             swift_api = swift.SwiftAPI()
@@ -118,6 +123,23 @@ class ImageHandler(object):
 
             ironic_utils.unlink_without_raise(published_file)
 
+    @classmethod
+    def unpublish_image_for_node(cls, node, prefix='', suffix=''):
+        """Withdraw the image previously made downloadable.
+
+        Depending on ironic settings, removes previously published file
+        from where it has been published - Swift or local HTTP server's
+        document root.
+
+        :param node: the node for which image was published.
+        :param prefix: object name prefix.
+        :param suffix: object name suffix.
+        """
+        name = _get_name(node, prefix=prefix, suffix=suffix)
+        cls(node.driver).unpublish_image(name)
+        LOG.debug('Removed image %(name)s for node %(node)s',
+                  {'node': node.uuid, 'name': name})
+
     def _append_filename_param(self, url, filename):
         """Append 'filename=<file>' parameter to given URL.
 
@@ -147,7 +169,7 @@ class ImageHandler(object):
 
         return urlparse.urlunparse(parsed_url)
 
-    def publish_image(self, image_file, object_name):
+    def publish_image(self, image_file, object_name, node_http_url=None):
         """Make image file downloadable.
 
         Depending on ironic settings, pushes given file into Swift or copies
@@ -156,10 +178,13 @@ class ImageHandler(object):
 
         :param image_file: path to file to publish
         :param object_name: name of the published file
+        :param node_http_url: a url to be used to publish the image. If set,
+                              the values from external_http_url and http_url
+                              from CONF.deploy won't be used.
         :return: a URL to download published file
         """
 
-        if self._is_swift_enabled():
+        if self.swift_enabled:
             container = self._container
             timeout = self._timeout
 
@@ -171,6 +196,8 @@ class ImageHandler(object):
                                     object_headers=object_headers)
 
             image_url = swift_api.get_temp_url(container, object_name, timeout)
+            image_url = self._append_filename_param(
+                image_url, os.path.basename(image_file))
 
         else:
             public_dir = os.path.join(CONF.deploy.http_root,
@@ -196,29 +223,36 @@ class ImageHandler(object):
                 shutil.copyfile(image_file, published_file)
                 os.chmod(published_file, self._file_permission)
 
-            image_url = os.path.join(
-                CONF.deploy.http_url, self._image_subdir, object_name)
-
-        image_url = self._append_filename_param(
-            image_url, os.path.basename(image_file))
+            http_url = (node_http_url or CONF.deploy.external_http_url
+                        or CONF.deploy.http_url)
+            image_url = os.path.join(http_url, self._image_subdir, object_name)
 
         return image_url
 
 
-def _get_floppy_image_name(node):
-    """Returns the floppy image name for a given node.
+@image_cache.cleanup(priority=75)
+class ISOImageCache(image_cache.ImageCache):
+
+    def __init__(self):
+        master_path = CONF.deploy.iso_master_path or None
+        super(self.__class__, self).__init__(
+            master_path,
+            # MiB -> B
+            cache_size=CONF.deploy.iso_cache_size * 1024 * 1024,
+            # min -> sec
+            cache_ttl=CONF.deploy.iso_cache_ttl * 60)
+
+
+def _get_name(node, prefix='', suffix=''):
+    """Get an object name for a given node.
 
     :param node: the node for which image name is to be provided.
     """
-    return "image-%s" % node.uuid
-
-
-def _get_iso_image_name(node):
-    """Returns the boot iso image name for a given node.
-
-    :param node: the node for which image name is to be provided.
-    """
-    return "boot-%s.iso" % node.uuid
+    if prefix:
+        name = "%s-%s" % (prefix, node.uuid)
+    else:
+        name = node.uuid
+    return name + suffix
 
 
 def cleanup_iso_image(task):
@@ -226,10 +260,18 @@ def cleanup_iso_image(task):
 
     :param task: A task from TaskManager.
     """
-    iso_object_name = _get_iso_image_name(task.node)
-    img_handler = ImageHandler(task.node.driver)
+    ImageHandler.unpublish_image_for_node(task.node, prefix='boot',
+                                          suffix='.iso')
 
-    img_handler.unpublish_image(iso_object_name)
+
+def override_api_url(params):
+    if not CONF.deploy.external_callback_url:
+        return params
+
+    params = params or {}
+    params[deploy_utils.IPA_URL_PARAM_NAME] = \
+        CONF.deploy.external_callback_url.rstrip('/')
+    return params
 
 
 def prepare_floppy_image(task, params=None):
@@ -251,7 +293,8 @@ def prepare_floppy_image(task, params=None):
     :raises: SwiftOperationError, if any operation with Swift fails.
     :returns: image URL for the floppy image.
     """
-    object_name = _get_floppy_image_name(task.node)
+    object_name = _get_name(task.node, prefix='image')
+    params = override_api_url(params)
 
     LOG.debug("Trying to create floppy image for node "
               "%(node)s", {'node': task.node.uuid})
@@ -263,8 +306,9 @@ def prepare_floppy_image(task, params=None):
         images.create_vfat_image(vfat_image_tmpfile, parameters=params)
 
         img_handler = ImageHandler(task.node.driver)
-
-        image_url = img_handler.publish_image(vfat_image_tmpfile, object_name)
+        node_http_url = task.node.driver_info.get("external_http_url")
+        image_url = img_handler.publish_image(vfat_image_tmpfile, object_name,
+                                              node_http_url)
 
     LOG.debug("Created floppy image %(name)s in Swift for node %(node)s, "
               "exposed as temporary URL "
@@ -280,10 +324,92 @@ def cleanup_floppy_image(task):
 
     :param task: an ironic node object.
     """
-    floppy_object_name = _get_floppy_image_name(task.node)
+    ImageHandler.unpublish_image_for_node(task.node, prefix='image')
+
+
+def prepare_configdrive_image(task, content):
+    """Prepare an image with configdrive.
+
+    Decodes base64 contents and writes it into a disk image that can be
+    attached e.g. to a virtual USB device. Images stored in Swift are
+    downloaded first.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param content: Config drive as a base64-encoded string.
+    :raises: ImageCreationFailed, if it failed while creating the image.
+    :raises: SwiftOperationError, if any operation with Swift fails.
+    :returns: image URL for the image.
+    """
+    with tempfile.TemporaryFile(dir=CONF.tempdir) as comp_tmpfile_obj:
+        if '://' in content:
+            with tempfile.NamedTemporaryFile(dir=CONF.tempdir) as tmpfile2:
+                images.fetch_into(task.context, content, tmpfile2)
+                tmpfile2.flush()
+
+                if utils.file_mime_type(tmpfile2.name) == "text/plain":
+                    tmpfile2.seek(0)
+                    base64.decode(tmpfile2, comp_tmpfile_obj)
+                else:
+                    # A binary image, use it as it is.
+                    return prepare_disk_image(task, tmpfile2.name,
+                                              prefix='configdrive')
+        else:
+            comp_tmpfile_obj.write(base64.b64decode(content))
+        comp_tmpfile_obj.seek(0)
+
+        gz = gzip.GzipFile(fileobj=comp_tmpfile_obj, mode='rb')
+        with tempfile.NamedTemporaryFile(
+                dir=CONF.tempdir, suffix='.img') as image_tmpfile_obj:
+            shutil.copyfileobj(gz, image_tmpfile_obj)
+            image_tmpfile_obj.flush()
+            return prepare_disk_image(task, image_tmpfile_obj.name,
+                                      prefix='configdrive')
+
+
+def prepare_disk_image(task, content, prefix=None):
+    """Prepare an image with the given content.
+
+    If content is already an HTTP URL, return it unchanged.
+
+    :param task: a TaskManager instance containing the node to act on.
+    :param content: Content as a string with a file name or bytes with
+        contents.
+    :param prefix: Prefix to use for the object name.
+    :raises: ImageCreationFailed, if it failed while creating the image.
+    :raises: SwiftOperationError, if any operation with Swift fails.
+    :returns: image URL for the image.
+    """
+    object_name = _get_name(task.node, prefix=prefix)
+
+    LOG.debug("Creating a disk image for node %s", task.node.uuid)
 
     img_handler = ImageHandler(task.node.driver)
-    img_handler.unpublish_image(floppy_object_name)
+    if isinstance(content, str):
+        image_url = img_handler.publish_image(content, object_name)
+    else:
+        with tempfile.NamedTemporaryFile(
+                dir=CONF.tempdir, suffix='.img') as image_tmpfile_obj:
+            image_tmpfile_obj.write(content)
+            image_tmpfile_obj.flush()
+
+            image_tmpfile = image_tmpfile_obj.name
+            image_url = img_handler.publish_image(image_tmpfile, object_name)
+
+    LOG.debug("Created a disk image %(name)s for node %(node)s, "
+              "exposed as URL %(url)s", {'node': task.node.uuid,
+                                         'name': object_name,
+                                         'url': image_url})
+
+    return image_url
+
+
+def cleanup_disk_image(task, prefix=None):
+    """Deletes the image if it was created for the node.
+
+    :param task: an ironic node object.
+    :param prefix: Prefix to use for the object name.
+    """
+    ImageHandler.unpublish_image_for_node(task.node, prefix=prefix)
 
 
 def _prepare_iso_image(task, kernel_href, ramdisk_href,
@@ -299,7 +425,7 @@ def _prepare_iso_image(task, kernel_href, ramdisk_href,
     :param kernel_href: URL or Glance UUID of the kernel to use
     :param ramdisk_href: URL or Glance UUID of the ramdisk to use
     :param bootloader_href: URL or Glance UUID of the EFI bootloader
-         image to use when creating UEFI bootbable ISO
+         image to use when creating UEFI bootable ISO
     :param root_uuid: optional uuid of the root partition.
     :param params: a dictionary containing 'parameter name'->'value'
         mapping to be passed to kernel command line.
@@ -318,55 +444,95 @@ def _prepare_iso_image(task, kernel_href, ramdisk_href,
             "building ISO, or explicit ISO for %(node)s") %
             {'node': task.node.uuid})
 
-    img_handler = ImageHandler(task.node.driver)
-    k_param = img_handler.kernel_params
-
     i_info = task.node.instance_info
+    is_ramdisk_boot = (
+        task.node.provision_state == states.DEPLOYING
+        and deploy_utils.get_boot_option(task.node) == 'ramdisk'
+    )
 
-    # NOTE(TheJulia): Until we support modifying a base iso, most of
-    # this logic actually does nothing in the end. But it should!
-    if deploy_utils.get_boot_option(task.node) == "ramdisk":
-        if not base_iso:
-            kernel_params = "root=/dev/ram0 text "
-            kernel_params += i_info.get("ramdisk_kernel_arguments", "")
-        else:
-            kernel_params = None
-
+    if is_ramdisk_boot:
+        download_source = (i_info.get('ramdisk_image_download_source')
+                           or CONF.deploy.ramdisk_image_download_source)
     else:
-        kernel_params = i_info.get('kernel_append_params', k_param)
+        download_source = CONF.deploy.ramdisk_image_download_source
 
-    if params and not base_iso:
-        kernel_params = ' '.join(
-            (kernel_params, ' '.join(
-                '%s=%s' % kv for kv in params.items())))
+    # NOTE(rpittau): if base_iso is defined as http address, we just access
+    # it directly.
+    if base_iso:
+        scheme = urlparse.urlparse(base_iso).scheme.lower()
+        if scheme == 'swift':
+            # FIXME(dtantsur): iLO supports swift: scheme. In the long run we
+            # should support it for all boot interfaces by using temporary
+            # URLs. Until it's done, return base_iso as it is.
+            return base_iso
+
+        if (download_source == 'swift'
+                and service_utils.is_glance_image(base_iso)):
+            base_iso = (
+                images.get_temp_url_for_glance_image(task.context, base_iso))
+            # get_temp_url_for_glance_image return an HTTP (or HTTPS - doesn't
+            # matter here) image.
+            scheme = 'http'
+
+        if download_source != 'local':
+            if scheme in ('http', 'https'):
+                return base_iso
+            LOG.debug("ramdisk_image_download_source set to http but "
+                      "boot_iso is not an HTTP URL: %(boot_iso)s",
+                      {"boot_iso": base_iso})
+
+    img_handler = ImageHandler(task.node.driver)
 
     boot_mode = boot_mode_utils.get_boot_mode(task.node)
 
-    LOG.debug("Trying to create %(boot_mode)s ISO image for node %(node)s "
-              "with kernel %(kernel_href)s, ramdisk %(ramdisk_href)s, "
-              "bootloader %(bootloader_href)s and kernel params %(params)s"
-              "", {'node': task.node.uuid,
-                   'boot_mode': boot_mode,
-                   'kernel_href': kernel_href,
-                   'ramdisk_href': ramdisk_href,
-                   'bootloader_href': bootloader_href,
-                   'params': kernel_params})
+    with tempfile.TemporaryDirectory(dir=CONF.tempdir) as boot_file_dir:
 
-    with tempfile.NamedTemporaryFile(
-            dir=CONF.tempdir, suffix='.iso') as boot_fileobj:
+        boot_iso_tmp_file = os.path.join(boot_file_dir, 'boot.iso')
+        if base_iso:
+            # NOTE(dtantsur): this should be "params or inject_files", but
+            # params are always populated in the calling code.
+            log_func = LOG.warning if inject_files else LOG.debug
+            log_func('Using pre-built %(boot_mode)s ISO %(iso)s for node '
+                     '%(node)s, custom configuration will not be available',
+                     {'boot_mode': boot_mode, 'node': task.node.uuid,
+                      'iso': base_iso})
 
-        boot_iso_tmp_file = boot_fileobj.name
-        images.create_boot_iso(
-            task.context, boot_iso_tmp_file,
-            kernel_href, ramdisk_href,
-            esp_image_href=bootloader_href,
-            root_uuid=root_uuid,
-            kernel_params=kernel_params,
-            boot_mode=boot_mode,
-            base_iso=base_iso,
-            inject_files=inject_files)
+            ISOImageCache().fetch_image(base_iso, boot_iso_tmp_file,
+                                        ctx=task.context, force_raw=False)
+        else:
+            if is_ramdisk_boot:
+                kernel_params = "root=/dev/ram0 text "
+                kernel_params += i_info.get("ramdisk_kernel_arguments", "")
+            else:
+                kernel_params = driver_utils.get_kernel_append_params(
+                    task.node, default=img_handler.kernel_params)
 
-        iso_object_name = _get_iso_image_name(task.node)
+            if params:
+                kernel_params = ' '.join(
+                    (kernel_params, ' '.join(
+                        ('%s=%s' % kv) if kv[1] is not None else kv[0]
+                        for kv in params.items())))
+
+            LOG.debug(
+                "Trying to create %(boot_mode)s ISO image for node %(node)s "
+                "with kernel %(kernel_href)s, ramdisk %(ramdisk_href)s, "
+                "bootloader %(bootloader_href)s and kernel params %(params)s",
+                {'node': task.node.uuid,
+                 'boot_mode': boot_mode,
+                 'kernel_href': kernel_href,
+                 'ramdisk_href': ramdisk_href,
+                 'bootloader_href': bootloader_href,
+                 'params': kernel_params})
+            images.create_boot_iso(
+                task.context, boot_iso_tmp_file,
+                kernel_href, ramdisk_href,
+                esp_image_href=bootloader_href,
+                root_uuid=root_uuid,
+                kernel_params=kernel_params,
+                boot_mode=boot_mode,
+                inject_files=inject_files)
+
+        iso_object_name = _get_name(task.node, prefix='boot', suffix='.iso')
 
         image_url = img_handler.publish_image(
             boot_iso_tmp_file, iso_object_name)
@@ -385,7 +551,14 @@ def _find_param(param_str, param_dict):
     for param_key in param_dict:
         if param_str in param_key:
             val = param_dict.get(param_key)
-    return val
+            if val is not None:
+                return val
+
+
+_TLS_REMOTE_FILE = 'etc/ironic-python-agent/ironic.crt'
+_TLS_CONFIG_TEMPLATE = """[DEFAULT]
+cafile = /%s
+""" % _TLS_REMOTE_FILE
 
 
 def prepare_deploy_iso(task, params, mode, d_info):
@@ -416,20 +589,29 @@ def prepare_deploy_iso(task, params, mode, d_info):
 
     kernel_str = '%s_kernel' % mode
     ramdisk_str = '%s_ramdisk' % mode
+    iso_str = '%s_iso' % mode
     bootloader_str = 'bootloader'
 
     kernel_href = _find_param(kernel_str, d_info)
     ramdisk_href = _find_param(ramdisk_str, d_info)
+    iso_href = _find_param(iso_str, d_info)
     bootloader_href = _find_param(bootloader_str, d_info)
 
+    params = override_api_url(params)
+
     # TODO(TheJulia): At some point we should support something like
     # boot_iso for the deploy interface, perhaps when we support config
     # injection.
     prepare_iso_image = functools.partial(
         _prepare_iso_image, task, kernel_href, ramdisk_href,
-        bootloader_href=bootloader_href, params=params)
+        bootloader_href=bootloader_href, params=params, base_iso=iso_href)
 
     inject_files = {}
+    if CONF.agent.api_ca_file:
+        inject_files[CONF.agent.api_ca_file] = _TLS_REMOTE_FILE
+        inject_files[_TLS_CONFIG_TEMPLATE.encode('utf-8')] = \
+            'etc/ironic-python-agent.d/ironic-tls.conf'
+
     network_data = task.driver.network.get_node_network_data(task)
     if network_data:
         LOG.debug('Injecting custom network data for node %s',
