@@ -20,7 +20,7 @@ import shlex
 from urllib import parse as urlparse
 
 import eventlet
-from futurist import periodics
+from keystoneauth1 import exceptions as ks_exception
 import openstack
 from oslo_log import log as logging
 
@@ -29,12 +29,13 @@ from ironic.common.i18n import _
 from ironic.common import keystone
 from ironic.common import states
 from ironic.common import utils
+from ironic.conductor import periodics
 from ironic.conductor import task_manager
 from ironic.conductor import utils as cond_utils
 from ironic.conf import CONF
 from ironic.drivers import base
 from ironic.drivers.modules import deploy_utils
-
+from ironic.drivers.modules import inspect_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -66,9 +67,14 @@ def _get_client(context):
     conf['ironic-inspector'] = conf.pop('inspector')
     # TODO(pas-ha) investigate possibility of passing user context here,
     # similar to what neutron/glance-related code does
-    return openstack.connection.Connection(
-        session=session,
-        oslo_conf=conf).baremetal_introspection
+    try:
+        return openstack.connection.Connection(
+            session=session,
+            oslo_conf=conf).baremetal_introspection
+    except ks_exception.DiscoveryFailure as exc:
+        raise exception.ConfigInvalid(
+            _("Could not contact ironic-inspector for version discovery: %s")
+            % exc)
 
 
 def _get_callback_endpoint(client):
@@ -122,7 +128,7 @@ def _tear_down_managed_boot(task):
         LOG.exception('Unable to remove inspection network for node %s',
                       task.node.uuid)
 
-    if CONF.inspector.power_off:
+    if CONF.inspector.power_off and not utils.fast_track_enabled(task.node):
         try:
             cond_utils.node_power_action(task, states.POWER_OFF)
         except Exception as exc:
@@ -153,7 +159,7 @@ def _ironic_manages_boot(task, raise_exc=False):
                   'not support managed boot for in-band inspection or '
                   'the required options are not populated: %(exc)s',
                   {'node': task.node.uuid,
-                   'iface': task.node.boot_interface,
+                   'iface': task.node.get_interface('boot'),
                    'exc': e})
         if raise_exc:
             raise
@@ -166,7 +172,7 @@ def _ironic_manages_boot(task, raise_exc=False):
                   'not support managed boot for in-band inspection or '
                   'the required options are not populated: %(exc)s',
                   {'node': task.node.uuid,
-                   'iface': task.node.network_interface,
+                   'iface': task.node.get_interface('network'),
                    'exc': e})
         if raise_exc:
             raise
@@ -182,9 +188,9 @@ def _parse_kernel_params():
         try:
             key, value = s.split('=', 1)
         except ValueError:
-            raise exception.InvalidParameterValue(
-                _('Invalid key-value pair in extra_kernel_params: %s') % s)
-        result[key] = value
+            result[s] = None
+        else:
+            result[key] = value
     return result
 
 
@@ -195,7 +201,7 @@ def _start_managed_inspection(task):
         endpoint = _get_callback_endpoint(client)
         params = dict(_parse_kernel_params(),
                       **{'ipa-inspection-callback-url': endpoint})
-        if CONF.deploy.fast_track:
+        if utils.fast_track_enabled(task.node):
             params['ipa-api-url'] = deploy_utils.get_ironic_api_url()
 
         cond_utils.node_power_action(task, states.POWER_OFF)
@@ -243,6 +249,21 @@ class Inspector(base.InspectInterface):
         :returns: states.INSPECTWAIT
         :raises: HardwareInspectionFailure on failure
         """
+        try:
+            enabled_macs = task.driver.management.get_mac_addresses(task)
+            if enabled_macs:
+                inspect_utils.create_ports_if_not_exist(task, enabled_macs)
+            else:
+                LOG.warning("Not attempting to create any port as no NICs "
+                            "were discovered in 'enabled' state for node "
+                            "%(node)s: %(mac_data)s",
+                            {'mac_data': enabled_macs,
+                             'node': task.node.uuid})
+
+        except exception.UnsupportedDriverExtension:
+            LOG.debug('Pre-creating ports prior to inspection not supported'
+                      ' on node %s.', task.node.uuid)
+
         ironic_manages_boot = _ironic_manages_boot(
             task, raise_exc=CONF.inspector.require_managed_boot)
 
@@ -276,21 +297,14 @@ class Inspector(base.InspectInterface):
                   'ironic-inspector', {'uuid': node_uuid})
         _get_client(task.context).abort_introspection(node_uuid)
 
-    @periodics.periodic(spacing=CONF.inspector.status_check_period)
-    def _periodic_check_result(self, manager, context):
+    @periodics.node_periodic(
+        purpose='checking hardware inspection status',
+        spacing=CONF.inspector.status_check_period,
+        filters={'provision_state': states.INSPECTWAIT},
+    )
+    def _periodic_check_result(self, task, manager, context):
         """Periodic task checking results of inspection."""
-        filters = {'provision_state': states.INSPECTWAIT}
-        node_iter = manager.iter_nodes(filters=filters)
-
-        for node_uuid, driver, conductor_group in node_iter:
-            try:
-                lock_purpose = 'checking hardware inspection status'
-                with task_manager.acquire(context, node_uuid,
-                                          shared=True,
-                                          purpose=lock_purpose) as task:
-                    _check_status(task)
-            except (exception.NodeLocked, exception.NodeNotFound):
-                continue
+        _check_status(task)
 
 
 def _start_inspection(node_uuid, context):
