@@ -487,6 +487,104 @@ def _get_ipmitool_args(driver_info, pw_file=None):
     return args
 
 
+def _ipmitool_timing_args():
+    if not _is_option_supported('timing'):
+        return []
+
+    if not CONF.ipmi.use_ipmitool_retries:
+        return [
+            '-R', '1',
+            '-N', str(CONF.ipmi.min_command_interval)
+        ]
+
+    timeout = CONF.ipmi.command_retry_timeout
+    interval = CONF.ipmi.min_command_interval
+    num_tries = 0
+    cmd_duration = 0
+
+    while cmd_duration + interval <= timeout:
+        cmd_duration += interval
+        # for each attempt, ipmitool adds one second to the previous
+        # interval value
+        interval += 1
+        num_tries += 1
+
+    return [
+        '-R', str(num_tries),
+        '-N', str(CONF.ipmi.min_command_interval)
+    ]
+
+
+def choose_cipher_suite(actual_cipher_suite):
+    """Gives the possible next avaible cipher suite version.
+
+    Based on CONF.ipmi.cipher_suite_versions and the last cipher suite version
+    used that failed. This function is only called if the node doesn't have
+    cipher_suite set. Starts using the last element of the list and decreasing
+    the index.
+
+    :param actual_cipher_suite: latest cipher suite used in the
+        ipmi call.
+
+    :returns: the next possible cipher suite or None in case of empty
+        configuration.
+    """
+    available_cs_versions = CONF.ipmi.cipher_suite_versions
+    if not available_cs_versions:
+        return None
+
+    if actual_cipher_suite is None:
+        return available_cs_versions[-1]
+    else:
+        try:
+            cs_index = available_cs_versions.index(actual_cipher_suite)
+        except ValueError:
+            return available_cs_versions[-1]
+
+        return available_cs_versions[max(cs_index - 1, 0)]
+
+
+def check_cipher_suite_errors(cmd_stderr):
+    """Checks if the command stderr contains cipher suite errors.
+
+    :param cmd_stderr: The command stderr.
+
+    :returns: True if the cmd_stderr contains a cipher suite error,
+        False otherwise.
+    """
+    cs_errors = ["Unsupported cipher suite ID",
+                 "Error in open session response message :"
+                 " no matching cipher suite"]
+    for cs_err in cs_errors:
+        if cmd_stderr is not None and cs_err in cmd_stderr:
+            return True
+    return False
+
+
+def update_cipher_suite_cmd(actual_cs, args):
+    """Updates variables and the cipher suite cmd.
+
+    This function updates the values for all parameters so they
+    can be used in the next retry of _exec_ipmitool.
+
+    :param actual_cs: a string that represents the cipher suite that was
+        used in the command.
+    :param args: a list that contains the ipmitool command that was executed,
+        it will be modified in-place.
+
+    :returns: the next actual_cs
+    """
+    actual_cs = choose_cipher_suite(actual_cs)
+    if '-C' in args:
+        cs_index = args.index('-C') + 1
+        args[cs_index] = actual_cs
+    else:
+        args.append('-C')
+        args.append(actual_cs)
+
+    return actual_cs
+
+
 def _exec_ipmitool(driver_info, command, check_exit_code=None,
                    kill_on_timeout=False):
     """Execute the ipmitool command.
@@ -505,20 +603,13 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None,
     """
     args = _get_ipmitool_args(driver_info)
 
+    change_cs = (CONF.ipmi.cipher_suite_versions != []
+                 and driver_info.get('cipher_suite') is None)
+    actual_cs = None
+
     timeout = CONF.ipmi.command_retry_timeout
 
-    # specify retry timing more precisely, if supported
-    num_tries = max((timeout // CONF.ipmi.min_command_interval), 1)
-
-    if _is_option_supported('timing'):
-        args.append('-R')
-        if CONF.ipmi.use_ipmitool_retries:
-            args.append(str(num_tries))
-        else:
-            args.append('1')
-
-        args.append('-N')
-        args.append(str(CONF.ipmi.min_command_interval))
+    args.extend(_ipmitool_timing_args())
 
     extra_args = {}
 
@@ -530,6 +621,7 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None,
 
     end_time = (time.time() + timeout)
 
+    num_tries = max((timeout // CONF.ipmi.min_command_interval), 1)
     while True:
         num_tries = num_tries - 1
         # NOTE(tenbrae): ensure that no communications are sent to a BMC more
@@ -552,6 +644,10 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None,
                 out, err = utils.execute(*cmd_args, **extra_args)
                 return out, err
             except processutils.ProcessExecutionError as e:
+                if change_cs and check_cipher_suite_errors(e.stderr):
+                    actual_cs = update_cipher_suite_cmd(actual_cs, args)
+                else:
+                    change_cs = False
                 with excutils.save_and_reraise_exception() as ctxt:
                     err_list = [
                         x for x in (
@@ -866,20 +962,18 @@ def _constructor_checks(driver):
 
 def _allocate_port(task, host=None):
     node = task.node
-    dii = node.driver_internal_info or {}
     allocated_port = console_utils.acquire_port(host=host)
-    dii['allocated_ipmi_terminal_port'] = allocated_port
-    node.driver_internal_info = dii
+    node.set_driver_internal_info('allocated_ipmi_terminal_port',
+                                  allocated_port)
     node.save()
     return allocated_port
 
 
 def _release_allocated_port(task):
     node = task.node
-    dii = node.driver_internal_info or {}
-    allocated_port = dii.pop('allocated_ipmi_terminal_port', None)
+    allocated_port = node.del_driver_internal_info(
+        'allocated_ipmi_terminal_port')
     if allocated_port:
-        node.driver_internal_info = dii
         node.save()
         console_utils.release_port(allocated_port)
 
@@ -921,25 +1015,6 @@ class IPMIPower(base.PowerInterface):
             call).
 
         """
-        # NOTE(TheJulia): Temporary until we promote detect_vendor as
-        # a higher level management method and able to automatically
-        # run detection upon either power state sync or in the enrollment
-        # to management step.
-        try:
-            properties = task.node.properties
-            if not properties.get('vendor'):
-                # We have no vendor stored, so we'll go ahead and
-                # call to store it.
-                vendor = task.driver.management.detect_vendor(task)
-                if vendor:
-                    task.upgrade_lock()
-                    props = task.node.properties
-                    props['vendor'] = vendor
-                    task.node.properties = props
-                    task.node.save()
-        except exception.UnsupportedDriverExtension:
-            pass
-
         driver_info = _parse_driver_info(task.node)
         return _power_status(driver_info)
 
@@ -1092,7 +1167,7 @@ class IPMIManagement(base.ManagementInterface):
             LOG.info('For node %(node_uuid)s, '
                      'driver_info[\'ipmi_disable_boot_timeout\'] is set '
                      'to False, so not sending ipmi boot-timeout-disable',
-                     {'node_uuid', task.node.uuid})
+                     {'node_uuid': task.node.uuid})
 
         ifbd = task.node.driver_info.get('ipmi_force_boot_device', False)
         if strutils.bool_from_string(ifbd):
@@ -1178,16 +1253,18 @@ class IPMIManagement(base.ManagementInterface):
 
         """
         driver_info = task.node.driver_info
-        driver_internal_info = task.node.driver_internal_info
+        node = task.node
         ifbd = driver_info.get('ipmi_force_boot_device', False)
 
         driver_info = _parse_driver_info(task.node)
 
         if (strutils.bool_from_string(ifbd)
-                and driver_internal_info.get('persistent_boot_device')
-                and driver_internal_info.get('is_next_boot_persistent', True)):
+                and node.driver_internal_info.get('persistent_boot_device')
+                and node.driver_internal_info.get('is_next_boot_persistent',
+                                                  True)):
             return {
-                'boot_device': driver_internal_info['persistent_boot_device'],
+                'boot_device': node.driver_internal_info[
+                    'persistent_boot_device'],
                 'persistent': True
             }
 
@@ -1226,12 +1303,7 @@ class IPMIManagement(base.ManagementInterface):
 
     @METRICS.timer('IPMIManagement.detect_vendor')
     def detect_vendor(self, task):
-        """Detects, stores, and returns the hardware vendor.
-
-        If the Node object ``properties`` field does not already contain
-        a ``vendor`` field, then this method is intended to query
-        Detects the BMC hardware vendor and stores the returned value
-        with-in the Node object ``properties`` field if detected.
+        """Detects and returns the hardware vendor.
 
         :param task: A task from TaskManager.
         :raises: InvalidParameterValue if an invalid component, indicator
@@ -1240,20 +1312,11 @@ class IPMIManagement(base.ManagementInterface):
         :returns: String representing the BMC reported Vendor or
                   Manufacturer, otherwise returns None.
         """
-        try:
-            driver_info = _parse_driver_info(task.node)
-            out, err = _exec_ipmitool(driver_info, "mc info")
-            re_obj = re.search("Manufacturer Name .*: (.+)", out)
-            if re_obj:
-                bmc_vendor = str(re_obj.groups('')[0]).lower().split(':')
-                # Pull unparsed data and save the vendor
-                return bmc_vendor[-1]
-        except (exception.PasswordFileFailedToCreate,
-                processutils.ProcessExecutionError) as e:
-            LOG.warning('IPMI get boot device failed to detect vendor '
-                        'of bmc for %(node)s. Error %(err)s',
-                        {'node': task.node.uuid,
-                         'err': e})
+        driver_info = _parse_driver_info(task.node)
+        out, err = _exec_ipmitool(driver_info, "mc info")
+        re_obj = re.search("Manufacturer Name *: (.+)", out)
+        if re_obj:
+            return re_obj.group(1).lower()
 
     @METRICS.timer('IPMIManagement.get_sensors_data')
     def get_sensors_data(self, task):
@@ -1497,6 +1560,13 @@ class IPMIShellinaboxConsole(IPMIConsole):
         if not driver_info['port']:
             driver_info['port'] = _allocate_port(task)
 
+        try:
+            self._exec_stop_console(driver_info)
+        except OSError:
+            # We need to drop any existing sol sessions with sol deactivate.
+            # OSError is raised when sol session is already deactivated,
+            # so we can ignore it.
+            pass
         self._start_console(driver_info,
                             console_utils.start_shellinabox_console)
 
@@ -1514,6 +1584,10 @@ class IPMIShellinaboxConsole(IPMIConsole):
                 _console_pwfile_path(task.node.uuid))
         _release_allocated_port(task)
 
+    def _exec_stop_console(self, driver_info):
+        cmd = "sol deactivate"
+        _exec_ipmitool(driver_info, cmd, check_exit_code=[0, 1])
+
     @METRICS.timer('IPMIShellinaboxConsole.get_console')
     def get_console(self, task):
         """Get the type and connection information about the console."""
