@@ -50,13 +50,8 @@ RESCUE_PROPERTIES = {
                         'required for rescue mode.'),
 }
 OPTIONAL_PROPERTIES = {
-    'kernel_append_params': _("Additional kernel parameters to pass down to "
-                              "instance kernel. These parameters can be "
-                              "consumed by the kernel or by the applications "
-                              "by reading /proc/cmdline. Mind severe cmdline "
-                              "size limit. Overrides "
-                              "[pxe]/kernel_append_params ironic "
-                              "option."),
+    'kernel_append_params': driver_utils.KERNEL_APPEND_PARAMS_DESCRIPTION %
+    {'option_group': 'pxe'},
 }
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
 COMMON_PROPERTIES.update(driver_utils.OPTIONAL_PROPERTIES)
@@ -266,50 +261,6 @@ class PXEBaseMixin(object):
                 anaconda_boot=(boot_option == "kickstart"))
             boot_device = boot_devices.PXE
 
-        elif boot_option != "local":
-            if task.driver.storage.should_write_image(task):
-                # Make sure that the instance kernel/ramdisk is cached.
-                # This is for the takeover scenario for active nodes.
-                instance_image_info = pxe_utils.get_instance_image_info(
-                    task, ipxe_enabled=self.ipxe_enabled)
-                pxe_utils.cache_ramdisk_kernel(task, instance_image_info,
-                                               ipxe_enabled=self.ipxe_enabled)
-
-            # If it's going to PXE boot we need to update the DHCP server
-            dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=self.ipxe_enabled, ip_version=4)
-            dhcp_opts += pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=self.ipxe_enabled, ip_version=6)
-            provider = dhcp_factory.DHCPFactory()
-            provider.update_dhcp(task, dhcp_opts)
-
-            iwdi = task.node.driver_internal_info.get('is_whole_disk_image')
-            try:
-                root_uuid_or_disk_id = task.node.driver_internal_info[
-                    'root_uuid_or_disk_id'
-                ]
-            except KeyError:
-                if not task.driver.storage.should_write_image(task):
-                    pass
-                elif not iwdi:
-                    LOG.warning("The UUID for the root partition can't be "
-                                "found, unable to switch the pxe config from "
-                                "deployment mode to service (boot) mode for "
-                                "node %(node)s", {"node": task.node.uuid})
-                else:
-                    LOG.warning("The disk id for the whole disk image can't "
-                                "be found, unable to switch the pxe config "
-                                "from deployment mode to service (boot) mode "
-                                "for node %(node)s. Booting the instance "
-                                "from disk.", {"node": task.node.uuid})
-                    pxe_utils.clean_up_pxe_config(
-                        task, ipxe_enabled=self.ipxe_enabled)
-                    boot_device = boot_devices.DISK
-            else:
-                pxe_utils.build_service_pxe_config(
-                    task, instance_image_info, root_uuid_or_disk_id,
-                    ipxe_enabled=self.ipxe_enabled)
-                boot_device = boot_devices.PXE
         else:
             # NOTE(dtantsur): create a PXE configuration as a safety net for
             # hardware uncapable of persistent boot. If on a reboot it will try
@@ -364,24 +315,7 @@ class PXEBaseMixin(object):
                     "'kickstart' boot option is set on the node but no "
                     "default kickstart template is specified"))
 
-        # Check the trusted_boot capabilities value.
         deploy_utils.validate_capabilities(node)
-        if deploy_utils.is_trusted_boot_requested(node):
-            # Check if 'boot_option' and boot mode is compatible with
-            # trusted boot.
-            if self.ipxe_enabled:
-                # NOTE(TheJulia): So in theory (huge theory here, not put to
-                # practice or tested), that one can define the kernel as tboot
-                # and define the actual kernel and ramdisk as appended data.
-                # Similar to how one can iPXE load the XEN hypervisor.
-                # tboot mailing list seem to indicate pxe/ipxe support, or
-                # more specifically avoiding breaking the scenarios of use,
-                # but there is also no definitive documentation on the subject.
-                LOG.warning('Trusted boot has been requested for %(node)s in '
-                            'concert with iPXE. This is not a supported '
-                            'configuration for an ironic deployment.',
-                            {'node': node.uuid})
-            pxe_utils.validate_boot_parameters_for_trusted_boot(node)
 
         # Check if we have invalid parameters being passed which will not work
         # for ramdisk configurations.
