@@ -46,19 +46,18 @@ OPTIONAL_PROPERTIES = {
                               "driver should use virtual media USB or floppy "
                               "device for passing configuration information "
                               "to the ramdisk. Defaults to False. Optional."),
-    'kernel_append_params': _("Additional kernel parameters to pass down to "
-                              "instance kernel. These parameters can be "
-                              "consumed by the kernel or by the applications "
-                              "by reading /proc/cmdline. Mind severe cmdline "
-                              "size limit. Overrides "
-                              "[redfish]/kernel_append_params ironic "
-                              "option."),
+    'kernel_append_params': driver_utils.KERNEL_APPEND_PARAMS_DESCRIPTION %
+    {'option_group': 'redfish'},
     'bootloader': _("URL or Glance UUID  of the EFI system partition "
                     "image containing EFI boot loader. This image will be "
                     "used by ironic when building UEFI-bootable ISO "
                     "out of kernel and ramdisk. Required for UEFI "
                     "when deploy_iso is not provided."),
-
+    'external_http_url': _("External URL that is used when the image could "
+                           "be served outside of the provisioning network. "
+                           "If set it will have priority over the following "
+                           "configs: CONF.deploy.external_http_url and "
+                           "CONF.deploy.http_url. Defaults to None.")
 }
 
 RESCUE_PROPERTIES = {
