@@ -19,21 +19,20 @@ import datetime
 import hashlib
 import json
 
+from ironic.api.controllers.v1 import allocation as al_controller
 from ironic.api.controllers.v1 import chassis as chassis_controller
+from ironic.api.controllers.v1 import deploy_template as dt_controller
 from ironic.api.controllers.v1 import node as node_controller
 from ironic.api.controllers.v1 import port as port_controller
 from ironic.api.controllers.v1 import portgroup as portgroup_controller
+from ironic.api.controllers.v1 import utils as api_utils
 from ironic.api.controllers.v1 import volume_connector as vc_controller
 from ironic.api.controllers.v1 import volume_target as vt_controller
-from ironic.drivers import base as drivers_base
 from ironic.tests.unit.db import utils as db_utils
 
 ADMIN_TOKEN = '4562138218392831'
 MEMBER_TOKEN = '4562138218392832'
 
-ADMIN_TOKEN_HASH = hashlib.sha256(ADMIN_TOKEN.encode()).hexdigest()
-MEMBER_TOKEN_HASH = hashlib.sha256(MEMBER_TOKEN.encode()).hexdigest()
-
 ADMIN_BODY = {
     'access': {
         'token': {'id': ADMIN_TOKEN,
@@ -58,21 +57,26 @@ MEMBER_BODY = {
     }
 }
 
+DEFAULT_CACHE_VALUES = {
+    ADMIN_TOKEN: ADMIN_BODY,
+    MEMBER_TOKEN: MEMBER_BODY
+}
+
 
 class FakeMemcache(object):
     """Fake cache that is used for keystone tokens lookup."""
 
-    # NOTE(lucasagomes): keystonemiddleware >= 2.0.0 the token cache
-    # keys are sha256 hashes of the token key. This was introduced in
-    # https://review.openstack.org/#/c/186971
-    _cache = {
-        'tokens/%s' % ADMIN_TOKEN: ADMIN_BODY,
-        'tokens/%s' % ADMIN_TOKEN_HASH: ADMIN_BODY,
-        'tokens/%s' % MEMBER_TOKEN: MEMBER_BODY,
-        'tokens/%s' % MEMBER_TOKEN_HASH: MEMBER_BODY,
-    }
-
-    def __init__(self):
+    def __init__(self, cache_values=None):
+        if not cache_values:
+            cache_values = DEFAULT_CACHE_VALUES
+        self._cache = {}
+        for k, v in cache_values.items():
+            self._cache['tokens/%s' % k] = v
+            # NOTE(lucasagomes): keystonemiddleware >= 2.0.0 the token cache
+            # keys are sha256 hashes of the token key. This was introduced in
+            # https://review.opendev.org/#/c/186971
+            self._cache['tokens/%s' %
+                        hashlib.sha256(k.encode()).hexdigest()] = v
         self.set_key = None
         self.set_value = None
         self.token_expiration = None
@@ -92,65 +96,46 @@ def remove_internal(values, internal):
     return {k: v for (k, v) in values.items() if k not in int_attr}
 
 
+def remove_other_fields(values, allowed_fields):
+    return {k: v for (k, v) in values.items() if k in allowed_fields}
+
+
 def node_post_data(**kw):
     node = db_utils.get_test_node(**kw)
-    # These values are not part of the API object
-    node.pop('version')
-    node.pop('conductor_affinity')
-    node.pop('chassis_id')
-    node.pop('tags')
-    node.pop('traits')
 
     # NOTE(jroll): pop out fields that were introduced in later API versions,
     # unless explicitly requested. Otherwise, these will cause tests using
     # older API versions to fail.
-    for iface in drivers_base.ALL_INTERFACES:
-        name = '%s_interface' % iface
-        if name not in kw:
-            node.pop(name)
-    if 'resource_class' not in kw:
-        node.pop('resource_class')
-    if 'fault' not in kw:
-        node.pop('fault')
+    for field in api_utils.VERSIONED_FIELDS:
+        if field not in kw:
+            node.pop(field, None)
 
-    internal = node_controller.NodePatchType.internal_attrs()
-    return remove_internal(node, internal)
+    return remove_other_fields(
+        node, node_controller.node_schema()['properties'])
 
 
 def port_post_data(**kw):
     port = db_utils.get_test_port(**kw)
-    # These values are not part of the API object
-    port.pop('version')
-    port.pop('node_id')
-    port.pop('portgroup_id')
-    internal = port_controller.PortPatchType.internal_attrs()
-    return remove_internal(port, internal)
+    return remove_other_fields(port,
+                               port_controller.PORT_SCHEMA['properties'])
 
 
 def volume_connector_post_data(**kw):
     connector = db_utils.get_test_volume_connector(**kw)
-    # These values are not part of the API object
-    connector.pop('node_id')
-    connector.pop('version')
-    internal = vc_controller.VolumeConnectorPatchType.internal_attrs()
-    return remove_internal(connector, internal)
+    return remove_other_fields(connector,
+                               vc_controller.CONNECTOR_SCHEMA['properties'])
 
 
 def volume_target_post_data(**kw):
     target = db_utils.get_test_volume_target(**kw)
-    # These values are not part of the API object
-    target.pop('node_id')
-    target.pop('version')
-    internal = vt_controller.VolumeTargetPatchType.internal_attrs()
-    return remove_internal(target, internal)
+    return remove_other_fields(target,
+                               vt_controller.TARGET_SCHEMA['properties'])
 
 
 def chassis_post_data(**kw):
     chassis = db_utils.get_test_chassis(**kw)
-    # version is not part of the API object
-    chassis.pop('version')
-    internal = chassis_controller.ChassisPatchType.internal_attrs()
-    return remove_internal(chassis, internal)
+    return remove_other_fields(
+        chassis, chassis_controller.CHASSIS_SCHEMA['properties'])
 
 
 def post_get_test_node(**kw):
@@ -178,8 +163,8 @@ def portgroup_post_data(**kw):
         if arg not in kw:
             portgroup.pop(arg)
 
-    internal = portgroup_controller.PortgroupPatchType.internal_attrs()
-    return remove_internal(portgroup, internal)
+    return remove_other_fields(
+        portgroup, portgroup_controller.PORTGROUP_SCHEMA['properties'])
 
 
 def post_get_test_portgroup(**kw):
@@ -188,3 +173,32 @@ def post_get_test_portgroup(**kw):
     node = db_utils.get_test_node()
     portgroup['node_uuid'] = kw.get('node_uuid', node['uuid'])
     return portgroup
+
+
+def allocation_post_data(node=None, **kw):
+    """Return an Allocation object without internal attributes."""
+    allocation = db_utils.get_test_allocation(**kw)
+    if node:
+        # This is not a database field, so it has to be handled explicitly
+        allocation['node'] = node
+    return remove_other_fields(
+        allocation, al_controller.ALLOCATION_SCHEMA['properties'])
+
+
+def deploy_template_post_data(**kw):
+    """Return a DeployTemplate object without internal attributes."""
+    template = db_utils.get_test_deploy_template(**kw)
+    # These values are not part of the API object
+    template.pop('version')
+    # Remove internal attributes from each step.
+    step_internal = api_utils.DEPLOY_STEP_SCHEMA['properties']
+    template['steps'] = [remove_other_fields(step, step_internal)
+                         for step in template['steps']]
+    # Remove internal attributes from the template.
+    return remove_other_fields(
+        template, dt_controller.TEMPLATE_SCHEMA['properties'])
+
+
+def post_get_test_deploy_template(**kw):
+    """Return a DeployTemplate object with appropriate attributes."""
+    return deploy_template_post_data(**kw)
