@@ -193,11 +193,11 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertTrue(mock_df.called)
             self.assertFalse(mock_reg.called)
 
-    def test_start_fails_on_no_enabled_interfaces(self):
-        self.config(enabled_boot_interfaces=[])
-        self.assertRaisesRegex(exception.ConfigInvalid,
-                               'options enabled_boot_interfaces',
-                               self.service.init_host)
+    def test_start_with_no_enabled_interfaces(self):
+        self.config(enabled_boot_interfaces=[],
+                    enabled_deploy_interfaces=[],
+                    enabled_hardware_types=['fake-hardware'])
+        self._start_service()
 
     @mock.patch.object(base_manager, 'LOG', autospec=True)
     @mock.patch.object(driver_factory, 'HardwareTypesFactory', autospec=True)
@@ -311,17 +311,6 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertEqual(2, mock_dbapi.call_count)
 
 
-class CheckInterfacesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
-    def test__check_enabled_interfaces_success(self):
-        base_manager._check_enabled_interfaces()
-
-    def test__check_enabled_interfaces_failure(self):
-        self.config(enabled_boot_interfaces=[])
-        self.assertRaisesRegex(exception.ConfigInvalid,
-                               'options enabled_boot_interfaces',
-                               base_manager._check_enabled_interfaces)
-
-
 class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     def test__conductor_service_record_keepalive(self):
         self._start_service()
@@ -411,23 +400,46 @@ class RegisterInterfacesTestCase(mgr_utils.ServiceSetUpMixin,
         esi_mock.side_effect = [
             collections.OrderedDict((
                 ('management', ['fake', 'noop']),
-                ('deploy', ['agent', 'iscsi']),
+                ('deploy', ['direct', 'ansible']),
             )),
             collections.OrderedDict((
                 ('management', ['fake']),
-                ('deploy', ['agent', 'fake']),
+                ('deploy', ['direct', 'fake']),
             )),
         ]
-        default_mock.side_effect = ('fake', 'agent', 'fake', 'agent')
+        default_mock.side_effect = ('fake', 'direct', 'fake', 'direct')
         expected_calls = [
-            mock.call(mock.ANY, 'fake-hardware', 'management',
-                      ['fake', 'noop'], 'fake'),
-            mock.call(mock.ANY, 'fake-hardware', 'deploy', ['agent', 'iscsi'],
-                      'agent'),
-            mock.call(mock.ANY, 'manual-management', 'management', ['fake'],
-                      'fake'),
-            mock.call(mock.ANY, 'manual-management', 'deploy',
-                      ['agent', 'fake'], 'agent'),
+            mock.call(
+                mock.ANY,
+                [{'hardware_type': 'fake-hardware',
+                  'interface_type': 'management',
+                  'interface_name': 'fake',
+                  'default': True},
+                 {'hardware_type': 'fake-hardware',
+                  'interface_type': 'management',
+                  'interface_name': 'noop',
+                  'default': False},
+                 {'hardware_type': 'fake-hardware',
+                  'interface_type': 'deploy',
+                  'interface_name': 'direct',
+                  'default': True},
+                 {'hardware_type': 'fake-hardware',
+                  'interface_type': 'deploy',
+                  'interface_name': 'ansible',
+                  'default': False},
+                 {'hardware_type': 'manual-management',
+                  'interface_type': 'management',
+                  'interface_name': 'fake',
+                  'default': True},
+                 {'hardware_type': 'manual-management',
+                  'interface_type': 'deploy',
+                  'interface_name': 'direct',
+                  'default': True},
+                 {'hardware_type': 'manual-management',
+                  'interface_type': 'deploy',
+                  'interface_name': 'fake',
+                  'default': False}]
+            )
         ]
 
         self.service._register_and_validate_hardware_interfaces(hardware_types)
@@ -448,7 +460,7 @@ class RegisterInterfacesTestCase(mgr_utils.ServiceSetUpMixin,
         esi_mock.side_effect = [
             collections.OrderedDict((
                 ('management', ['fake', 'noop']),
-                ('deploy', ['agent', 'iscsi']),
+                ('deploy', ['direct', 'ansible']),
             )),
         ]
         default_mock.side_effect = exception.NoValidDefaultForInterface("boo")
@@ -519,6 +531,12 @@ class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                        fields.NotificationStatus.START),
              mock.call(mock.ANY, 'console_restore',
                        fields.NotificationStatus.ERROR)])
+        history = objects.NodeHistory.list_by_node_id(self.context,
+                                                      test_node.id)
+        entry = history[0]
+        self.assertEqual('startup failure', entry['event_type'])
+        self.assertEqual('ERROR', entry['severity'])
+        self.assertIsNotNone(entry['event'])
 
     @mock.patch.object(base_manager, 'LOG', autospec=True)
     def test__start_consoles_node_locked(self, log_mock, mock_notify,
@@ -576,3 +594,9 @@ class MiscTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.service._fail_transient_state(states.DEPLOYING, 'unknown err')
         node.refresh()
         self.assertEqual(states.DEPLOYFAIL, node.provision_state)
+        history = objects.NodeHistory.list_by_node_id(self.context,
+                                                      node.id)
+        entry = history[0]
+        self.assertEqual('transition', entry['event_type'])
+        self.assertEqual('ERROR', entry['severity'])
+        self.assertEqual('unknown err', entry['event'])
