@@ -15,6 +15,7 @@ from unittest import mock
 
 from oslo_db.sqlalchemy import utils as db_utils
 from oslo_utils import uuidutils
+import sqlalchemy as sa
 from testtools import matchers
 
 from ironic.common import context
@@ -42,6 +43,25 @@ class UpgradingTestCase(base.DbTestCase):
         self.assertRaises(exception.DatabaseVersionTooOld,
                           self.dbapi.check_versions)
 
+    @mock.patch.object(release_mappings, 'get_object_versions', autospec=True)
+    @mock.patch.object(db_utils, 'column_exists', autospec=True)
+    def test_check_versions_handles_missing_table(
+            self, column_exists, mock_release_mappings):
+        column_exists.side_effect = sa.exc.NoSuchTableError('meow')
+        mock_release_mappings.return_value = {'Node': {'1.0'}}
+        self.assertTrue(
+            self.dbapi.check_versions(permit_initial_version=True))
+        self.assertEqual(1, column_exists.call_count)
+
+    @mock.patch.object(release_mappings, 'get_object_versions', autospec=True)
+    @mock.patch.object(db_utils, 'column_exists', autospec=True)
+    def test_check_versions_raises_missing_table(
+            self, column_exists, mock_release_mappings):
+        column_exists.side_effect = sa.exc.NoSuchTableError('meow')
+        mock_release_mappings.return_value = {'Node': {'1.0', '1.1'}}
+        self.assertRaises(sa.exc.NoSuchTableError, self.dbapi.check_versions)
+        self.assertEqual(1, column_exists.call_count)
+
     def test_check_versions(self):
         for v in self.object_versions['Node']:
             node = utils.create_test_node(uuid=uuidutils.generate_uuid(),
@@ -237,73 +257,3 @@ class UpdateToLatestVersionsTestCase(base.DbTestCase):
         for uuid in nodes:
             node = self.dbapi.get_node_by_uuid(uuid)
             self.assertEqual(self.node_ver, node.version)
-
-
-class MigrateFromIscsiTestCase(base.DbTestCase):
-
-    def setUp(self):
-        super(MigrateFromIscsiTestCase, self).setUp()
-        self.context = context.get_admin_context()
-        self.dbapi = db_api.get_instance()
-        self.config(enabled_deploy_interfaces='direct')
-
-    def test_empty_db(self):
-        self.assertEqual(
-            (0, 0), self.dbapi.migrate_from_iscsi_deploy(self.context, 10))
-
-    def test_already_direct_exists(self):
-        utils.create_test_node(deploy_interface='direct')
-        self.assertEqual(
-            (0, 0), self.dbapi.update_to_latest_versions(self.context, 10))
-
-    def test_migrate_by_2(self):
-        utils.create_test_node(deploy_interface='direct')
-        for _i in range(3):
-            uuid = uuidutils.generate_uuid()
-            utils.create_test_node(uuid=uuid, deploy_interface='iscsi')
-        self.assertEqual(
-            (3, 2), self.dbapi.migrate_from_iscsi_deploy(self.context, 2))
-        self.assertEqual(
-            (1, 1), self.dbapi.migrate_from_iscsi_deploy(self.context, 2))
-
-    def test_migrate_all(self):
-        utils.create_test_node(deploy_interface='direct')
-        for _i in range(3):
-            uuid = uuidutils.generate_uuid()
-            utils.create_test_node(uuid=uuid, deploy_interface='iscsi')
-        self.assertEqual(
-            (3, 3), self.dbapi.migrate_from_iscsi_deploy(self.context, 0))
-
-    def test_migration_impossible(self):
-        self.config(enabled_deploy_interfaces='iscsi')
-        for _i in range(3):
-            uuid = uuidutils.generate_uuid()
-            utils.create_test_node(uuid=uuid, deploy_interface='iscsi')
-        self.assertEqual(
-            (0, 0), self.dbapi.migrate_from_iscsi_deploy(self.context, 0))
-
-    def test_migration_impossible2(self):
-        self.config(image_download_source='swift', group='agent')
-        for _i in range(3):
-            uuid = uuidutils.generate_uuid()
-            utils.create_test_node(uuid=uuid, deploy_interface='iscsi')
-        self.assertEqual(
-            (0, 0), self.dbapi.migrate_from_iscsi_deploy(self.context, 0))
-
-    def test_migration_impossible3(self):
-        self.config(default_deploy_interface='iscsi')
-        for _i in range(3):
-            uuid = uuidutils.generate_uuid()
-            utils.create_test_node(uuid=uuid, deploy_interface='iscsi')
-        self.assertEqual(
-            (0, 0), self.dbapi.migrate_from_iscsi_deploy(self.context, 0))
-
-    def test_force_migration(self):
-        self.config(enabled_deploy_interfaces='iscsi')
-        utils.create_test_node(deploy_interface='direct')
-        for _i in range(3):
-            uuid = uuidutils.generate_uuid()
-            utils.create_test_node(uuid=uuid, deploy_interface='iscsi')
-        self.assertEqual(
-            (3, 3), self.dbapi.migrate_from_iscsi_deploy(self.context, 0,
-                                                         force=True))
