@@ -16,14 +16,18 @@
 
 
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 
 from ironic.common import states
 from ironic.db import api as db_api
 from ironic.drivers import base as drivers_base
+from ironic.objects import allocation
 from ironic.objects import bios
 from ironic.objects import chassis
 from ironic.objects import conductor
+from ironic.objects import deploy_template
 from ironic.objects import node
+from ironic.objects import node_history
 from ironic.objects import port
 from ironic.objects import portgroup
 from ironic.objects import trait
@@ -89,6 +93,10 @@ def get_test_drac_info():
         "drac_protocol": "https",
         "drac_username": "admin",
         "drac_password": "fake",
+        "redfish_address": "1.2.3.4",
+        "redfish_system_id": "/redfish/v1/Systems/System.Embedded.1",
+        "redfish_username": "admin",
+        "redfish_password": "fake"
     }
 
 
@@ -97,7 +105,7 @@ def get_test_irmc_info():
         "irmc_address": "1.2.3.4",
         "irmc_username": "admin0",
         "irmc_password": "fake0",
-        "irmc_port": 80,
+        "irmc_port": "80",
         "irmc_auth_method": "digest",
     }
 
@@ -160,9 +168,10 @@ def get_test_node(**kw):
         "local_gb": "10",
         "memory_mb": "4096",
     }
-    # NOTE(deva): API unit tests confirm that sensitive fields in instance_info
-    #             and driver_info will get scrubbed from the API response
-    #             but other fields (eg, 'foo') do not.
+    # NOTE(tenbrae): API unit tests confirm that sensitive fields in
+    #                instance_info and driver_info will get scrubbed
+    #                from the API response but other fields
+    #                (eg, 'foo') do not.
     fake_instance_info = {
         "configdrive": "TG9yZW0gaXBzdW0gZG9sb3Igc2l0IGFtZXQ=",
         "image_url": "http://example.com/test_image_url",
@@ -214,6 +223,19 @@ def get_test_node(**kw):
         'tags': kw.get('tags', []),
         'resource_class': kw.get('resource_class'),
         'traits': kw.get('traits', []),
+        'automated_clean': kw.get('automated_clean', None),
+        'protected': kw.get('protected', False),
+        'protected_reason': kw.get('protected_reason', None),
+        'conductor': kw.get('conductor'),
+        'owner': kw.get('owner', None),
+        'allocation_id': kw.get('allocation_id'),
+        'description': kw.get('description'),
+        'retired': kw.get('retired', False),
+        'retired_reason': kw.get('retired_reason', None),
+        'lessee': kw.get('lessee', None),
+        'network_data': kw.get('network_data'),
+        'boot_mode': kw.get('boot_mode', None),
+        'secure_boot': kw.get('secure_boot', None),
     }
 
     for iface in drivers_base.ALL_INTERFACES:
@@ -261,6 +283,8 @@ def get_test_port(**kw):
         'pxe_enabled': kw.get('pxe_enabled', True),
         'internal_info': kw.get('internal_info', {"bar": "buzz"}),
         'physical_network': kw.get('physical_network'),
+        'is_smartnic': kw.get('is_smartnic', False),
+        'name': kw.get('name'),
     }
 
 
@@ -405,41 +429,6 @@ def create_test_conductor(**kw):
     return dbapi.register_conductor(conductor)
 
 
-def get_test_ucs_info():
-    return {
-        "ucs_username": "admin",
-        "ucs_password": "password",
-        "ucs_service_profile": "org-root/ls-devstack",
-        "ucs_address": "ucs-b",
-    }
-
-
-def get_test_cimc_info():
-    return {
-        "cimc_username": "admin",
-        "cimc_password": "password",
-        "cimc_address": "1.2.3.4",
-    }
-
-
-def get_test_oneview_properties():
-    return {
-        "cpu_arch": "x86_64",
-        "cpus": "8",
-        "local_gb": "10",
-        "memory_mb": "4096",
-        "capabilities": ("server_hardware_type_uri:fake_sht_uri,"
-                         "enclosure_group_uri:fake_eg_uri,"
-                         "server_profile_template_uri:fake_spt_uri"),
-    }
-
-
-def get_test_oneview_driver_info():
-    return {
-        'server_hardware_uri': 'fake_sh_uri',
-    }
-
-
 def get_test_redfish_info():
     return {
         "redfish_address": "https://example.com",
@@ -488,7 +477,7 @@ def create_test_portgroup(**kw):
 def get_test_node_tag(**kw):
     return {
         # TODO(rloo): Replace None below with the object NodeTag VERSION,
-        #             after this lands: https://review.openstack.org/#/c/233357
+        #             after this lands: https://review.opendev.org/#/c/233357
         'version': kw.get('version', None),
         "tag": kw.get("tag", "tag1"),
         "node_id": kw.get("node_id", "123"),
@@ -580,7 +569,12 @@ def create_test_bios_setting(**kw):
     node_id = bios_setting['node_id']
     version = bios_setting['version']
     settings = [{'name': bios_setting['name'],
-                 'value': bios_setting['value']}]
+                 'value': bios_setting['value'],
+                 'attribute_type': bios_setting['attribute_type'],
+                 'allowable_values': bios_setting['allowable_values'],
+                 'read_only': bios_setting['read_only'],
+                 'reset_required': bios_setting['reset_required'],
+                 'unique': bios_setting['unique']}]
     return dbapi.create_bios_setting_list(node_id, settings, version)[0]
 
 
@@ -589,6 +583,15 @@ def get_test_bios_setting(**kw):
         'node_id': kw.get('node_id', '123'),
         'name': kw.get('name', 'virtualization'),
         'value': kw.get('value', 'on'),
+        'attribute_type': kw.get('attribute_type', 'Enumeration'),
+        'allowable_values': kw.get('allowable_values', ['on', 'off']),
+        'lower_bound': kw.get('lower_bound', None),
+        'max_length': kw.get('max_length', None),
+        'min_length': kw.get('max_length', None),
+        'read_only': kw.get('read_only', False),
+        'reset_required': kw.get('reset_required', True),
+        'unique': kw.get('unique', False),
+        'upper_bound': kw.get('upper_bound', None),
         'version': kw.get('version', bios.BIOSSetting.VERSION),
         'created_at': kw.get('created_at'),
         'updated_at': kw.get('updated_at'),
@@ -601,3 +604,120 @@ def get_test_bios_setting_setting_list():
         {'name': 'hyperthread', 'value': 'enabled'},
         {'name': 'numlock', 'value': 'off'}
     ]
+
+
+def get_test_allocation(**kw):
+    return {
+        'candidate_nodes': kw.get('candidate_nodes', []),
+        'conductor_affinity': kw.get('conductor_affinity'),
+        'created_at': kw.get('created_at'),
+        'extra': kw.get('extra', {}),
+        'id': kw.get('id', 42),
+        'last_error': kw.get('last_error'),
+        'name': kw.get('name'),
+        'node_id': kw.get('node_id'),
+        'resource_class': kw.get('resource_class', 'baremetal'),
+        'state': kw.get('state', 'allocating'),
+        'traits': kw.get('traits', []),
+        'updated_at': kw.get('updated_at'),
+        'uuid': kw.get('uuid', uuidutils.generate_uuid()),
+        'version': kw.get('version', allocation.Allocation.VERSION),
+        'owner': kw.get('owner', None),
+    }
+
+
+def create_test_allocation(**kw):
+    allocation = get_test_allocation(**kw)
+    if 'id' not in kw:
+        del allocation['id']
+    dbapi = db_api.get_instance()
+    return dbapi.create_allocation(allocation)
+
+
+def get_test_deploy_template(**kw):
+    default_uuid = uuidutils.generate_uuid()
+    return {
+        'version': kw.get('version', deploy_template.DeployTemplate.VERSION),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+        'id': kw.get('id', 234),
+        'name': kw.get('name', u'CUSTOM_DT1'),
+        'uuid': kw.get('uuid', default_uuid),
+        'steps': kw.get('steps', [get_test_deploy_template_step(
+            deploy_template_id=kw.get('id', 234))]),
+        'extra': kw.get('extra', {}),
+    }
+
+
+def get_test_deploy_template_step(**kw):
+    return {
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+        'id': kw.get('id', 345),
+        'deploy_template_id': kw.get('deploy_template_id', 234),
+        'interface': kw.get('interface', 'raid'),
+        'step': kw.get('step', 'create_configuration'),
+        'args': kw.get('args', {'logical_disks': []}),
+        'priority': kw.get('priority', 10),
+    }
+
+
+def create_test_deploy_template(**kw):
+    """Create a deployment template in the DB and return DeployTemplate model.
+
+    :param kw: kwargs with overriding values for the deploy template.
+    :returns: Test DeployTemplate DB object.
+    """
+    template = get_test_deploy_template(**kw)
+    dbapi = db_api.get_instance()
+    # Let DB generate an ID if one isn't specified explicitly.
+    if 'id' not in kw:
+        del template['id']
+    if 'steps' not in kw:
+        for step in template['steps']:
+            del step['id']
+            del step['deploy_template_id']
+    else:
+        for kw_step, template_step in zip(kw['steps'], template['steps']):
+            if 'id' not in kw_step:
+                del template_step['id']
+    return dbapi.create_deploy_template(template)
+
+
+def get_test_ibmc_info():
+    return {
+        "ibmc_address": "https://example.com",
+        "ibmc_username": "username",
+        "ibmc_password": "password",
+        "verify_ca": False,
+    }
+
+
+def get_test_history(**kw):
+    return {
+        'id': kw.get('id', 345),
+        'version': kw.get('version', node_history.NodeHistory.VERSION),
+        'uuid': kw.get('uuid', '6f8a5d5c-0f2d-4b2c-a62a-a38e300e3f31'),
+        'node_id': kw.get('node_id', 123),
+        'event': kw.get('event', 'Something is wrong'),
+        'conductor': kw.get('conductor', 'host-1'),
+        'severity': kw.get('severity', 'ERROR'),
+        'event_type': kw.get('event_type', 'provisioning'),
+        'user': kw.get('user', 'fake-user'),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+    }
+
+
+def create_test_history(**kw):
+    """Create test history entry in DB and return NodeHistory DB object.
+
+    :param kw: kwargs with overriding values for port's attributes.
+    :returns: Test NodeHistory DB object.
+    """
+    history = get_test_history(**kw)
+    # Let DB generate ID if it isn't specified explicitly
+    if 'id' not in kw:
+        del history['id']
+    dbapi = db_api.get_instance()
+    return dbapi.create_node_history(history)
