@@ -28,6 +28,8 @@ from oslo_utils import importutils
 import ironic.common.boot_devices
 from ironic.common import exception
 from ironic.common import molds
+from ironic.common import states
+from ironic.conductor import periodics
 from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.drivers.modules import deploy_utils
@@ -889,6 +891,8 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
         mock_get_system.return_value.managers = [fake_manager1, fake_manager2]
         self.management.export_configuration(task, 'edge')
 
+        fake_manager_oem2.export_system_configuration.assert_called_once_with(
+            include_destructive_fields=False)
         mock_save_configuration.assert_called_once_with(
             task,
             'edge',
@@ -1024,7 +1028,7 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
 
         self.management._check_import_configuration_task.assert_not_called()
 
-    @mock.patch.object(drac_mgmt.LOG, 'info', autospec=True)
+    @mock.patch.object(periodics.LOG, 'info', autospec=True)
     @mock.patch.object(task_manager, 'acquire', autospec=True)
     def test__query_import_configuration_status_node_notfound(
             self, mock_acquire, mock_log):
@@ -1047,7 +1051,7 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
         self.management._check_import_configuration_task.assert_not_called()
         self.assertTrue(mock_log.called)
 
-    @mock.patch.object(drac_mgmt.LOG, 'info', autospec=True)
+    @mock.patch.object(periodics.LOG, 'info', autospec=True)
     @mock.patch.object(task_manager, 'acquire', autospec=True)
     def test__query_import_configuration_status_node_locked(
             self, mock_acquire, mock_log):
@@ -1435,6 +1439,48 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
             mock_manager_oem.job_service.delete_jobs.assert_called_once_with(
                 job_ids=['JID_CLEARALL'])
 
+    @mock.patch.object(drac_mgmt, 'LOG', autospec=True)
+    @mock.patch.object(drac_utils, 'redfish_utils', autospec=True)
+    def test_clear_job_queue_missing_attr_verify_step(self,
+                                                      mock_redfish_utils,
+                                                      mock_log):
+        mock_system = mock_redfish_utils.get_system.return_value
+        mock_manager = mock.MagicMock()
+        mock_system.managers = [mock_manager]
+        mock_manager_oem = mock_manager.get_oem_extension.return_value
+        mock_manager_oem.job_service.delete_jobs.side_effect = (
+            exception.RedfishError("Oem/Dell/DellJobService is missing"))
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.provision_state = states.VERIFYING
+            task.driver.management.clear_job_queue(task)
+            mock_log.warning.assert_called_once_with(
+                'iDRAC on node %(node)s does not support '
+                'clearing Lifecycle Controller job queue '
+                'using the idrac-redfish driver. '
+                'If using iDRAC9, consider upgrading firmware. '
+                'If using iDRAC8, consider switching to '
+                'idrac-wsman for management interface if '
+                'possible.',
+                {'node': task.node.uuid})
+
+    @mock.patch.object(drac_mgmt, 'LOG', autospec=True)
+    @mock.patch.object(drac_utils, 'redfish_utils', autospec=True)
+    def test_clear_job_queue_missing_attr_clean_step(self,
+                                                     mock_redfish_utils,
+                                                     mock_log):
+        mock_system = mock_redfish_utils.get_system.return_value
+        mock_manager = mock.MagicMock()
+        mock_system.managers = [mock_manager]
+        mock_manager_oem = mock_manager.get_oem_extension.return_value
+        mock_manager_oem.job_service.delete_jobs.side_effect = (
+            exception.RedfishError("Oem/Dell/DellJobService is missing"))
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.provision_state = states.CLEANING
+            self.assertRaises(ironic.common.exception.RedfishError,
+                              task.driver.management.clear_job_queue, task)
+
     @mock.patch.object(redfish_utils, 'wait_until_get_system_ready',
                        autospec=True)
     @mock.patch.object(drac_utils, 'redfish_utils', autospec=True)
@@ -1449,6 +1495,53 @@ class DracRedfishManagementTestCase(test_utils.BaseDracTest):
             task.driver.management.reset_idrac(task)
             mock_manager_oem.reset_idrac.assert_called_once_with()
 
+    @mock.patch.object(redfish_utils, 'wait_until_get_system_ready',
+                       autospec=True)
+    @mock.patch.object(drac_mgmt, 'LOG', autospec=True)
+    @mock.patch.object(drac_utils, 'redfish_utils', autospec=True)
+    def test_reset_idrac_missing_attr_verify_step(self,
+                                                  mock_redfish_utils,
+                                                  mock_log,
+                                                  mock_wait_system_ready):
+        mock_system = mock_redfish_utils.get_system.return_value
+        mock_manager = mock.MagicMock()
+        mock_system.managers = [mock_manager]
+        mock_manager_oem = mock_manager.get_oem_extension.return_value
+        mock_manager_oem.reset_idrac.side_effect = (
+            exception.RedfishError("Oem/Dell/DelliDRACCardService is missing"))
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.provision_state = states.VERIFYING
+            task.driver.management.reset_idrac(task)
+            mock_log.warning.assert_called_once_with(
+                'iDRAC on node %(node)s does not support '
+                'iDRAC reset using the idrac-redfish driver. '
+                'If using iDRAC9, consider upgrading firmware. '
+                'If using iDRAC8, consider switching to '
+                'idrac-wsman for management interface if '
+                'possible.',
+                {'node': task.node.uuid})
+
+    @mock.patch.object(redfish_utils, 'wait_until_get_system_ready',
+                       autospec=True)
+    @mock.patch.object(drac_mgmt, 'LOG', autospec=True)
+    @mock.patch.object(drac_utils, 'redfish_utils', autospec=True)
+    def test_reset_idrac_missing_attr_clean_step(self,
+                                                 mock_redfish_utils,
+                                                 mock_log,
+                                                 mock_wait_system_ready):
+        mock_system = mock_redfish_utils.get_system.return_value
+        mock_manager = mock.MagicMock()
+        mock_system.managers = [mock_manager]
+        mock_manager_oem = mock_manager.get_oem_extension.return_value
+        mock_manager_oem.reset_idrac.side_effect = (
+            exception.RedfishError("Oem/Dell/DelliDRACCardService is missing"))
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            task.node.provision_state = states.CLEANING
+            self.assertRaises(ironic.common.exception.RedfishError,
+                              task.driver.management.reset_idrac, task)
+
     @mock.patch.object(redfish_utils, 'wait_until_get_system_ready',
                        autospec=True)
     @mock.patch.object(drac_utils, 'redfish_utils', autospec=True)
