@@ -1,3 +1,4 @@
+# Copyright 2022 Hewlett Packard Enterprise Development LP
 # Copyright 2014 Hewlett-Packard Development Company, L.P.
 # All Rights Reserved.
 #
@@ -30,6 +31,7 @@ from oslo_utils import uuidutils
 
 from ironic.common import boot_devices
 from ironic.common import exception
+from ironic.common import image_service
 from ironic.common import images
 from ironic.common import swift
 from ironic.conductor import task_manager
@@ -374,6 +376,22 @@ class IloCommonMethodsTestCase(BaseIloTest):
             expected_info = dict(self.info, **ipmi_info)
             self.assertEqual(expected_info, actual_info)
 
+    def test_update_redfish_properties(self):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=False) as task:
+            redfish_info = {
+                "redfish_address": "1.2.3.4",
+                "redfish_username": "admin",
+                "redfish_password": "fake",
+                "redfish_verify_ca": None,
+                "redfish_system_id": "/redfish/v1/Systems/1"
+            }
+            task.node.driver_info = self.info
+            ilo_common.update_redfish_properties(task)
+            actual_info = task.node.driver_info
+            expected_info = dict(self.info, **redfish_info)
+            self.assertEqual(expected_info, actual_info)
+
     def test__get_floppy_image_name(self):
         image_name_expected = 'image-' + self.node.uuid
         image_name_actual = ilo_common._get_floppy_image_name(self.node)
@@ -1504,3 +1522,37 @@ class IloCommonMethodsTestCase(BaseIloTest):
             self.assertRaises(exception.IloOperationError,
                               ilo_common.setup_uefi_https,
                               task, iso, True)
+
+    @mock.patch.object(image_service, 'FileImageService', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(image_service, 'HttpImageService', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test_download_file_url(self, open_mock, http_mock, file_mock):
+        url = "file:///test1/iLO.crt"
+        target_file = "/a/b/c"
+        fd_mock = mock.MagicMock(spec=io.BytesIO)
+        open_mock.return_value = fd_mock
+        fd_mock.__enter__.return_value = fd_mock
+        ilo_common.download(target_file, url)
+        open_mock.assert_called_once_with(target_file, 'wb')
+        http_mock.assert_not_called()
+        file_mock.return_value.download.assert_called_once_with(
+            "/test1/iLO.crt", fd_mock)
+
+    @mock.patch.object(image_service, 'FileImageService', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(image_service, 'HttpImageService', spec_set=True,
+                       autospec=True)
+    @mock.patch.object(builtins, 'open', autospec=True)
+    def test_download_http_url(self, open_mock, http_mock, file_mock):
+        url = "http://1.1.1.1/iLO.crt"
+        target_file = "/a/b/c"
+        fd_mock = mock.MagicMock(spec=io.BytesIO)
+        open_mock.return_value = fd_mock
+        fd_mock.__enter__.return_value = fd_mock
+        ilo_common.download(target_file, url)
+        http_mock.return_value.download.assert_called_once_with(
+            "http://1.1.1.1/iLO.crt", fd_mock)
+        file_mock.assert_not_called()
+        open_mock.assert_called_once_with(target_file, 'wb')
