@@ -81,7 +81,8 @@ class IloInspectTestCase(test_common.BaseIloTest):
                                                        ilo_object_mock)
             get_capabilities_mock.assert_called_once_with(task.node,
                                                           ilo_object_mock)
-            create_port_mock.assert_called_once_with(task, macs)
+            create_port_mock.assert_called_once_with(
+                task, ['aa:aa:aa:aa:aa:aa', 'bb:bb:bb:bb:bb:bb'])
 
     @mock.patch.object(ilo_inspect.LOG, 'warning',
                        spec_set=True, autospec=True)
@@ -126,7 +127,8 @@ class IloInspectTestCase(test_common.BaseIloTest):
             self.assertTrue(log_mock.called)
             get_capabilities_mock.assert_called_once_with(task.node,
                                                           ilo_object_mock)
-            create_port_mock.assert_called_once_with(task, macs)
+            create_port_mock.assert_called_once_with(
+                task, ['aa:aa:aa:aa:aa:aa', 'bb:bb:bb:bb:bb:bb'])
 
     @mock.patch.object(ilo_inspect.LOG, 'warning',
                        spec_set=True, autospec=True)
@@ -168,10 +170,13 @@ class IloInspectTestCase(test_common.BaseIloTest):
             self.assertTrue(log_mock.called)
             get_capabilities_mock.assert_called_once_with(task.node,
                                                           ilo_object_mock)
-            create_port_mock.assert_called_once_with(task, macs)
+            create_port_mock.assert_called_once_with(
+                task, ['aa:aa:aa:aa:aa:aa', 'bb:bb:bb:bb:bb:bb'])
 
     @mock.patch.object(ilo_inspect.LOG, 'warning',
                        spec_set=True, autospec=True)
+    @mock.patch.object(ilo_inspect, '_get_security_parameters',
+                       spec_set=True, autospec=True)
     @mock.patch.object(ilo_inspect, '_get_capabilities', spec_set=True,
                        autospec=True)
     @mock.patch.object(inspect_utils, 'create_ports_if_not_exist',
@@ -187,21 +192,26 @@ class IloInspectTestCase(test_common.BaseIloTest):
                               get_essential_mock,
                               create_port_mock,
                               get_capabilities_mock,
+                              get_security_params_mock,
                               log_mock):
         ilo_object_mock = get_ilo_object_mock.return_value
         properties = {'memory_mb': '512', 'local_gb': 10,
                       'cpus': '1', 'cpu_arch': 'x86_64'}
         macs = {'NIC.LOM.1.1': 'aa:aa:aa:aa:aa:aa'}
         capabilities = {'server_model': 'Gen10'}
+        security_params = (
+            {'security_parameters': {'Password Complexity': 'ok'}})
         result = {'properties': properties, 'macs': macs}
         get_essential_mock.return_value = result
         get_capabilities_mock.return_value = capabilities
+        get_security_params_mock.return_value = security_params
         power_mock.return_value = states.POWER_ON
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
-            expected_properties = {'memory_mb': '512', 'local_gb': 10,
-                                   'cpus': '1', 'cpu_arch': 'x86_64',
-                                   'capabilities': 'server_model:Gen10'}
+            expected_properties = {
+                'memory_mb': '512', 'local_gb': 10, 'cpus': '1',
+                'cpu_arch': 'x86_64', 'capabilities': 'server_model:Gen10',
+                'security_parameters': {'Password Complexity': 'ok'}}
             task.driver.inspect.inspect_hardware(task)
             self.assertEqual(expected_properties, task.node.properties)
             power_mock.assert_called_once_with(mock.ANY, task)
@@ -210,7 +220,8 @@ class IloInspectTestCase(test_common.BaseIloTest):
             self.assertFalse(log_mock.called)
             get_capabilities_mock.assert_called_once_with(task.node,
                                                           ilo_object_mock)
-            create_port_mock.assert_called_once_with(task, macs)
+            create_port_mock.assert_called_once_with(
+                task, ['aa:aa:aa:aa:aa:aa'])
 
     @mock.patch.object(ilo_inspect, '_get_capabilities', spec_set=True,
                        autospec=True)
@@ -249,7 +260,8 @@ class IloInspectTestCase(test_common.BaseIloTest):
                                                        ilo_object_mock)
             get_capabilities_mock.assert_called_once_with(task.node,
                                                           ilo_object_mock)
-            create_port_mock.assert_called_once_with(task, macs)
+            create_port_mock.assert_called_once_with(
+                task, ['aa:aa:aa:aa:aa:aa', 'bb:bb:bb:bb:bb:bb'])
 
     @mock.patch.object(ilo_inspect, '_get_capabilities', spec_set=True,
                        autospec=True)
@@ -288,7 +300,8 @@ class IloInspectTestCase(test_common.BaseIloTest):
                                                        ilo_object_mock)
             get_capabilities_mock.assert_called_once_with(task.node,
                                                           ilo_object_mock)
-            create_port_mock.assert_called_once_with(task, macs)
+            create_port_mock.assert_called_once_with(
+                task, ['aa:aa:aa:aa:aa:aa', 'bb:bb:bb:bb:bb:bb'])
 
     @mock.patch.object(ilo_inspect, '_get_capabilities', spec_set=True,
                        autospec=True)
@@ -334,7 +347,8 @@ class IloInspectTestCase(test_common.BaseIloTest):
                                                        ilo_object_mock)
             get_capabilities_mock.assert_called_once_with(task.node,
                                                           ilo_object_mock)
-            create_port_mock.assert_called_once_with(task, macs)
+            create_port_mock.assert_called_once_with(
+                task, ['aa:aa:aa:aa:aa:aa', 'bb:bb:bb:bb:bb:bb'])
 
 
 class TestInspectPrivateMethods(test_common.BaseIloTest):
@@ -466,3 +480,15 @@ class TestInspectPrivateMethods(test_common.BaseIloTest):
             expected.update({key: 'true'})
         cap = ilo_inspect._create_supported_capabilities_dict(capabilities)
         self.assertEqual(expected, cap)
+
+    def test___create_supported_capabilities_dict_gpu_capabilities(self):
+        capabilities = {'gpu_Nvidia_count': 1, 'gpu_Nvidia_Tesla_M10_count': 1,
+                        'gpu_Nvidia_Tesla_M10': True}
+        expected = {}
+        expected.update(capabilities)
+        for key in ilo_inspect.CAPABILITIES_KEYS:
+            capabilities.update({key: 'true'})
+            expected.update({key: 'true'})
+        capabilities.update({'unknown_property': 'true'})
+        cap = ilo_inspect._create_supported_capabilities_dict(capabilities)
+        self.assertEqual(expected, cap)
