@@ -93,12 +93,12 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             mock_drives.append(_mock_drive(
                 identity=i, block_size_bytes=512, capacity_bytes=899527000000,
                 media_type='HDD', name='Drive',
-                protocol='Serial Attached SCSI'))
+                protocol=sushy.PROTOCOL_TYPE_SAS))
         for i in [self.drive_id5, self.drive_id6, self.drive_id7]:
             mock_drives.append(_mock_drive(
                 identity=i, block_size_bytes=512, capacity_bytes=479559942144,
                 media_type='SSD', name='Solid State Drive',
-                protocol='Serial AT Attachment'))
+                protocol=sushy.PROTOCOL_TYPE_SATA))
         self.mock_storage.drives = mock_drives
         mock_controller = mock.Mock()
         mock_controller.raid_types = ['RAID1', 'RAID5', 'RAID10']
@@ -294,7 +294,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             ]
         }
         created_volumes = [_mock_volume(
-            '1', raid_type=sushy.RAID_TYPE_RAID5,
+            '1', raid_type=sushy.RAIDType.RAID5,
             capacity_bytes=100 * units.Gi)]
         volumes = mock.MagicMock()
         # Called after volumes created
@@ -442,9 +442,9 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             ]
         }
         created_volumes = [
-            _mock_volume('1', raid_type=sushy.RAID_TYPE_RAID5,
+            _mock_volume('1', raid_type=sushy.RAIDType.RAID5,
                          capacity_bytes=100 * units.Gi),
-            _mock_volume('2', raid_type=sushy.RAID_TYPE_RAID1,
+            _mock_volume('2', raid_type=sushy.RAIDType.RAID1,
                          capacity_bytes=500 * units.Gi)]
         resource = mock.MagicMock(spec=['resource_name'])
         resource.resource_name = 'volume'
@@ -706,10 +706,10 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
         self.mock_storage.volumes.create.return_value = resource
         created_volumes = [
             _mock_volume(
-                '1', raid_type=sushy.RAID_TYPE_RAID10,
+                '1', raid_type=sushy.RAIDType.RAID10,
                 capacity_bytes=50 * units.Gi),
             _mock_volume(
-                '2', raid_type=sushy.RAID_TYPE_RAID5,
+                '2', raid_type=sushy.RAIDType.RAID5,
                 capacity_bytes=100 * units.Gi)]
         # Called after volumes created
         self.mock_storage.volumes.get_members.return_value = created_volumes
@@ -957,10 +957,10 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
         self.mock_storage.volumes.create.return_value = resource
         created_volumes = [
             _mock_volume(
-                '1', raid_type=sushy.RAID_TYPE_RAID5,
+                '1', raid_type=sushy.RAIDType.RAID5,
                 capacity_bytes=100 * units.Gi),
             _mock_volume(
-                '2', raid_type=sushy.RAID_TYPE_RAID1,
+                '2', raid_type=sushy.RAIDType.RAID1,
                 capacity_bytes=500 * units.Gi)]
         # Called after volumes created
         self.mock_storage.volumes.get_members.return_value = created_volumes
@@ -1036,7 +1036,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
         mock_volumes = []
         for i in ["1", "2"]:
             mock_volumes.append(_mock_volume(
-                i, volume_type='Mirrored', raid_type=sushy.RAID_TYPE_RAID1))
+                i, volume_type='Mirrored', raid_type=sushy.RAIDType.RAID1))
         op_apply_time_support = mock.MagicMock()
         op_apply_time_support.mapped_supported_values = [
             sushy.APPLY_TIME_IMMEDIATE, sushy.APPLY_TIME_ON_RESET]
@@ -1088,7 +1088,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
         mock_volumes = []
         for i in ["1", "2"]:
             mock_volumes.append(_mock_volume(
-                i, volume_type='Mirrored', raid_type=sushy.RAID_TYPE_RAID1))
+                i, volume_type='Mirrored', raid_type=sushy.RAIDType.RAID1))
         op_apply_time_support = mock.MagicMock()
         op_apply_time_support.mapped_supported_values = [
             sushy.APPLY_TIME_ON_RESET]
@@ -1395,17 +1395,16 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             self.assertEqual([], task.node.raid_config['logical_disks'])
             self.assertIsNotNone(task.node.raid_config.get('last_updated'))
 
-    @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
     @mock.patch.object(redfish_utils, 'get_task_monitor',
                        autospec=True)
     @mock.patch.object(redfish_raid.RedfishRAID,
                        '_submit_create_configuration', autospec=True)
     @mock.patch.object(redfish_raid.RedfishRAID,
                        '_submit_delete_configuration', autospec=True)
-    @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
+    @mock.patch.object(deploy_utils, 'reboot_to_finish_step', autospec=True)
     def test__check_node_raid_config_pending_create(
-            self, mock_build_agent_opt, mock_submit_delete, mock_submit_create,
-            mock_get_task_monitor, mock_node_power_action, mock_get_system):
+            self, mock_reboot, mock_submit_delete, mock_submit_create,
+            mock_get_task_monitor, mock_get_system):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             task.node.clean_step = {'interface': 'raid',
@@ -1442,7 +1441,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
             mock_submit_create.assert_called_with(
                 raid, task, raid_configs['pending'])
             mock_submit_delete.assert_not_called()
-            mock_build_agent_opt.assert_called_with(task.node)
+            mock_reboot.assert_called_with(task)
             # Not yet updated as in progress
             self.assertEqual({}, task.node.raid_config)
 
@@ -1452,9 +1451,9 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
                        '_submit_create_configuration', autospec=True)
     @mock.patch.object(redfish_raid.RedfishRAID,
                        '_submit_delete_configuration', autospec=True)
-    @mock.patch.object(deploy_utils, 'build_agent_options', autospec=True)
+    @mock.patch.object(deploy_utils, 'reboot_to_finish_step', autospec=True)
     def test__check_node_raid_config_pending_delete(
-            self, mock_build_agent_opt, mock_submit_delete, mock_submit_create,
+            self, mock_reboot, mock_submit_delete, mock_submit_create,
             mock_get_task_monitor, mock_get_system):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
@@ -1481,7 +1480,7 @@ class RedfishRAIDTestCase(db_base.DbTestCase):
 
             mock_submit_create.assert_not_called()
             mock_submit_delete.assert_called_with(raid, task)
-            mock_build_agent_opt.assert_not_called()
+            mock_reboot.assert_not_called()
             # Not yet updated as in progress
             self.assertEqual({}, task.node.raid_config)
 
