@@ -16,6 +16,7 @@
 import collections
 import copy
 import os
+import time
 from unittest import mock
 
 from oslo_config import cfg
@@ -167,64 +168,82 @@ class RedfishUtilsTestCase(db_base.DbTestCase):
         response = redfish_utils.parse_driver_info(self.node)
         self.assertEqual(self.parsed_driver_info, response)
 
-    @mock.patch.object(sushy, 'Sushy', autospec=True)
-    @mock.patch('ironic.drivers.modules.redfish.utils.'
-                'SessionCache._sessions', {})
-    def test_get_system(self, mock_sushy):
-        fake_conn = mock_sushy.return_value
-        fake_system = fake_conn.get_system.return_value
-        response = redfish_utils.get_system(self.node)
-        self.assertEqual(fake_system, response)
-        fake_conn.get_system.assert_called_once_with(
-            '/redfish/v1/Systems/FAKESYSTEM')
+    def test_get_task_monitor(self):
+        redfish_utils._get_connection = mock.Mock()
+        fake_monitor = mock.Mock()
+        redfish_utils._get_connection.return_value = fake_monitor
+        uri = '/redfish/v1/TaskMonitor/FAKEMONITOR'
 
-    @mock.patch.object(sushy, 'Sushy', autospec=True)
-    @mock.patch('ironic.drivers.modules.redfish.utils.'
-                'SessionCache._sessions', {})
-    def test_get_system_resource_not_found(self, mock_sushy):
-        fake_conn = mock_sushy.return_value
-        fake_conn.get_system.side_effect = (
-            sushy.exceptions.ResourceNotFoundError('GET',
-                                                   '/',
-                                                   requests.Response()))
+        response = redfish_utils.get_task_monitor(self.node, uri)
+
+        self.assertEqual(fake_monitor, response)
+
+    def test_get_task_monitor_error(self):
+        redfish_utils._get_connection = mock.Mock()
+        uri = '/redfish/v1/TaskMonitor/FAKEMONITOR'
+        redfish_utils._get_connection.side_effect =\
+            sushy.exceptions.ResourceNotFoundError('GET', uri, mock.Mock())
 
         self.assertRaises(exception.RedfishError,
-                          redfish_utils.get_system, self.node)
-        fake_conn.get_system.assert_called_once_with(
-            '/redfish/v1/Systems/FAKESYSTEM')
+                          redfish_utils.get_task_monitor, self.node, uri)
 
-    @mock.patch.object(sushy, 'Sushy', autospec=True)
-    @mock.patch('ironic.drivers.modules.redfish.utils.'
-                'SessionCache._sessions', {})
-    def test_get_system_multiple_systems(self, mock_sushy):
-        self.node.driver_info.pop('redfish_system_id')
-        fake_conn = mock_sushy.return_value
-        redfish_utils.get_system(self.node)
-        fake_conn.get_system.assert_called_once_with(None)
+    def test_get_update_service(self):
+        redfish_utils._get_connection = mock.Mock()
+        mock_update_service = mock.Mock()
+        redfish_utils._get_connection.return_value = mock_update_service
 
-    @mock.patch('time.sleep', autospec=True)
-    @mock.patch.object(sushy, 'Sushy', autospec=True)
-    @mock.patch('ironic.drivers.modules.redfish.utils.'
-                'SessionCache._sessions', {})
-    def test_get_system_resource_connection_error_retry(self, mock_sushy,
-                                                        mock_sleep):
+        result = redfish_utils.get_update_service(self.node)
+
+        self.assertEqual(mock_update_service, result)
+
+    def test_get_update_service_error(self):
+        redfish_utils._get_connection = mock.Mock()
+        redfish_utils._get_connection.side_effect =\
+            sushy.exceptions.MissingAttributeError
+
+        self.assertRaises(exception.RedfishError,
+                          redfish_utils.get_update_service, self.node)
+
+    def test_get_event_service(self):
+        redfish_utils._get_connection = mock.Mock()
+        mock_event_service = mock.Mock()
+        redfish_utils._get_connection.return_value = mock_event_service
+
+        result = redfish_utils.get_event_service(self.node)
+
+        self.assertEqual(mock_event_service, result)
+
+    def test_get_event_service_error(self):
+        redfish_utils._get_connection = mock.Mock()
+        redfish_utils._get_connection.side_effect =\
+            sushy.exceptions.MissingAttributeError
+
+        self.assertRaises(exception.RedfishError,
+                          redfish_utils.get_event_service, self.node)
+
+
+class RedfishUtilsAuthTestCase(db_base.DbTestCase):
+
+    def setUp(self):
+        super(RedfishUtilsAuthTestCase, self).setUp()
+        # Default configurations
+        self.config(enabled_hardware_types=['redfish'],
+                    enabled_power_interfaces=['redfish'],
+                    enabled_boot_interfaces=['redfish-virtual-media'],
+                    enabled_management_interfaces=['redfish'])
         # Redfish specific configurations
-        self.config(connection_attempts=3, group='redfish')
-
-        fake_conn = mock_sushy.return_value
-        fake_conn.get_system.side_effect = sushy.exceptions.ConnectionError()
-
-        self.assertRaises(exception.RedfishConnectionError,
-                          redfish_utils.get_system, self.node)
-
-        expected_get_system_calls = [
-            mock.call(self.parsed_driver_info['system_id']),
-            mock.call(self.parsed_driver_info['system_id']),
-            mock.call(self.parsed_driver_info['system_id']),
-        ]
-        fake_conn.get_system.assert_has_calls(expected_get_system_calls)
-        mock_sleep.assert_called_with(
-            redfish_utils.CONF.redfish.connection_retry_interval)
+        self.config(connection_attempts=1, group='redfish')
+        self.node = obj_utils.create_test_node(
+            self.context, driver='redfish', driver_info=INFO_DICT)
+        self.parsed_driver_info = {
+            'address': 'https://example.com',
+            'system_id': '/redfish/v1/Systems/FAKESYSTEM',
+            'username': 'username',
+            'password': 'password',
+            'verify_ca': True,
+            'auth_type': 'auto',
+            'node_uuid': self.node.uuid
+        }
 
     @mock.patch.object(sushy, 'Sushy', autospec=True)
     @mock.patch('ironic.drivers.modules.redfish.utils.'
@@ -233,6 +252,7 @@ class RedfishUtilsTestCase(db_base.DbTestCase):
         redfish_utils.get_system(self.node)
         redfish_utils.get_system(self.node)
         self.assertEqual(1, mock_sushy.call_count)
+        self.assertEqual(len(redfish_utils.SessionCache._sessions), 1)
 
     @mock.patch.object(sushy, 'Sushy', autospec=True)
     def test_ensure_new_session_address(self, mock_sushy):
@@ -250,6 +270,21 @@ class RedfishUtilsTestCase(db_base.DbTestCase):
         redfish_utils.get_system(self.node)
         self.assertEqual(2, mock_sushy.call_count)
 
+    @mock.patch.object(sushy, 'Sushy', autospec=True)
+    def test_ensure_new_session_password(self, mock_sushy):
+        d_info = self.node.driver_info
+        d_info['redfish_username'] = 'foo'
+        d_info['redfish_password'] = 'bar'
+        self.node.driver_info = d_info
+        self.node.save()
+        redfish_utils.get_system(self.node)
+        d_info['redfish_password'] = 'foo'
+        self.node.driver_info = d_info
+        self.node.save()
+        redfish_utils.SessionCache._sessions = collections.OrderedDict()
+        redfish_utils.get_system(self.node)
+        self.assertEqual(2, mock_sushy.call_count)
+
     @mock.patch.object(sushy, 'Sushy', autospec=True)
     @mock.patch('ironic.drivers.modules.redfish.utils.'
                 'SessionCache.AUTH_CLASSES', autospec=True)
@@ -340,19 +375,191 @@ class RedfishUtilsTestCase(db_base.DbTestCase):
             auth=mock_basic_auth.return_value
         )
 
-    def test_get_update_service(self):
-        redfish_utils._get_connection = mock.Mock()
-        mock_update_service = mock.Mock()
-        redfish_utils._get_connection.return_value = mock_update_service
 
-        result = redfish_utils.get_update_service(self.node)
+class RedfishUtilsSystemTestCase(db_base.DbTestCase):
 
-        self.assertEqual(mock_update_service, result)
+    def setUp(self):
+        super(RedfishUtilsSystemTestCase, self).setUp()
+        # Default configurations
+        self.config(enabled_hardware_types=['redfish'],
+                    enabled_power_interfaces=['redfish'],
+                    enabled_boot_interfaces=['redfish-virtual-media'],
+                    enabled_management_interfaces=['redfish'])
+        # Redfish specific configurations
+        self.config(connection_attempts=1, group='redfish')
+        self.node = obj_utils.create_test_node(
+            self.context, driver='redfish', driver_info=INFO_DICT)
+        self.parsed_driver_info = {
+            'address': 'https://example.com',
+            'system_id': '/redfish/v1/Systems/FAKESYSTEM',
+            'username': 'username',
+            'password': 'password',
+            'verify_ca': True,
+            'auth_type': 'auto',
+            'node_uuid': self.node.uuid
+        }
 
-    def test_get_update_service_error(self):
-        redfish_utils._get_connection = mock.Mock()
-        redfish_utils._get_connection.side_effect =\
-            sushy.exceptions.MissingAttributeError
+    @mock.patch.object(sushy, 'Sushy', autospec=True)
+    @mock.patch('ironic.drivers.modules.redfish.utils.'
+                'SessionCache._sessions', {})
+    def test_get_system(self, mock_sushy):
+        fake_conn = mock_sushy.return_value
+        fake_system = fake_conn.get_system.return_value
+        response = redfish_utils.get_system(self.node)
+        self.assertEqual(fake_system, response)
+        fake_conn.get_system.assert_called_once_with(
+            '/redfish/v1/Systems/FAKESYSTEM')
+
+    @mock.patch.object(sushy, 'Sushy', autospec=True)
+    @mock.patch('ironic.drivers.modules.redfish.utils.'
+                'SessionCache._sessions', {})
+    def test_get_system_resource_not_found(self, mock_sushy):
+        fake_conn = mock_sushy.return_value
+        fake_conn.get_system.side_effect = (
+            sushy.exceptions.ResourceNotFoundError('GET',
+                                                   '/',
+                                                   requests.Response()))
 
         self.assertRaises(exception.RedfishError,
-                          redfish_utils.get_update_service, self.node)
+                          redfish_utils.get_system, self.node)
+        fake_conn.get_system.assert_called_once_with(
+            '/redfish/v1/Systems/FAKESYSTEM')
+
+    @mock.patch.object(sushy, 'Sushy', autospec=True)
+    @mock.patch('ironic.drivers.modules.redfish.utils.'
+                'SessionCache._sessions', {})
+    def test_get_system_multiple_systems(self, mock_sushy):
+        self.node.driver_info.pop('redfish_system_id')
+        fake_conn = mock_sushy.return_value
+        redfish_utils.get_system(self.node)
+        fake_conn.get_system.assert_called_once_with(None)
+
+    @mock.patch.object(time, 'sleep', lambda seconds: None)
+    @mock.patch.object(sushy, 'Sushy', autospec=True)
+    @mock.patch('ironic.drivers.modules.redfish.utils.'
+                'SessionCache._sessions', {})
+    def test_get_system_resource_connection_error_retry(self, mock_sushy):
+        # Redfish specific configurations
+        self.config(connection_attempts=3, group='redfish')
+
+        fake_conn = mock.Mock()
+        fake_conn.get_system.side_effect = sushy.exceptions.ConnectionError()
+        mock_sushy.return_value = fake_conn
+
+        self.assertRaises(exception.RedfishConnectionError,
+                          redfish_utils.get_system, self.node)
+
+        expected_get_system_calls = [
+            mock.call(self.parsed_driver_info['system_id']),
+            mock.call(self.parsed_driver_info['system_id']),
+            mock.call(self.parsed_driver_info['system_id']),
+        ]
+        fake_conn.get_system.assert_has_calls(expected_get_system_calls)
+        self.assertEqual(fake_conn.get_system.call_count,
+                         redfish_utils.CONF.redfish.connection_attempts)
+
+    @mock.patch.object(time, 'sleep', lambda seconds: None)
+    @mock.patch.object(sushy, 'Sushy', autospec=True)
+    @mock.patch('ironic.drivers.modules.redfish.utils.'
+                'SessionCache._sessions', {})
+    def test_wait_until_get_system_ready(self, mock_sushy):
+        self.config(connection_attempts=2, group='redfish')
+        uri = '/redfish/v1/Systems/FAKESYSTEM'
+        fake_conn = mock_sushy.return_value
+        fake_system = mock.Mock()
+        fake_conn.get_system.side_effect = [
+            sushy.exceptions.BadRequestError('GET', uri, fake_system),
+            fake_system
+        ]
+        response = redfish_utils.wait_until_get_system_ready(self.node)
+        self.assertEqual(fake_system, response)
+        self.assertEqual(fake_conn.get_system.call_count, 2)
+        fake_conn.get_system.assert_called_with(uri)
+
+    @mock.patch.object(time, 'sleep', lambda seconds: None)
+    @mock.patch.object(sushy, 'Sushy', autospec=True)
+    @mock.patch('ironic.drivers.modules.redfish.utils.'
+                'SessionCache._sessions', {})
+    def test_wait_until_get_system_ready_with_connection_error(self,
+                                                               mock_sushy):
+        self.config(connection_attempts=2, group='redfish')
+        uri = '/redfish/v1/Systems/FAKESYSTEM'
+        fake_conn = mock_sushy.return_value
+        fake_system = mock.Mock()
+        fake_conn.get_system.side_effect = [
+            sushy.exceptions.BadRequestError('GET', uri, fake_system),
+            sushy.exceptions.BadRequestError('GET', uri, fake_system)
+        ]
+        self.assertRaises(exception.RedfishConnectionError,
+                          redfish_utils.wait_until_get_system_ready, self.node)
+
+        self.assertEqual(fake_conn.get_system.call_count, 2)
+
+    @mock.patch.object(time, 'sleep', lambda seconds: None)
+    @mock.patch.object(sushy, 'Sushy', autospec=True)
+    @mock.patch('ironic.drivers.modules.redfish.utils.'
+                'SessionCache._sessions', {})
+    def test_get_system_resource_access_error_retry(self, mock_sushy):
+
+        # Sushy access errors HTTP Errors
+        class fake_response(object):
+            status_code = 401
+            body = None
+
+            def json():
+                return {}
+
+        fake_conn = mock_sushy.return_value
+        fake_system = mock.Mock()
+        fake_conn.get_system.side_effect = iter(
+            [
+                sushy.exceptions.AccessError(
+                    method='GET',
+                    url='http://path/to/url',
+                    response=fake_response),
+                fake_system,
+            ])
+
+        self.assertRaises(exception.RedfishError,
+                          redfish_utils.get_system, self.node)
+        # Retry, as in next power sync perhaps
+        client = redfish_utils.get_system(self.node)
+        client('foo')
+
+        expected_get_system_calls = [
+            mock.call(self.parsed_driver_info['system_id']),
+            mock.call(self.parsed_driver_info['system_id']),
+        ]
+        fake_conn.get_system.assert_has_calls(expected_get_system_calls)
+        fake_system.assert_called_with('foo')
+        self.assertEqual(fake_conn.get_system.call_count, 2)
+
+    @mock.patch.object(time, 'sleep', lambda seconds: None)
+    @mock.patch.object(sushy, 'Sushy', autospec=True)
+    @mock.patch('ironic.drivers.modules.redfish.utils.'
+                'SessionCache._sessions', {})
+    def test_get_system_resource_attribute_error(self, mock_sushy):
+
+        fake_conn = mock_sushy.return_value
+        fake_system = mock.Mock()
+        fake_conn.get_system.side_effect = iter(
+            [
+                AttributeError,
+                fake_system,
+            ])
+        # We need to check for AttributeError explicitly as
+        # otherwise we break existing tests if we try to catch
+        # it explicitly.
+        self.assertRaises(exception.RedfishError,
+                          redfish_utils.get_system, self.node)
+        # Retry, as in next power sync perhaps
+        client = redfish_utils.get_system(self.node)
+        client('bar')
+        expected_get_system_calls = [
+            mock.call(self.parsed_driver_info['system_id']),
+            mock.call(self.parsed_driver_info['system_id']),
+        ]
+
+        fake_conn.get_system.assert_has_calls(expected_get_system_calls)
+        fake_system.assert_called_once_with('bar')
+        self.assertEqual(fake_conn.get_system.call_count, 2)
