@@ -105,70 +105,73 @@ class RedfishImageHandlerTestCase(db_base.DbTestCase):
             mock_swift_api.delete_object.assert_called_once_with(
                 'ironic_redfish_container', object_name)
 
-    @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch.object(image_utils, 'shutil', autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(os, 'mkdir', autospec=True)
     def test_publish_image_local_link(
-            self, mock_mkdir, mock_link, mock_shutil, mock_chmod,
-            mock_execute):
+            self, mock_mkdir, mock_link, mock_shutil, mock_chmod):
         self.config(use_swift=False, group='redfish')
         self.config(http_url='http://localhost', group='deploy')
         img_handler_obj = image_utils.ImageHandler(self.node.driver)
+
         url = img_handler_obj.publish_image('file.iso', 'boot.iso')
+
         self.assertEqual(
             'http://localhost/redfish/boot.iso', url)
+
         mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
         mock_link.assert_called_once_with(
             'file.iso', '/httpboot/redfish/boot.iso')
         mock_chmod.assert_called_once_with('file.iso', 0o644)
-        mock_execute.assert_called_once_with(
-            '/usr/sbin/restorecon', '-i', '-R', 'v', '/httpboot/redfish')
 
-    @mock.patch.object(utils, 'execute', autospec=True)
-    @mock.patch.object(os, 'chmod', autospec=True)
-    @mock.patch.object(image_utils, 'shutil', autospec=True)
-    @mock.patch.object(os, 'link', autospec=True)
-    @mock.patch.object(os, 'mkdir', autospec=True)
-    def test_publish_image_local_link_no_restorecon(
-            self, mock_mkdir, mock_link, mock_shutil, mock_chmod,
-            mock_execute):
-        self.config(use_swift=False, group='redfish')
-        self.config(http_url='http://localhost', group='deploy')
-        img_handler_obj = image_utils.ImageHandler(self.node.driver)
-        url = img_handler_obj.publish_image('file.iso', 'boot.iso')
-        self.assertEqual(
-            'http://localhost/redfish/boot.iso', url)
-        mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
-        mock_link.assert_called_once_with(
-            'file.iso', '/httpboot/redfish/boot.iso')
-        mock_chmod.assert_called_once_with('file.iso', 0o644)
-        mock_execute.return_value = FileNotFoundError
-        mock_shutil.assert_not_called()
-
-    @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch.object(image_utils, 'shutil', autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(os, 'mkdir', autospec=True)
     def test_publish_image_external_ip(
-            self, mock_mkdir, mock_link, mock_shutil, mock_chmod,
-            mock_execute):
+            self, mock_mkdir, mock_link, mock_shutil, mock_chmod):
         self.config(use_swift=False, group='redfish')
         self.config(http_url='http://localhost',
                     external_http_url='http://non-local.host',
                     group='deploy')
         img_handler_obj = image_utils.ImageHandler(self.node.driver)
+
         url = img_handler_obj.publish_image('file.iso', 'boot.iso')
+
         self.assertEqual(
             'http://non-local.host/redfish/boot.iso', url)
+
+        mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
+        mock_link.assert_called_once_with(
+            'file.iso', '/httpboot/redfish/boot.iso')
+        mock_chmod.assert_called_once_with('file.iso', 0o644)
+
+    @mock.patch.object(os, 'chmod', autospec=True)
+    @mock.patch.object(image_utils, 'shutil', autospec=True)
+    @mock.patch.object(os, 'link', autospec=True)
+    @mock.patch.object(os, 'mkdir', autospec=True)
+    def test_publish_image_external_ip_node_override(
+            self, mock_mkdir, mock_link, mock_shutil, mock_chmod):
+        self.config(use_swift=False, group='redfish')
+        self.config(http_url='http://localhost',
+                    external_http_url='http://non-local.host',
+                    group='deploy')
+        img_handler_obj = image_utils.ImageHandler(self.node.driver)
+        self.node.driver_info["external_http_url"] = "http://node.override.url"
+
+        override_url = self.node.driver_info.get("external_http_url")
+
+        url = img_handler_obj.publish_image('file.iso', 'boot.iso',
+                                            override_url)
+
+        self.assertEqual(
+            'http://node.override.url/redfish/boot.iso', url)
+
         mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
         mock_link.assert_called_once_with(
             'file.iso', '/httpboot/redfish/boot.iso')
         mock_chmod.assert_called_once_with('file.iso', 0o644)
-        mock_execute.assert_called_once_with(
-            '/usr/sbin/restorecon', '-i', '-R', 'v', '/httpboot/redfish')
 
     @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch.object(image_utils, 'shutil', autospec=True)
@@ -294,8 +297,8 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
 
             object_name = 'image-%s' % task.node.uuid
 
-            mock_publish_image.assert_called_once_with(mock.ANY,
-                                                       mock.ANY, object_name)
+            mock_publish_image.assert_called_once_with(mock.ANY, mock.ANY,
+                                                       object_name, None)
 
             mock_create_vfat_image.assert_called_once_with(
                 mock.ANY, parameters=None)
@@ -318,8 +321,63 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
 
             object_name = 'image-%s' % task.node.uuid
 
-            mock_publish_image.assert_called_once_with(mock.ANY,
-                                                       mock.ANY, object_name)
+            mock_publish_image.assert_called_once_with(mock.ANY, mock.ANY,
+                                                       object_name, None)
+
+            mock_create_vfat_image.assert_called_once_with(
+                mock.ANY, parameters={"ipa-api-url": "http://callback"})
+
+            self.assertEqual(expected_url, url)
+
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_vfat_image', autospec=True)
+    def test_prepare_floppy_image_publish_with_config_external_http_url(
+            self, mock_create_vfat_image, mock_publish_image):
+        self.config(external_callback_url='http://callback/',
+                    external_http_url='http://config.external.url',
+                    group='deploy')
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            expected_url = 'https://config.external.url/c.f?e=f'
+
+            mock_publish_image.return_value = expected_url
+
+            url = image_utils.prepare_floppy_image(task)
+
+            object_name = 'image-%s' % task.node.uuid
+
+            mock_publish_image.assert_called_once_with(mock.ANY, mock.ANY,
+                                                       object_name, None)
+
+            mock_create_vfat_image.assert_called_once_with(
+                mock.ANY, parameters={"ipa-api-url": "http://callback"})
+
+            self.assertEqual(expected_url, url)
+
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_vfat_image', autospec=True)
+    def test_prepare_floppy_image_publish_with_node_external_http_url(
+            self, mock_create_vfat_image, mock_publish_image):
+        self.config(external_callback_url='http://callback/',
+                    external_http_url='http://config.external.url',
+                    group='deploy')
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            task.node.driver_info["external_http_url"] = \
+                "https://node.external"
+            override_url = task.node.driver_info.get("external_http_url")
+            expected_url = '"https://node.external/c.f?e=f'
+
+            mock_publish_image.return_value = expected_url
+
+            url = image_utils.prepare_floppy_image(task)
+
+            object_name = 'image-%s' % task.node.uuid
+
+            mock_publish_image.assert_called_once_with(
+                mock.ANY, mock.ANY, object_name, override_url)
 
             mock_create_vfat_image.assert_called_once_with(
                 mock.ANY, parameters={"ipa-api-url": "http://callback"})
@@ -602,6 +660,29 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
                 inject_files=None)
 
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_boot_iso', autospec=True)
+    def test__prepare_iso_image_kernel_params_defaults(
+            self, mock_create_boot_iso, mock_publish_image):
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            kernel_params = 'network-config=base64-cloudinit-blob'
+
+            task.node.driver_info['kernel_append_params'] = \
+                f'%default% {kernel_params}'
+
+            image_utils._prepare_iso_image(
+                task, 'http://kernel/img', 'http://ramdisk/img',
+                bootloader_href=None, root_uuid=task.node.uuid)
+
+            mock_create_boot_iso.assert_called_once_with(
+                mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
+                boot_mode='uefi', esp_image_href=None,
+                kernel_params=f'nofb nomodeset vga=normal {kernel_params}',
+                root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
+                inject_files=None)
+
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
