@@ -133,15 +133,17 @@ class iPXEBootTestCase(db_base.DbTestCase):
 
     @mock.patch.object(image_service.GlanceImageService, 'show', autospec=True)
     @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
-                return_value='ramdisk', autospec=True)
+                autospec=True)
     def test_validate_with_boot_iso(self, mock_boot_option, mock_glance):
-        i_info = self.node.driver_info
-        i_info['boot_iso'] = "http://localhost:1234/boot.iso"
+        self.node.instance_info = {
+            'boot_iso': "glance://image"
+        }
+        self.node.save()
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             task.driver.boot.validate(task)
-            self.assertTrue(mock_boot_option.called)
-            self.assertTrue(mock_glance.called)
+            mock_boot_option.assert_called_with(task.node)
+            mock_glance.assert_not_called()
 
     @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
                 return_value='ramdisk', autospec=True)
@@ -167,16 +169,6 @@ class iPXEBootTestCase(db_base.DbTestCase):
             task.driver.boot.validate(task)
             mock_boot_option.assert_called_with(task.node)
 
-    @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
-                return_value='netboot', autospec=True)
-    def test_validate_fail_missing_image_source(self, mock_boot_option):
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            del task.node['instance_info']['image_source']
-            self.assertRaises(exception.MissingParameterValue,
-                              task.driver.boot.validate, task)
-            mock_boot_option.assert_called_with(task.node)
-
     def test_validate_fail_no_port(self):
         new_node = obj_utils.create_test_node(
             self.context,
@@ -188,71 +180,6 @@ class iPXEBootTestCase(db_base.DbTestCase):
             self.assertRaises(exception.MissingParameterValue,
                               task.driver.boot.validate, task)
 
-    def test_validate_fail_trusted_boot_with_secure_boot(self):
-        instance_info = {"boot_option": "netboot",
-                         "secure_boot": "true",
-                         "trusted_boot": "true"}
-        properties = {'capabilities': 'trusted_boot:true'}
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            task.node.instance_info['capabilities'] = instance_info
-            task.node.properties = properties
-            task.node.driver_internal_info['is_whole_disk_image'] = False
-            self.assertRaises(exception.InvalidParameterValue,
-                              task.driver.boot.validate, task)
-
-    def test_validate_fail_invalid_trusted_boot_value(self):
-        properties = {'capabilities': 'trusted_boot:value'}
-        instance_info = {"trusted_boot": "value"}
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            task.node.properties = properties
-            task.node.instance_info['capabilities'] = instance_info
-            self.assertRaises(exception.InvalidParameterValue,
-                              task.driver.boot.validate, task)
-
-    @mock.patch.object(image_service.GlanceImageService, 'show',
-                       autospec=True)
-    def test_validate_fail_no_image_kernel_ramdisk_props(self, mock_glance):
-        instance_info = {"boot_option": "netboot"}
-        mock_glance.return_value = {'properties': {}}
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            task.node.instance_info['capabilities'] = instance_info
-            self.assertRaises(exception.MissingParameterValue,
-                              task.driver.boot.validate,
-                              task)
-
-    @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
-                return_value='netboot', autospec=True)
-    @mock.patch.object(image_service.GlanceImageService, 'show',
-                       autospec=True)
-    def test_validate_fail_glance_image_doesnt_exists(self, mock_glance,
-                                                      mock_boot_option):
-        mock_glance.side_effect = exception.ImageNotFound('not found')
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            self.assertRaises(exception.InvalidParameterValue,
-                              task.driver.boot.validate, task)
-            mock_boot_option.assert_called_with(task.node)
-
-    @mock.patch('ironic.drivers.modules.deploy_utils.get_boot_option',
-                return_value='netboot', autospec=True)
-    @mock.patch.object(image_service.GlanceImageService, 'show',
-                       autospec=True)
-    def test_validate_fail_glance_conn_problem(self, mock_glance,
-                                               mock_boot_option):
-        exceptions = (exception.GlanceConnectionFailed('connection fail'),
-                      exception.ImageNotAuthorized('not authorized'),
-                      exception.Invalid('invalid'))
-        mock_glance.side_effect = exceptions
-        for exc in exceptions:
-            with task_manager.acquire(self.context, self.node.uuid,
-                                      shared=True) as task:
-                self.assertRaises(exception.InvalidParameterValue,
-                                  task.driver.boot.validate, task)
-                mock_boot_option.assert_called_with(task.node)
-
     def test_validate_inspection(self):
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.driver.boot.validate_inspection(task)
@@ -390,10 +317,11 @@ class iPXEBootTestCase(db_base.DbTestCase):
             os.path.join(
                 CONF.deploy.http_root,
                 os.path.basename(CONF.pxe.ipxe_boot_script)),
-            'foo')
+            'foo', 0o644)
         render_mock.assert_called_once_with(
             CONF.pxe.ipxe_boot_script,
-            {'ipxe_for_mac_uri': 'pxelinux.cfg/'})
+            {'ipxe_for_mac_uri': 'pxelinux.cfg/',
+             'ipxe_fallback_script': None})
 
     @mock.patch.object(os.path, 'isfile', lambda path: False)
     @mock.patch('ironic.common.utils.file_has_content', autospec=True)
@@ -410,10 +338,11 @@ class iPXEBootTestCase(db_base.DbTestCase):
             os.path.join(
                 CONF.deploy.http_root,
                 os.path.basename(CONF.pxe.ipxe_boot_script)),
-            'foo')
+            'foo', 0o644)
         render_mock.assert_called_once_with(
             CONF.pxe.ipxe_boot_script,
-            {'ipxe_for_mac_uri': 'pxelinux.cfg/'})
+            {'ipxe_for_mac_uri': 'pxelinux.cfg/',
+             'ipxe_fallback_script': None})
 
     @mock.patch.object(os.path, 'isfile', lambda path: True)
     @mock.patch.object(common_utils, 'file_has_content', lambda *args: True)
@@ -437,7 +366,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
             os.path.join(
                 CONF.deploy.http_root,
                 os.path.basename(CONF.pxe.ipxe_boot_script)),
-            'foo')
+            'foo', 0o644)
 
     @mock.patch.object(common_utils, 'render_template', lambda *args: 'foo')
     @mock.patch('ironic.common.utils.write_to_file', autospec=True)
@@ -451,7 +380,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
             os.path.join(
                 CONF.deploy.http_root,
                 os.path.basename(CONF.pxe.ipxe_boot_script)),
-            'foo')
+            'foo', 0o644)
 
     def test_prepare_ramdisk_cleaning(self):
         self.node.provision_state = states.CLEANING
@@ -582,164 +511,168 @@ class iPXEBootTestCase(db_base.DbTestCase):
         self.node.save()
         self._test_clean_up_ramdisk(mode='rescue')
 
-    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
-    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
-    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
-    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
-    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_netboot(
-            self, get_image_info_mock, cache_mock,
-            dhcp_factory_mock, switch_pxe_config_mock,
-            set_boot_device_mock):
-        provider_mock = mock.MagicMock()
-        dhcp_factory_mock.return_value = provider_mock
-        image_info = {'kernel': ('', '/path/to/kernel'),
-                      'ramdisk': ('', '/path/to/ramdisk')}
-        instance_info = {"boot_option": "netboot"}
-        get_image_info_mock.return_value = image_info
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=True)
-            dhcp_opts += pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=True, ip_version=6)
-            pxe_config_path = pxe_utils.get_pxe_config_file_path(
-                task.node.uuid, ipxe_enabled=True)
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.instance_info['capabilities'] = instance_info
-            task.node.driver_internal_info['root_uuid_or_disk_id'] = (
-                "30212642-09d3-467f-8e09-21685826ab50")
-            task.node.driver_internal_info['is_whole_disk_image'] = False
-
-            task.driver.boot.prepare_instance(task)
-
-            get_image_info_mock.assert_called_once_with(
-                task, ipxe_enabled=True)
-            cache_mock.assert_called_once_with(task, image_info,
-                                               ipxe_enabled=True)
-            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
-            switch_pxe_config_mock.assert_called_once_with(
-                pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'bios', False, False, False, False, ipxe_enabled=True,
-                anaconda_boot=False)
-            set_boot_device_mock.assert_called_once_with(task,
-                                                         boot_devices.PXE,
-                                                         persistent=True)
-
-    @mock.patch('os.path.isfile', return_value=False, autospec=True)
     @mock.patch.object(pxe_utils, 'create_pxe_config', autospec=True)
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
     @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
     @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
     @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_netboot_active(
+    def test_prepare_instance_ramdisk(
             self, get_image_info_mock, cache_mock,
             dhcp_factory_mock, switch_pxe_config_mock,
-            set_boot_device_mock, create_pxe_config_mock, isfile_mock):
+            set_boot_device_mock, mock_create_pxe_config):
         provider_mock = mock.MagicMock()
         dhcp_factory_mock.return_value = provider_mock
         image_info = {'kernel': ('', '/path/to/kernel'),
                       'ramdisk': ('', '/path/to/ramdisk')}
-        instance_info = {"boot_option": "netboot"}
+        i_info_caps = {"boot_mode": "bios"}
+        kernel_arg = "meow"
         get_image_info_mock.return_value = image_info
-        self.node.provision_state = states.ACTIVE
-        self.node.save()
+
         with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.node.properties['capabilities'] = 'boot_mode:bios'
+            i_info = task.node.instance_info
+            i_info['capabilities'] = i_info_caps
+            i_info['kernel_append_params'] = kernel_arg
+            task.node.instance_info = i_info
+            task.node.deploy_interface = 'ramdisk'
+            task.node.save()
             dhcp_opts = pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=True)
             dhcp_opts += pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=True, ip_version=6)
             pxe_config_path = pxe_utils.get_pxe_config_file_path(
                 task.node.uuid, ipxe_enabled=True)
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.instance_info['capabilities'] = instance_info
-            task.node.driver_internal_info['root_uuid_or_disk_id'] = (
-                "30212642-09d3-467f-8e09-21685826ab50")
-            task.node.driver_internal_info['is_whole_disk_image'] = False
 
             task.driver.boot.prepare_instance(task)
-
             get_image_info_mock.assert_called_once_with(
                 task, ipxe_enabled=True)
             cache_mock.assert_called_once_with(task, image_info,
                                                ipxe_enabled=True)
             provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
-            create_pxe_config_mock.assert_called_once_with(
-                task, mock.ANY, CONF.pxe.ipxe_config_template,
-                ipxe_enabled=True)
             switch_pxe_config_mock.assert_called_once_with(
-                pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'bios', False, False, False, False, ipxe_enabled=True,
-                anaconda_boot=False)
-            self.assertFalse(set_boot_device_mock.called)
+                pxe_config_path, None,
+                'bios', False, iscsi_boot=False, ramdisk_boot=True,
+                ipxe_enabled=True, anaconda_boot=False)
+            set_boot_device_mock.assert_called_once_with(task,
+                                                         boot_devices.PXE,
+                                                         persistent=True)
+            expected_params = {
+                'aki_path': 'http://myserver/' + task.node.uuid + '/kernel',
+                'ari_path': 'http://myserver/' + task.node.uuid + '/ramdisk',
+                'pxe_append_params': 'meow ipa-debug=1 ipa-global-request-id'
+                                     '=' + task.context.request_id,
+                'tftp_server': mock.ANY,
+                'ipxe_timeout': 0}
+            mock_create_pxe_config.assert_called_once_with(
+                task, expected_params, mock.ANY, ipxe_enabled=True)
 
+    @mock.patch.object(pxe_utils, 'create_pxe_config', autospec=True)
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
     @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
     @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
     @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_netboot_missing_root_uuid(
+    def test_prepare_instance_ramdisk_bios(
             self, get_image_info_mock, cache_mock,
             dhcp_factory_mock, switch_pxe_config_mock,
-            set_boot_device_mock):
+            set_boot_device_mock, mock_create_pxe_config):
         provider_mock = mock.MagicMock()
         dhcp_factory_mock.return_value = provider_mock
         image_info = {'kernel': ('', '/path/to/kernel'),
                       'ramdisk': ('', '/path/to/ramdisk')}
+        kernel_arg = "meow"
         get_image_info_mock.return_value = image_info
-        instance_info = {"boot_option": "netboot"}
+
         with task_manager.acquire(self.context, self.node.uuid) as task:
+            i_info = task.node.instance_info
+            i_info['kernel_append_params'] = kernel_arg
+            task.node.instance_info = i_info
+            task.node.deploy_interface = 'ramdisk'
+            task.node.save()
             dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=True, ip_version=4)
+                task, ipxe_enabled=True)
             dhcp_opts += pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=True, ip_version=6)
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.instance_info['capabilities'] = instance_info
-            task.node.driver_internal_info['is_whole_disk_image'] = False
+            pxe_config_path = pxe_utils.get_pxe_config_file_path(
+                task.node.uuid, ipxe_enabled=True)
 
             task.driver.boot.prepare_instance(task)
-
             get_image_info_mock.assert_called_once_with(
                 task, ipxe_enabled=True)
             cache_mock.assert_called_once_with(task, image_info,
                                                ipxe_enabled=True)
             provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
-            self.assertFalse(switch_pxe_config_mock.called)
-            self.assertFalse(set_boot_device_mock.called)
+            switch_pxe_config_mock.assert_called_once_with(
+                pxe_config_path, None,
+                'uefi', False, iscsi_boot=False, ramdisk_boot=True,
+                ipxe_enabled=True, anaconda_boot=False)
+            set_boot_device_mock.assert_called_once_with(task,
+                                                         boot_devices.PXE,
+                                                         persistent=True)
+            expected_params = {
+                'aki_path': 'http://myserver/' + task.node.uuid + '/kernel',
+                'ari_path': 'http://myserver/' + task.node.uuid + '/ramdisk',
+                'pxe_append_params': 'meow ipa-debug=1 ipa-global-request-id'
+                                     '=' + task.context.request_id,
+                'tftp_server': mock.ANY,
+                'ipxe_timeout': 0}
+            mock_create_pxe_config.assert_called_once_with(
+                task, expected_params, mock.ANY, ipxe_enabled=True)
 
-    # NOTE(TheJulia): The log mock below is attached to the iPXE interface
-    # which directly logs the warning that is being checked for.
-    @mock.patch.object(pxe_base.LOG, 'warning', autospec=True)
-    @mock.patch.object(pxe_utils, 'clean_up_pxe_config', autospec=True)
+    @mock.patch('os.path.isfile', lambda filename: False)
+    @mock.patch.object(pxe_utils, 'create_pxe_config', autospec=True)
+    @mock.patch.object(deploy_utils, 'is_iscsi_boot', lambda task: True)
+    @mock.patch.object(noop_storage.NoopStorage, 'should_write_image',
+                       lambda task: False)
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
+    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
     @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
     @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
     @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_whole_disk_image_missing_root_uuid(
+    def test_prepare_instance_netboot_iscsi_bios(
             self, get_image_info_mock, cache_mock,
-            dhcp_factory_mock, set_boot_device_mock,
-            clean_up_pxe_mock, log_mock):
+            dhcp_factory_mock, switch_pxe_config_mock,
+            set_boot_device_mock, create_pxe_config_mock):
+        http_url = 'http://192.1.2.3:1234'
+        self.config(http_url=http_url, group='deploy')
         provider_mock = mock.MagicMock()
         dhcp_factory_mock.return_value = provider_mock
-        get_image_info_mock.return_value = {}
-        instance_info = {"boot_option": "netboot"}
+        vol_id = uuidutils.generate_uuid()
+        obj_utils.create_test_volume_target(
+            self.context, node_id=self.node.id, volume_type='iscsi',
+            boot_index=0, volume_id='1234', uuid=vol_id,
+            properties={'target_lun': 0,
+                        'target_portal': 'fake_host:3260',
+                        'target_iqn': 'fake_iqn',
+                        'auth_username': 'fake_username',
+                        'auth_password': 'fake_password'})
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=True)
+            task.node.driver_internal_info = {
+                'boot_from_volume': vol_id}
+            task.node.properties['capabilities'] = 'boot_mode:bios'
+            task.node.instance_info['capabilities'] = {'boot_mode': 'bios'}
+            dhcp_opts = pxe_utils.dhcp_options_for_instance(task,
+                                                            ipxe_enabled=True)
             dhcp_opts += pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=True, ip_version=6)
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.instance_info['capabilities'] = instance_info
-            task.node.driver_internal_info['is_whole_disk_image'] = True
+            pxe_config_path = pxe_utils.get_pxe_config_file_path(
+                task.node.uuid, ipxe_enabled=True)
+
             task.driver.boot.prepare_instance(task)
-            get_image_info_mock.assert_called_once_with(
-                task, ipxe_enabled=True)
-            cache_mock.assert_called_once_with(task, {}, ipxe_enabled=True)
+            self.assertFalse(get_image_info_mock.called)
+            self.assertFalse(cache_mock.called)
             provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
-            self.assertTrue(log_mock.called)
-            clean_up_pxe_mock.assert_called_once_with(task, ipxe_enabled=True)
-            set_boot_device_mock.assert_called_once_with(
-                task, boot_devices.DISK, persistent=True)
+            create_pxe_config_mock.assert_called_once_with(
+                task, mock.ANY, CONF.pxe.ipxe_config_template,
+                ipxe_enabled=True)
+            switch_pxe_config_mock.assert_called_once_with(
+                pxe_config_path, None, boot_modes.LEGACY_BIOS, False,
+                ipxe_enabled=True, iscsi_boot=True, ramdisk_boot=False,
+                anaconda_boot=False)
+            set_boot_device_mock.assert_called_once_with(task,
+                                                         boot_devices.PXE,
+                                                         persistent=True)
 
     @mock.patch('os.path.isfile', lambda filename: False)
     @mock.patch.object(pxe_utils, 'create_pxe_config', autospec=True)
@@ -768,6 +701,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
                         'target_iqn': 'fake_iqn',
                         'auth_username': 'fake_username',
                         'auth_password': 'fake_password'})
+
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.node.driver_internal_info = {
                 'boot_from_volume': vol_id}
@@ -777,7 +711,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
                 task, ipxe_enabled=True, ip_version=6)
             pxe_config_path = pxe_utils.get_pxe_config_file_path(
                 task.node.uuid, ipxe_enabled=True)
-            task.node.properties['capabilities'] = 'boot_mode:bios'
+
             task.driver.boot.prepare_instance(task)
             self.assertFalse(get_image_info_mock.called)
             self.assertFalse(cache_mock.called)
@@ -786,7 +720,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
                 task, mock.ANY, CONF.pxe.ipxe_config_template,
                 ipxe_enabled=True)
             switch_pxe_config_mock.assert_called_once_with(
-                pxe_config_path, None, boot_modes.LEGACY_BIOS, False,
+                pxe_config_path, None, boot_modes.UEFI, False,
                 ipxe_enabled=True, iscsi_boot=True, ramdisk_boot=False,
                 anaconda_boot=False)
             set_boot_device_mock.assert_called_once_with(task,
@@ -808,8 +742,8 @@ class iPXEBootTestCase(db_base.DbTestCase):
         self.config(http_url=http_url, group='deploy')
         provider_mock = mock.MagicMock()
         dhcp_factory_mock.return_value = provider_mock
-        self.node.instance_info = {'boot_iso': 'http://1.2.3.4:1234/boot.iso',
-                                   'capabilities': {'boot_option': 'ramdisk'}}
+        self.node.deploy_interface = 'ramdisk'
+        self.node.instance_info = {'boot_iso': 'http://1.2.3.4:1234/boot.iso'}
         image_info = {'kernel': ('', '/path/to/kernel'),
                       'deploy_kernel': ('', '/path/to/kernel'),
                       'ramdisk': ('', '/path/to/ramdisk'),
@@ -818,7 +752,6 @@ class iPXEBootTestCase(db_base.DbTestCase):
         self.node.provision_state = states.DEPLOYING
         self.node.save()
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            print(task.node)
             dhcp_opts = pxe_utils.dhcp_options_for_instance(task,
                                                             ipxe_enabled=True)
             dhcp_opts += pxe_utils.dhcp_options_for_instance(
@@ -833,7 +766,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
                 task, mock.ANY, CONF.pxe.ipxe_config_template,
                 ipxe_enabled=True)
             switch_pxe_config_mock.assert_called_once_with(
-                pxe_config_path, None, boot_modes.LEGACY_BIOS, False,
+                pxe_config_path, None, boot_modes.UEFI, False,
                 ipxe_enabled=True, iscsi_boot=False, ramdisk_boot=True,
                 anaconda_boot=False)
             set_boot_device_mock.assert_called_once_with(task,
@@ -847,7 +780,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
     @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
     @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
     @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_netboot_ramdisk_with_kernel_arg(
+    def test_prepare_instance_ramdisk_with_kernel_arg(
             self, get_image_info_mock, cache_mock,
             dhcp_factory_mock, switch_pxe_config_mock,
             set_boot_device_mock, create_pxe_config_mock):
@@ -891,7 +824,7 @@ class iPXEBootTestCase(db_base.DbTestCase):
                 task, expected_params, CONF.pxe.ipxe_config_template,
                 ipxe_enabled=True)
             switch_pxe_config_mock.assert_called_once_with(
-                pxe_config_path, None, boot_modes.LEGACY_BIOS, False,
+                pxe_config_path, None, boot_modes.UEFI, False,
                 ipxe_enabled=True, iscsi_boot=False, ramdisk_boot=True,
                 anaconda_boot=False)
             set_boot_device_mock.assert_called_once_with(task,
@@ -902,14 +835,9 @@ class iPXEBootTestCase(db_base.DbTestCase):
                        autospec=True)
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(pxe_utils, 'clean_up_pxe_config', autospec=True)
-    def test_prepare_instance_localboot(self, clean_up_pxe_config_mock,
-                                        set_boot_device_mock,
-                                        secure_boot_mock):
+    def test_prepare_instance(self, clean_up_pxe_config_mock,
+                              set_boot_device_mock, secure_boot_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            instance_info = task.node.instance_info
-            instance_info['capabilities'] = {'boot_option': 'local'}
-            task.node.instance_info = instance_info
-            task.node.save()
             task.driver.boot.prepare_instance(task)
             clean_up_pxe_config_mock.assert_called_once_with(
                 task, ipxe_enabled=True)
@@ -920,15 +848,11 @@ class iPXEBootTestCase(db_base.DbTestCase):
 
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(pxe_utils, 'clean_up_pxe_config', autospec=True)
-    def test_prepare_instance_localboot_active(self, clean_up_pxe_config_mock,
-                                               set_boot_device_mock):
+    def test_prepare_instance_active(self, clean_up_pxe_config_mock,
+                                     set_boot_device_mock):
         self.node.provision_state = states.ACTIVE
         self.node.save()
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            instance_info = task.node.instance_info
-            instance_info['capabilities'] = {'boot_option': 'local'}
-            task.node.instance_info = instance_info
-            task.node.save()
             task.driver.boot.prepare_instance(task)
             clean_up_pxe_config_mock.assert_called_once_with(
                 task, ipxe_enabled=True)
@@ -940,14 +864,12 @@ class iPXEBootTestCase(db_base.DbTestCase):
     @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
     @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
     @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_localboot_with_fallback(
+    def test_prepare_instance_with_fallback(
             self, get_image_info_mock, cache_mock,
             dhcp_factory_mock, switch_pxe_config_mock,
             clean_up_pxe_config_mock, set_boot_device_mock):
         self.config(enable_netboot_fallback=True, group='pxe')
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.instance_info = task.node.instance_info
-            task.node.instance_info['capabilities'] = {'boot_option': 'local'}
             task.node.driver_internal_info['root_uuid_or_disk_id'] = (
                 "30212642-09d3-467f-8e09-21685826ab50")
             task.node.driver_internal_info['is_whole_disk_image'] = False
@@ -961,8 +883,8 @@ class iPXEBootTestCase(db_base.DbTestCase):
                                                          persistent=True)
             switch_pxe_config_mock.assert_called_once_with(
                 pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'bios', True, False, False, False, ipxe_enabled=True,
-                anaconda_boot=False)
+                'uefi', True, iscsi_boot=False, ramdisk_boot=False,
+                ipxe_enabled=True, anaconda_boot=False)
             # No clean up
             self.assertFalse(clean_up_pxe_config_mock.called)
             # No netboot configuration beyond the PXE files
