@@ -77,7 +77,8 @@ class PXEBootTestCase(db_base.DbTestCase):
 
         self.config(enabled_boot_interfaces=[self.boot_interface,
                                              'ipxe', 'fake'])
-        self.config(enabled_deploy_interfaces=['fake', 'direct', 'anaconda'])
+        self.config(enabled_deploy_interfaces=['fake', 'direct', 'anaconda',
+                                               'ramdisk'])
         self.node = obj_utils.create_test_node(
             self.context,
             driver=self.driver,
@@ -144,15 +145,6 @@ class PXEBootTestCase(db_base.DbTestCase):
             del task.node['instance_info']['image_source']
             task.driver.boot.validate(task)
 
-    def test_validate_fail_missing_image_source(self):
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            task.node['instance_info']['capabilities'] = {
-                'boot_option': 'netboot'}
-            del task.node['instance_info']['image_source']
-            self.assertRaises(exception.MissingParameterValue,
-                              task.driver.boot.validate, task)
-
     def test_validate_fail_no_port(self):
         new_node = obj_utils.create_test_node(
             self.context,
@@ -164,63 +156,18 @@ class PXEBootTestCase(db_base.DbTestCase):
             self.assertRaises(exception.MissingParameterValue,
                               task.driver.boot.validate, task)
 
-    def test_validate_fail_trusted_boot_with_secure_boot(self):
-        instance_info = {"boot_option": "netboot",
-                         "secure_boot": "true",
-                         "trusted_boot": "true"}
-        properties = {'capabilities': 'trusted_boot:true'}
+    @mock.patch.object(deploy_utils, 'get_boot_option',
+                       return_value='ramdisk', autospec=True)
+    @mock.patch.object(deploy_utils, 'validate_image_properties',
+                       autospec=True)
+    @mock.patch.object(deploy_utils, 'get_image_instance_info', autospec=True)
+    def test_validate_non_local(self, mock_get_iinfo, mock_validate,
+                                mock_boot_opt):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
-            task.node.instance_info['capabilities'] = instance_info
-            task.node.properties = properties
-            task.node.driver_internal_info['is_whole_disk_image'] = False
-            self.assertRaises(exception.InvalidParameterValue,
-                              task.driver.boot.validate, task)
-
-    def test_validate_fail_invalid_trusted_boot_value(self):
-        properties = {'capabilities': 'trusted_boot:value'}
-        instance_info = {"trusted_boot": "value"}
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            task.node.properties = properties
-            task.node.instance_info['capabilities'] = instance_info
-            self.assertRaises(exception.InvalidParameterValue,
-                              task.driver.boot.validate, task)
-
-    @mock.patch.object(image_service.GlanceImageService, 'show', autospec=True)
-    def test_validate_fail_no_image_kernel_ramdisk_props(self, mock_glance):
-        instance_info = {"boot_option": "netboot"}
-        mock_glance.return_value = {'properties': {}}
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            task.node.instance_info['capabilities'] = instance_info
-            self.assertRaises(exception.MissingParameterValue,
-                              task.driver.boot.validate,
-                              task)
-
-    @mock.patch.object(image_service.GlanceImageService, 'show', autospec=True)
-    def test_validate_fail_glance_image_doesnt_exists(self, mock_glance):
-        mock_glance.side_effect = exception.ImageNotFound('not found')
-        with task_manager.acquire(self.context, self.node.uuid,
-                                  shared=True) as task:
-            task.node.instance_info['capabilities'] = {
-                'boot_option': 'netboot'}
-            self.assertRaises(exception.InvalidParameterValue,
-                              task.driver.boot.validate, task)
-
-    @mock.patch.object(image_service.GlanceImageService, 'show', autospec=True)
-    def test_validate_fail_glance_conn_problem(self, mock_glance):
-        exceptions = (exception.GlanceConnectionFailed('connection fail'),
-                      exception.ImageNotAuthorized('not authorized'),
-                      exception.Invalid('invalid'))
-        mock_glance.side_effect = exceptions
-        for exc in exceptions:
-            with task_manager.acquire(self.context, self.node.uuid,
-                                      shared=True) as task:
-                task.node.instance_info['capabilities'] = {
-                    'boot_option': 'netboot'}
-                self.assertRaises(exception.InvalidParameterValue,
-                                  task.driver.boot.validate, task)
+            task.driver.boot.validate(task)
+            mock_validate.assert_called_once_with(
+                task, mock_get_iinfo.return_value)
 
     def test_validate_inspection(self):
         with task_manager.acquire(self.context, self.node.uuid) as task:
@@ -509,217 +456,13 @@ class PXEBootTestCase(db_base.DbTestCase):
         self.node.save()
         self._test_clean_up_ramdisk(mode='rescue')
 
-    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
-    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
-    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
-    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
-    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_netboot_bios(
-            self, get_image_info_mock, cache_mock,
-            dhcp_factory_mock, switch_pxe_config_mock,
-            set_boot_device_mock):
-        provider_mock = mock.MagicMock()
-        dhcp_factory_mock.return_value = provider_mock
-        image_info = {'kernel': ('', '/path/to/kernel'),
-                      'ramdisk': ('', '/path/to/ramdisk')}
-        get_image_info_mock.return_value = image_info
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.driver_internal_info['root_uuid_or_disk_id'] = (
-                "30212642-09d3-467f-8e09-21685826ab50")
-            task.node.driver_internal_info['is_whole_disk_image'] = False
-            task.node.instance_info = {
-                'capabilities': {'boot_option': 'netboot',
-                                 'boot_mode': 'bios'}}
-            dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False, ip_version=4)
-            dhcp_opts += pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False, ip_version=6)
-            pxe_config_path = pxe_utils.get_pxe_config_file_path(
-                task.node.uuid)
-            task.driver.boot.prepare_instance(task)
-
-            get_image_info_mock.assert_called_once_with(
-                task, ipxe_enabled=False)
-            cache_mock.assert_called_once_with(
-                task, image_info, ipxe_enabled=False)
-            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
-            switch_pxe_config_mock.assert_called_once_with(
-                pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'bios', False, False, False, False, ipxe_enabled=False,
-                anaconda_boot=False)
-            set_boot_device_mock.assert_called_once_with(task,
-                                                         boot_devices.PXE,
-                                                         persistent=True)
-
-    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
-    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
-    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
-    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
-    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_netboot_uefi(
-            self, get_image_info_mock, cache_mock,
-            dhcp_factory_mock, switch_pxe_config_mock,
-            set_boot_device_mock):
-        provider_mock = mock.MagicMock()
-        dhcp_factory_mock.return_value = provider_mock
-        image_info = {'kernel': ('', '/path/to/kernel'),
-                      'ramdisk': ('', '/path/to/ramdisk')}
-        get_image_info_mock.return_value = image_info
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.driver_internal_info['root_uuid_or_disk_id'] = (
-                "30212642-09d3-467f-8e09-21685826ab50")
-            task.node.driver_internal_info['is_whole_disk_image'] = False
-            task.node.instance_info = {
-                'capabilities': {'boot_option': 'netboot'}}
-            dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False, ip_version=4)
-            dhcp_opts += pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False, ip_version=6)
-            pxe_config_path = pxe_utils.get_pxe_config_file_path(
-                task.node.uuid)
-            task.driver.boot.prepare_instance(task)
-
-            get_image_info_mock.assert_called_once_with(
-                task, ipxe_enabled=False)
-            cache_mock.assert_called_once_with(
-                task, image_info, ipxe_enabled=False)
-            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
-            switch_pxe_config_mock.assert_called_once_with(
-                pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'uefi', False, False, False, False, ipxe_enabled=False,
-                anaconda_boot=False)
-            set_boot_device_mock.assert_called_once_with(task,
-                                                         boot_devices.PXE,
-                                                         persistent=True)
-
-    @mock.patch('os.path.isfile', return_value=False, autospec=True)
-    @mock.patch.object(pxe_utils, 'create_pxe_config', autospec=True)
-    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
-    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
-    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
-    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
-    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_netboot_active(
-            self, get_image_info_mock, cache_mock,
-            dhcp_factory_mock, switch_pxe_config_mock,
-            set_boot_device_mock, create_pxe_config_mock, isfile_mock):
-        provider_mock = mock.MagicMock()
-        dhcp_factory_mock.return_value = provider_mock
-        image_info = {'kernel': ('', '/path/to/kernel'),
-                      'ramdisk': ('', '/path/to/ramdisk')}
-        instance_info = {"boot_option": "netboot"}
-        get_image_info_mock.return_value = image_info
-        self.node.provision_state = states.ACTIVE
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.driver_internal_info['root_uuid_or_disk_id'] = (
-                "30212642-09d3-467f-8e09-21685826ab50")
-            task.node.driver_internal_info['is_whole_disk_image'] = False
-            task.node.instance_info['capabilities'] = instance_info
-            task.driver.boot.prepare_instance(task)
-            dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False)
-            dhcp_opts += pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False, ip_version=6)
-            pxe_config_path = pxe_utils.get_pxe_config_file_path(
-                task.node.uuid)
-
-            get_image_info_mock.assert_called_once_with(
-                task, ipxe_enabled=False)
-            cache_mock.assert_called_once_with(
-                task, image_info, ipxe_enabled=False)
-            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
-            create_pxe_config_mock.assert_called_once_with(
-                task, mock.ANY, CONF.pxe.pxe_config_template,
-                ipxe_enabled=False)
-            switch_pxe_config_mock.assert_called_once_with(
-                pxe_config_path, "30212642-09d3-467f-8e09-21685826ab50",
-                'bios', False, False, False, False, ipxe_enabled=False,
-                anaconda_boot=False)
-            self.assertFalse(set_boot_device_mock.called)
-
-    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
-    @mock.patch.object(deploy_utils, 'switch_pxe_config', autospec=True)
-    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
-    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
-    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_netboot_missing_root_uuid(
-            self, get_image_info_mock, cache_mock,
-            dhcp_factory_mock, switch_pxe_config_mock,
-            set_boot_device_mock):
-        provider_mock = mock.MagicMock()
-        dhcp_factory_mock.return_value = provider_mock
-        image_info = {'kernel': ('', '/path/to/kernel'),
-                      'ramdisk': ('', '/path/to/ramdisk')}
-        instance_info = {"boot_option": "netboot"}
-        get_image_info_mock.return_value = image_info
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.instance_info['capabilities'] = instance_info
-            task.node.driver_internal_info['is_whole_disk_image'] = False
-            dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False)
-            dhcp_opts += pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False, ip_version=6)
-
-            task.driver.boot.prepare_instance(task)
-
-            get_image_info_mock.assert_called_once_with(task,
-                                                        ipxe_enabled=False)
-            cache_mock.assert_called_once_with(
-                task, image_info, ipxe_enabled=False)
-            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
-            self.assertFalse(switch_pxe_config_mock.called)
-            self.assertFalse(set_boot_device_mock.called)
-
-    @mock.patch.object(pxe_base.LOG, 'warning', autospec=True)
-    @mock.patch.object(pxe_utils, 'clean_up_pxe_config', autospec=True)
-    @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
-    @mock.patch.object(dhcp_factory, 'DHCPFactory', autospec=True)
-    @mock.patch.object(pxe_utils, 'cache_ramdisk_kernel', autospec=True)
-    @mock.patch.object(pxe_utils, 'get_instance_image_info', autospec=True)
-    def test_prepare_instance_whole_disk_image_missing_root_uuid(
-            self, get_image_info_mock, cache_mock,
-            dhcp_factory_mock, set_boot_device_mock,
-            clean_up_pxe_mock, log_mock):
-        provider_mock = mock.MagicMock()
-        dhcp_factory_mock.return_value = provider_mock
-        get_image_info_mock.return_value = {}
-        instance_info = {"boot_option": "netboot"}
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.properties['capabilities'] = 'boot_mode:bios'
-            task.node.instance_info['capabilities'] = instance_info
-            task.node.driver_internal_info['is_whole_disk_image'] = True
-            dhcp_opts = pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False)
-            dhcp_opts += pxe_utils.dhcp_options_for_instance(
-                task, ipxe_enabled=False, ip_version=6)
-            task.driver.boot.prepare_instance(task)
-            get_image_info_mock.assert_called_once_with(task,
-                                                        ipxe_enabled=False)
-            cache_mock.assert_called_once_with(
-                task, {}, ipxe_enabled=False)
-            provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
-            self.assertTrue(log_mock.called)
-            clean_up_pxe_mock.assert_called_once_with(
-                task, ipxe_enabled=False)
-            set_boot_device_mock.assert_called_once_with(
-                task, boot_devices.DISK, persistent=True)
-
     @mock.patch.object(boot_mode_utils, 'configure_secure_boot_if_needed',
                        autospec=True)
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(pxe_utils, 'clean_up_pxe_config', autospec=True)
-    def test_prepare_instance_localboot(self, clean_up_pxe_config_mock,
-                                        set_boot_device_mock,
-                                        secure_boot_mock):
+    def test_prepare_instance(self, clean_up_pxe_config_mock,
+                              set_boot_device_mock, secure_boot_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            instance_info = task.node.instance_info
-            instance_info['capabilities'] = {'boot_option': 'local'}
-            task.node.instance_info = instance_info
-            task.node.save()
             task.driver.boot.prepare_instance(task)
             clean_up_pxe_config_mock.assert_called_once_with(
                 task, ipxe_enabled=False)
@@ -730,15 +473,11 @@ class PXEBootTestCase(db_base.DbTestCase):
 
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(pxe_utils, 'clean_up_pxe_config', autospec=True)
-    def test_prepare_instance_localboot_active(self, clean_up_pxe_config_mock,
-                                               set_boot_device_mock):
+    def test_prepare_instance_active(self, clean_up_pxe_config_mock,
+                                     set_boot_device_mock):
         self.node.provision_state = states.ACTIVE
         self.node.save()
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            instance_info = task.node.instance_info
-            instance_info['capabilities'] = {'boot_option': 'local'}
-            task.node.instance_info = instance_info
-            task.node.save()
             task.driver.boot.prepare_instance(task)
             clean_up_pxe_config_mock.assert_called_once_with(
                 task, ipxe_enabled=False)
@@ -764,9 +503,7 @@ class PXEBootTestCase(db_base.DbTestCase):
         self.node.provision_state = states.DEPLOYING
         get_image_info_mock.return_value = image_info
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            instance_info = task.node.instance_info
-            instance_info['capabilities'] = {'boot_option': 'ramdisk'}
-            task.node.instance_info = instance_info
+            task.node.deploy_interface = 'ramdisk'
             task.node.save()
             dhcp_opts = pxe_utils.dhcp_options_for_instance(
                 task, ipxe_enabled=False)
